@@ -1,6 +1,5 @@
 /*=============================================================================
 XMOTO
-Copyright (C) 2005-2006 Rasmus Neckelmann (neckelmann@gmail.com)
 
 This file is part of XMOTO.
 
@@ -27,6 +26,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "xmscene/Block.h"
 #include "xmscene/Zone.h"
 #include "xmscene/Entity.h"
+#include "xmscene/PhysicsSettings.h"
 
 /* 
    Prior to version 0.1.11, the far largest time sink in the game was the 
@@ -42,18 +42,25 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
     * Static level geometry is stored in grid-structure for faster access. 
 */
 
-namespace vapp {
-
   #define CD_MIN(a,b)             ((a)>(b)?(b):(a))
   #define CD_MAX(a,b)             ((a)<(b)?(b):(a))
   
   #define CD_EPSILON              0.01f
 
-  #define EMPTY_AND_CLEAR_VECTOR(v) for(int i=0;i<(v).size();i++) {\
+  #define EMPTY_AND_CLEAR_VECTOR(v) for(unsigned int i=0;i<(v).size();i++) {\
                                       delete (v)[i]; \
                                     } \
                                     (v).clear()
 
+CollisionSystem::CollisionSystem() {
+  m_pGrid = NULL;
+  m_bDebugFlag=false;
+}
+
+CollisionSystem::~CollisionSystem() {
+  reset();
+}
+
   /*===========================================================================
   Reset collision system
   ===========================================================================*/
@@ -74,7 +81,7 @@ namespace vapp {
     m_staticBlocksHandler.reset();
     m_staticBlocksHandlerSecondLayer.reset();
 
-    for(int i=0; i<m_layerBlocksHandlers.size(); i++){
+    for(unsigned int i=0; i<m_layerBlocksHandlers.size(); i++){
       delete m_layerBlocksHandlers[i];
     }
     m_layerBlocksHandlers.clear();
@@ -85,7 +92,7 @@ namespace vapp {
   ===========================================================================*/
   void CollisionSystem::setDims(float fMinX,float fMinY,
 				float fMaxX,float fMaxY,
-				int numberBackgroundLayers,
+				unsigned int numberBackgroundLayers,
 				std::vector<Vector2f>& layerOffsets) {
     /* Find suitable grid properties - first horizontal */
     /* The choice of the number of cell in the grid is quite
@@ -93,11 +100,11 @@ namespace vapp {
        As a consequence, I DECIDE to fix the size of the cells to 3x3. */
 
     /* Horizontal */
-    m_nGridWidth = ceil((fMaxX - fMinX)/3.0);
+    m_nGridWidth = (int)(ceil((fMaxX - fMinX)/3.0));
     m_fCellWidth = (fMaxX - fMinX)/m_nGridWidth;
 
     /* Then vertical */
-    m_nGridHeight = ceil((fMaxY - fMinY)/3.0);
+    m_nGridHeight = (int)ceil((fMaxY - fMinY)/3.0);
     m_fCellHeight = (fMaxY - fMinY)/m_nGridHeight;
 
     /* Set bounding box */
@@ -128,7 +135,7 @@ namespace vapp {
 					     Vector2f(m_fMaxX, m_fMaxY),
 					     m_nGridWidth, m_nGridHeight);
 
-    for(int i=0; i<numberBackgroundLayers; i++){
+    for(unsigned int i=0; i<numberBackgroundLayers; i++){
       ElementHandler<Block>* pHandler = new ElementHandler<Block>();
       pHandler->setDims(Vector2f(m_fMinX, m_fMinY),
 			Vector2f(m_fMaxX, m_fMaxY),
@@ -248,10 +255,14 @@ namespace vapp {
     int nMaxCX = (int)floor(((fMaxX - m_fMinX + CD_EPSILON) * (float)m_nGridWidth) / (m_fMaxX - m_fMinX));
     int nMaxCY = (int)floor(((fMaxY - m_fMinY + CD_EPSILON) * (float)m_nGridHeight) / (m_fMaxY - m_fMinY));
     
-    if(nMinCX < 0) nMinCX = 0;
-    if(nMinCY < 0) nMinCY = 0;
-    if(nMaxCX > m_nGridWidth-1) nMaxCX = m_nGridWidth-1;
-    if(nMaxCY > m_nGridHeight-1) nMaxCY = m_nGridHeight-1;        
+    if(nMinCX < 0)
+      nMinCX = 0;
+    if(nMinCY < 0)
+      nMinCY = 0;
+    if(nMaxCX > m_nGridWidth-1)
+      nMaxCX = m_nGridWidth-1;
+    if(nMaxCY > m_nGridHeight-1)
+      nMaxCY = m_nGridHeight-1;
     
     if(m_bDebugFlag) {
       m_CheckedLines.clear();
@@ -263,14 +274,14 @@ namespace vapp {
     AABB BBox;
     BBox.addPointToAABB2f(fMinX, fMinY);
     BBox.addPointToAABB2f(fMaxX, fMaxY);
-    std::vector<Block*> blocks = getDynBlocksNearPosition(BBox);
+    std::vector<Block*>& blocks = getDynBlocksNearPosition(BBox);
 
-    for(int i=0; i<blocks.size(); i++){
+    for(unsigned int i=0; i<blocks.size(); i++){
       Block* pBlock = blocks[i];
       if(pBlock->isBackground() == true)
 	continue;
       std::vector<Line*>& blockLines = pBlock->getCollisionLines();
-      for(int j=0; j<blockLines.size(); j++){
+      for(unsigned int j=0; j<blockLines.size(); j++){
 	if(_CheckCircleAndLine(blockLines[j], x, y, r)){
 	  return true;
 	}
@@ -297,7 +308,7 @@ namespace vapp {
         /* TODO: currently we will probably check the same lines several times each... AVOID THAT! */
         
         /* Check all lines in cell */
-        for(int j=0;j<m_pGrid[i].Lines.size();j++) {  
+        for(unsigned int j=0; j<m_pGrid[i].Lines.size(); j++) {  
           if(m_bDebugFlag)
             m_CheckedLines.push_back(m_pGrid[i].Lines[j]);
             
@@ -338,18 +349,11 @@ namespace vapp {
     /* None of the touched cells contain any geometry */
     return false;
   }
-      
-  /*===========================================================================
-  Boolean check of collision between circle-path and system
-  ===========================================================================*/
-  bool CollisionSystem::checkCirclePath(float x1,float y1,float x2,float y2,float r) {
-    return false;
-  }
-      
+
   /*===========================================================================
   Calculate collision between line and system
   ===========================================================================*/
-  int CollisionSystem::collideLine(float x1,float y1,float x2,float y2,dContact *pContacts,int nMaxC) {
+int CollisionSystem::collideLine(float x1,float y1,float x2,float y2,dContact *pContacts,int nMaxC, PhysicsSettings* i_physicsSettings) {
     int nNumC = 0;
 
     /* Calculate bounding box of line */
@@ -375,20 +379,18 @@ namespace vapp {
     AABB BBox;
     BBox.addPointToAABB2f(fMinX, fMinY);
     BBox.addPointToAABB2f(fMaxX, fMaxY);
-    std::vector<Block*> blocks = getDynBlocksNearPosition(BBox);
+    std::vector<Block*>& blocks = getDynBlocksNearPosition(BBox);
 
-    for(int i=0; i<blocks.size(); i++){
+    for(unsigned int i=0; i<blocks.size(); i++){
       Block* pBlock = blocks[i];
       if(pBlock->isBackground() == true)
 	continue;
       std::vector<Line*>& blockLines = pBlock->getCollisionLines();
-      for(int j=0; j<blockLines.size(); j++){
+      for(unsigned int j=0; j<blockLines.size(); j++){
 
 	/* Is the beginning "behind" the line? */
 	float vx = blockLines[j]->x2 - blockLines[j]->x1;
 	float vy = blockLines[j]->y2 - blockLines[j]->y1;
-	float enx = -vy;
-	float eny = vx;
 
 	/* Too small? */
 	if(fabs(vx) < 0.0001f && fabs(vy) < 0.0001f) {
@@ -405,7 +407,7 @@ namespace vapp {
 	  Vector2f W = Vector2f(vx,vy);
 	  W.normalize();
 
-	  _SetWheelContactParams(&c,T,W,0.0f, blockLines[j]->fGrip);                                           
+	  _SetWheelContactParams(&c,T,W,0.0f, blockLines[j]->fGrip, i_physicsSettings);
 	  int nOldC = nNumC;
 	  nNumC = _AddContactToList(pContacts,nNumC,&c,nMaxC);                         
 	  if(nNumC != nOldC)
@@ -421,12 +423,13 @@ namespace vapp {
         int i = cx + cy*m_nGridWidth;
             
         /* Empty? That would be nice */
-        if(m_pGrid[i].Lines.empty()) continue;
+        if(m_pGrid[i].Lines.empty())
+	  continue;
         
         /* TODO: currently we will probably check the same lines several times each... AVOID THAT! */
         
         /* Check all lines in cell */
-        for(int j=0;j<m_pGrid[i].Lines.size();j++) {
+        for(unsigned int j=0;j<m_pGrid[i].Lines.size();j++) {
           /* Is the beginning "behind" the line? */
 	  // nicolas : it seems not work
           float vx = m_pGrid[i].Lines[j]->x2 - m_pGrid[i].Lines[j]->x1;
@@ -453,7 +456,7 @@ namespace vapp {
             Vector2f W = Vector2f(vx,vy);
             W.normalize();
 
-	    _SetWheelContactParams(&c,T,W, 0.0f, m_pGrid[i].Lines[j]->fGrip);
+	    _SetWheelContactParams(&c,T,W, 0.0f, m_pGrid[i].Lines[j]->fGrip, i_physicsSettings);
 	    nNumC = _AddContactToList(pContacts,nNumC,&c,nMaxC);                 
           }                                      
         }          
@@ -467,7 +470,8 @@ namespace vapp {
   /*===========================================================================
   Calculate precise intersections between circle and geometry, if any
   ===========================================================================*/
-  int CollisionSystem::_CollideCircleAndLine(Line *pLine,float x,float y,float r,dContact *pContacts,int nOldNumC,int nMaxC, float fGrip) {
+  int CollisionSystem::_CollideCircleAndLine(Line *pLine,float x,float y,float r,dContact *pContacts,int nOldNumC,int nMaxC,
+					     float fGrip, PhysicsSettings* i_physicsSettings) {
     int nNumC = nOldNumC;
 
     /* First do a bounding box collision check */
@@ -507,7 +511,7 @@ namespace vapp {
 //      float fDepth = _CalculateCircleLineDepth(Vector2f(x,y),r,Vector2f(pLine->x1,pLine->y1),Vector2f(pLine->x2,pLine->y2));
       double fDepth = _CalculateDepth(Vector2f(x,y),r,Vector2f(pLine->x1,pLine->y1));
       _SetWheelContactParams(&c,Vector2f(pLine->x1,pLine->y1),
-                              W,fDepth, fGrip);                                   
+			     W,fDepth, fGrip, i_physicsSettings);
       nNumC = _AddContactToList(pContacts,nNumC,&c,nMaxC);            
       //return nNumC;
     }
@@ -527,7 +531,7 @@ namespace vapp {
 //      float fDepth = _CalculateCircleLineDepth(Vector2f(x,y),r,Vector2f(pLine->x1,pLine->y1),Vector2f(pLine->x2,pLine->y2));
       double fDepth = _CalculateDepth(Vector2f(x,y),r,Vector2f(pLine->x2,pLine->y2));
       _SetWheelContactParams(&c,Vector2f(pLine->x2,pLine->y2),
-                              W,fDepth, fGrip);                                   
+			     W,fDepth, fGrip, i_physicsSettings);
       nNumC = _AddContactToList(pContacts,nNumC,&c,nMaxC);            
       //return nNumC;
     }
@@ -550,11 +554,11 @@ namespace vapp {
       //if((xxx % 200) == 0) 
       //  printf("%f\n",fDepth);
         
-      _SetWheelContactParams(&c,T1,W,fDepth, fGrip); 
+      _SetWheelContactParams(&c,T1,W,fDepth, fGrip, i_physicsSettings); 
       nNumC = _AddContactToList(pContacts,nNumC,&c,nMaxC);
         
       if(n>1) {
-        _SetWheelContactParams(&c,T2,W,fDepth, fGrip); 
+        _SetWheelContactParams(&c,T2,W,fDepth, fGrip, i_physicsSettings);
 //        _SetWheelContactParams(&c,T2,W,_CalculateDepth(Vector2f(x,y),r,T2));                                   
         nNumC = _AddContactToList(pContacts,nNumC,&c,nMaxC);
       }
@@ -563,7 +567,7 @@ namespace vapp {
     return nNumC;
   }
   
-  int CollisionSystem::collideCircle(float x,float y,float r,dContact *pContacts,int nMaxC) {
+  int CollisionSystem::collideCircle(float x,float y,float r,dContact *pContacts,int nMaxC, PhysicsSettings* i_physicsSettings) {
     int nNumC = 0;
   
     /* Calculate bounding box of circle */
@@ -594,18 +598,18 @@ namespace vapp {
     AABB BBox;
     BBox.addPointToAABB2f(fMinX, fMinY);
     BBox.addPointToAABB2f(fMaxX, fMaxY);
-    std::vector<Block*> blocks = getDynBlocksNearPosition(BBox);
+    std::vector<Block*>& blocks = getDynBlocksNearPosition(BBox);
 
-    for(int i=0; i<blocks.size(); i++){
+    for(unsigned int i=0; i<blocks.size(); i++){
       Block* pBlock = blocks[i];
       if(pBlock->isBackground() == true)
 	continue;
       std::vector<Line*>& blockLines = pBlock->getCollisionLines();
-      for(int j=0; j<blockLines.size(); j++){
+      for(unsigned int j=0; j<blockLines.size(); j++){
 	int nOldC = nNumC;
 	nNumC = _CollideCircleAndLine(blockLines[j], x, y, r,
 				      pContacts,nNumC,nMaxC,
-				      blockLines[j]->fGrip);
+				      blockLines[j]->fGrip, i_physicsSettings);
 
 	if(nOldC != nNumC)
 	  m_bDynamicTouched = true;
@@ -613,8 +617,8 @@ namespace vapp {
     }
 
     /* For each cell we might have touched something in... */
-    for(int cx=nMinCX;cx<=nMaxCX;cx++) {
-      for(int cy=nMinCY;cy<=nMaxCY;cy++) {
+    for(int cx=nMinCX; cx<=nMaxCX; cx++) {
+      for(int cy=nMinCY; cy<=nMaxCY; cy++) {
         int i = cx + cy*m_nGridWidth;
 
         if(m_bDebugFlag) {
@@ -627,14 +631,15 @@ namespace vapp {
         }
             
         /* Empty? That would be nice */
-        if(m_pGrid[i].Lines.empty()) continue;
+        if(m_pGrid[i].Lines.empty())
+	  continue;
         
         /* TODO: currently we will probably check the same lines several times each... AVOID THAT! */
         
         /* Check all lines in cell */
-        for(int j=0;j<m_pGrid[i].Lines.size();j++) {
+        for(unsigned int j=0; j<m_pGrid[i].Lines.size(); j++) {
           nNumC = _CollideCircleAndLine(m_pGrid[i].Lines[j],x,y,r,pContacts,nNumC,nMaxC,
-					m_pGrid[i].Lines[j]->fGrip);
+					m_pGrid[i].Lines[j]->fGrip, i_physicsSettings);
         }          
       }
     }
@@ -661,8 +666,9 @@ namespace vapp {
     p->fCellHeight = m_fCellHeight;
     
     int nEmpty = 0;
-    for(int i=0;i<m_nGridWidth*m_nGridHeight;i++) {
-      if(m_pGrid[i].Lines.empty()) nEmpty++;
+    for(int i=0; i<m_nGridWidth*m_nGridHeight; i++) {
+      if(m_pGrid[i].Lines.empty())
+	nEmpty++;
     }
     
     p->fPercentageOfEmptyCells = (100.0f * (float)nEmpty) / (float)(m_nGridWidth*m_nGridHeight);
@@ -671,7 +677,8 @@ namespace vapp {
   /*===========================================================================
   Helpers 
   ===========================================================================*/
-  void CollisionSystem::_SetWheelContactParams(dContact *pc,const Vector2f &Pos,const Vector2f &NormalT,double fDepth, float fGrip) {
+void CollisionSystem::_SetWheelContactParams(dContact *pc,const Vector2f &Pos,const Vector2f &NormalT,
+					     double fDepth, float fGrip, PhysicsSettings* i_physicsSettings) {
     memset(pc,0,sizeof(dContact));
     Vector2f Normal = NormalT;
     Normal.normalize();
@@ -687,8 +694,8 @@ namespace vapp {
     pc->surface.mu = fGrip;
     //pc->surface.mu = 0.9; //.7; //8;
     //pc->surface.bounce = 0.3;
-    pc->surface.soft_erp = PHYS_WHEEL_ERP;     
-    pc->surface.soft_cfm = PHYS_WHEEL_CFM; 
+    pc->surface.soft_erp = i_physicsSettings->BikeWheelErp();
+    pc->surface.soft_cfm = i_physicsSettings->BikeWheelCfm();
     pc->surface.mode = dContactApprox1_1 | dContactSlip1;
   }  
 
@@ -714,17 +721,17 @@ namespace vapp {
     return 0.0f;
   }
   
-  int CollisionSystem::_AddContactToList(dContact *pContacts,int nNumContacts,dContact *pc,int nMaxContacts) {
-    int i;    
-    if(nNumContacts == nMaxContacts) return nNumContacts;
+  int CollisionSystem::_AddContactToList(dContact *pContacts, int nNumContacts,dContact *pc,int nMaxContacts) {
+    if(nNumContacts == nMaxContacts)
+      return nNumContacts;
     
-    for(int i=0;i<nNumContacts;i++) {
+    for(int i=0; i<nNumContacts; i++) {
       if(fabs(pContacts[i].geom.pos[0] - pc->geom.pos[0]) < 0.1f &&
-         fabs(pContacts[i].geom.pos[1] - pc->geom.pos[1]) < 0.1f) return nNumContacts;
+         fabs(pContacts[i].geom.pos[1] - pc->geom.pos[1]) < 0.1f)
+	return nNumContacts;
     }
-    //printf(" [%f %f]  [%f %f]\n",pc->geom.pos[0],pc->geom.pos[1],
-      //pc->geom.normal[0],pc->geom.normal[1]);
-    memcpy(&pContacts[nNumContacts],pc,sizeof(dContact));
+
+    memcpy(&pContacts[nNumContacts], pc, sizeof(dContact));
     nNumContacts++;
     return nNumContacts;
   }
@@ -747,7 +754,7 @@ namespace vapp {
     m_entitiesHandler.moveElement(id);
   }
 
-  std::vector<Entity*> CollisionSystem::getEntitiesNearPosition(AABB& BBox)
+  std::vector<Entity*>& CollisionSystem::getEntitiesNearPosition(AABB& BBox)
   {
     return m_entitiesHandler.getElementsNearPosition(BBox);
   }
@@ -775,9 +782,9 @@ namespace vapp {
   */
 
   /* dynamic blocks */
-  void CollisionSystem::addDynBlock(Block* id)
+  ColElement<Block>* CollisionSystem::addDynBlock(Block* id)
   {
-    m_dynBlocksHandler.addElement(id);
+    return m_dynBlocksHandler.addElement(id);
   }
 
   void CollisionSystem::removeDynBlock(Block* id)
@@ -787,10 +794,10 @@ namespace vapp {
 
   void CollisionSystem::moveDynBlock(Block* id)
   {
-    m_dynBlocksHandler.moveElement(id);
+    m_dynBlocksHandler.moveElement(id->getColElement());
   }
 
-  std::vector<Block*> CollisionSystem::getDynBlocksNearPosition(AABB& BBox)
+  std::vector<Block*>& CollisionSystem::getDynBlocksNearPosition(AABB& BBox)
   {
     return m_dynBlocksHandler.getElementsNearPosition(BBox);
   }
@@ -806,7 +813,7 @@ namespace vapp {
     }
   }
 
-  std::vector<Block*> CollisionSystem::getStaticBlocksNearPosition(AABB& BBox, int layer)
+  std::vector<Block*>& CollisionSystem::getStaticBlocksNearPosition(AABB& BBox, int layer)
   {
     if(layer == -1){
       return m_staticBlocksHandler.getElementsNearPosition(BBox);
@@ -821,7 +828,7 @@ namespace vapp {
     m_layerBlocksHandlers[layer]->addElement(id);
   }
 
-  std::vector<Block*> CollisionSystem::getBlocksNearPositionInLayer(AABB& BBox, int layer)
+  std::vector<Block*>& CollisionSystem::getBlocksNearPositionInLayer(AABB& BBox, int layer)
   {
     return m_layerBlocksHandlers[layer]->getElementsNearPosition(BBox);
   }
@@ -853,6 +860,8 @@ namespace vapp {
     }
 
     EMPTY_AND_CLEAR_VECTOR(m_ColElements);
+    // reserve 256 entry to avoid mallocs during the game
+    m_returnedElements.reserve(256);
   }
 
   template <class T> void
@@ -875,23 +884,26 @@ namespace vapp {
     m_gridWidth  = gridWidth;
     m_gridHeight = gridHeight;
 
+    m_widthDivisor  = (float)(m_gridWidth)  / (m_max.x - m_min.x);
+    m_heightDivisor = (float)(m_gridHeight) / (m_max.y - m_min.y);
+
     m_pGrid = new GridCell[m_gridWidth * m_gridHeight];    
   }
 
-  template <class T> void
-  ElementHandler<T>::addElement(T* id)
+  template <class T> struct ColElement<T>* ElementHandler<T>::addElement(T* id)
   {
-    ColElement* pNewElem = new ColElement;
+    struct ColElement<T>* pNewElem = new struct ColElement<T>;
     pNewElem->id = id;
     m_ColElements.push_back(pNewElem);
 
     _addColElementInCells(pNewElem);
+    return pNewElem;      
   }
 
   template <class T> void
   ElementHandler<T>::removeElement(T* id)
   {
-    ColElement* pColElem = _getAndRemoveColElement(id);
+    struct ColElement<T>* pColElem = _getAndRemoveColElement(id);
     _removeColElementFromCells(pColElem);
 
     delete pColElem;
@@ -900,28 +912,48 @@ namespace vapp {
   template <class T> void
   ElementHandler<T>::moveElement(T* id)
   {
-    ColElement* pColElem = _getColElement(id);
+    struct ColElement<T>* pColElem = _getColElement(id);
     _removeColElementFromCells(pColElem);
     _addColElementInCells(pColElem);
   }
 
-  template <class T> std::vector<T*>
+  template <class T> void
+  ElementHandler<T>::moveElement(struct ColElement<T>* pColElem)
+  {
+    _removeColElementFromCells(pColElem);
+    _addColElementInCells(pColElem);
+  }
+
+
+
+  inline int my_floor(float x) {
+    int ix = (int)x;
+    return (ix >= 0) ? ix : ix-1;
+  }
+
+
+
+  template <class T> std::vector<T*>&
   ElementHandler<T>::getElementsNearPosition(AABB& BBox)
   {
-    std::vector<T*> elements;
+    m_returnedElements.clear();
     Vector2f BMin = BBox.getBMin();
     Vector2f BMax = BBox.getBMax();
 
     /* grid coordonates */
-    int nMinCX = (int)floor(((BMin.x - m_min.x - CD_EPSILON) * (float)m_gridWidth)  / (m_max.x - m_min.x));
-    int nMinCY = (int)floor(((BMin.y - m_min.y - CD_EPSILON) * (float)m_gridHeight) / (m_max.y - m_min.y));
-    int nMaxCX = (int)floor(((BMax.x - m_min.x + CD_EPSILON) * (float)m_gridWidth)  / (m_max.x - m_min.x));
-    int nMaxCY = (int)floor(((BMax.y - m_min.y + CD_EPSILON) * (float)m_gridHeight) / (m_max.y - m_min.y));
+    int nMinCX = my_floor(((BMin.x - m_min.x - CD_EPSILON) * m_widthDivisor));
+    int nMinCY = my_floor(((BMin.y - m_min.y - CD_EPSILON) * m_heightDivisor));
+    int nMaxCX = my_floor(((BMax.x - m_min.x + CD_EPSILON) * m_widthDivisor));
+    int nMaxCY = my_floor(((BMax.y - m_min.y + CD_EPSILON) * m_heightDivisor));
 
-    if(nMinCX < 0) nMinCX = 0;
-    if(nMinCY < 0) nMinCY = 0;
-    if(nMaxCX >= m_gridWidth)  nMaxCX = m_gridWidth;
-    if(nMaxCY >= m_gridHeight) nMaxCY = m_gridHeight;        
+    if(nMinCX < 0)
+      nMinCX = 0;
+    if(nMinCY < 0)
+      nMinCY = 0;
+    if(nMaxCX >= m_gridWidth)
+      nMaxCX = m_gridWidth;
+    if(nMaxCY >= m_gridHeight)
+      nMaxCY = m_gridHeight;        
 
     if(m_bDebugFlag) {
       m_CheckedElements.clear();
@@ -939,13 +971,13 @@ namespace vapp {
 	  continue;
 
 	int cell = i+j*m_gridWidth;
-	std::vector<ColElement*>& gridCellColElements = m_pGrid[cell].ColElements;
-	for(int k=0; k<gridCellColElements.size(); k++){
+	std::vector<struct ColElement<T>*>& gridCellColElements = m_pGrid[cell].ColElements;
+	for(unsigned int k=0; k<gridCellColElements.size(); k++){
 	  if(gridCellColElements[k]->curCheck != m_curCheck){
 	    
 	    gridCellColElements[k]->curCheck = m_curCheck;
 
-	    elements.push_back(gridCellColElements[k]->id);
+	    m_returnedElements.push_back(gridCellColElements[k]->id);
 	    
 	    if(m_bDebugFlag) {
 	      m_CheckedElements.push_back(gridCellColElements[k]->id);
@@ -956,14 +988,27 @@ namespace vapp {
     }
 
     //printf("ElementHandler::getElementsNearPosition end\n");
-    return elements;
+    return m_returnedElements;
   }
 
   /*=====================================================
     Generic element handling helper functions
   =====================================================*/
+
+  template <class T> struct ColElement<T>*
+  ElementHandler<T>::_getColElement(T* id)
+  {
+    for(unsigned int i=0; i<m_ColElements.size(); i++){
+      if(m_ColElements[i]->id == id){
+	return m_ColElements[i];
+      }
+    }
+
+    throw Exception("Collision element not found");
+  }
+
   template <class T> void
-  ElementHandler<T>::_addColElementInCells(ColElement* pColElem)
+  ElementHandler<T>::_addColElementInCells(struct ColElement<T>* pColElem)
   {
     /* current check */
     pColElem->curCheck = 0;
@@ -974,15 +1019,19 @@ namespace vapp {
     Vector2f BMax = BBox.getBMax();
 
     /* grid coordonates */
-    int nMinCX = (int)floor(((BMin.x - m_min.x - CD_EPSILON) * (float)m_gridWidth)  / (m_max.x - m_min.x));
-    int nMinCY = (int)floor(((BMin.y - m_min.y - CD_EPSILON) * (float)m_gridHeight) / (m_max.y - m_min.y));
-    int nMaxCX = (int)floor(((BMax.x - m_min.x + CD_EPSILON) * (float)m_gridWidth)  / (m_max.x - m_min.x));
-    int nMaxCY = (int)floor(((BMax.y - m_min.y + CD_EPSILON) * (float)m_gridHeight) / (m_max.y - m_min.y));
+    int nMinCX = my_floor(((BMin.x - m_min.x - CD_EPSILON) * m_widthDivisor));
+    int nMinCY = my_floor(((BMin.y - m_min.y - CD_EPSILON) * m_heightDivisor));
+    int nMaxCX = my_floor(((BMax.x - m_min.x + CD_EPSILON) * m_widthDivisor));
+    int nMaxCY = my_floor(((BMax.y - m_min.y + CD_EPSILON) * m_heightDivisor));
 
-    if(nMinCX < 0) nMinCX = 0;
-    if(nMinCY < 0) nMinCY = 0;
-    if(nMaxCX >= m_gridWidth)  nMaxCX = m_gridWidth;
-    if(nMaxCY >= m_gridHeight) nMaxCY = m_gridHeight;        
+    if(nMinCX < 0)
+      nMinCX = 0;
+    if(nMinCY < 0)
+      nMinCY = 0;
+    if(nMaxCX >= m_gridWidth)
+      nMaxCX = m_gridWidth;
+    if(nMaxCY >= m_gridHeight)
+      nMaxCY = m_gridHeight;        
 
     /* For each cells touched by the element, add it to the grid */
     for(int i=nMinCX; i<=nMaxCX; i++){
@@ -999,14 +1048,28 @@ namespace vapp {
     }
   }
 
+  template <class T> struct ColElement<T>*
+  ElementHandler<T>::_getAndRemoveColElement(T* id)
+  {
+    for(unsigned int i=0; i<m_ColElements.size(); i++){
+      if(m_ColElements[i]->id == id){
+	struct ColElement<T>* pColElem = m_ColElements[i];
+	m_ColElements.erase(m_ColElements.begin()+i);
+
+	return pColElem;
+      }
+    }
+    throw Exception("Collision element not found");
+  }
+
   template <class T> void
-  ElementHandler<T>::_removeColElementFromCells(ColElement* pColElem)
+  ElementHandler<T>::_removeColElementFromCells(struct ColElement<T>* pColElem)
   {
     /* for each grid cell with the ColElem in it */
-    for(int i=0; i<pColElem->gridCells.size(); i++){
+    for(unsigned int i=0; i<pColElem->gridCells.size(); i++){
       int cell = pColElem->gridCells[i];
 
-      for(int j=0; j<m_pGrid[cell].ColElements.size(); j++){
+      for(unsigned int j=0; j<m_pGrid[cell].ColElements.size(); j++){
 	if(m_pGrid[cell].ColElements[j] == pColElem){
 
 	  /* remove ColElem from cell*/
@@ -1020,42 +1083,3 @@ namespace vapp {
     /* remove cell from ColElem*/
     pColElem->gridCells.clear();
   }
-
-  /*==================================================
-    I can't get this two function to compile, so i put
-    them into the .h file...
-    They got problems with returning ColElement*, here
-    is gcc 4.0.3 error:
-    "Collision.cpp:898: erreur: expected constructor, destructor, or type conversion before «*» token"
-
-    If somebody knows how to fix, feel free to send a patch. 
-    ================================================*/
-
-  /*
-  template <class T> ElementHandler<T>::ColElement*
-  ElementHandler<T>::_getAndRemoveColElement(T* id)
-  {
-    for(int i=0; i<m_ColElements.size(); i++){
-      if(m_ColElements[i]->id == id){
-	ColElement* pColElem = m_ColElements[i];
-	m_ColElements.erase(m_ColElements.begin()+i);
-	return pColElem;
-      }
-    }
-    throw Exception("Collision element not found");
-  }
-  */
-  /*
-  template <class T> ElementHandler<T>::ColElement*
-  ElementHandler<T>::_getColElement(T* id)
-  {
-    for(int i=0; i<m_ColElements.size(); i++){
-      if(m_ColElements[i]->id == id){
-	return m_ColElements[i];
-      }      
-    }
-
-    throw Exception("Collision element not found");
-  }
-  */
-}
