@@ -73,9 +73,10 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
     GAME_EVENT_PLAYSOUND                    = 37,
     GAME_EVENT_PLAYMUSIC                    = 38,
     GAME_EVENT_STOPMUSIC                    = 39,
-    GAME_EVENT_SETPHYSICSBLOCKPOS           = 40,
+    /*           = 40, */
     GAME_EVENT_SETPHYSICSBLOCKSELFROTATION  = 41,
-    GAME_EVENT_SETPHYSICSBLOCKTRANSLATION   = 42
+    GAME_EVENT_SETPHYSICSBLOCKTRANSLATION   = 42,
+		GAME_EVENT_CAMERASETPOS                 = 43
   };
 
 class SceneEvent;
@@ -388,25 +389,6 @@ class MGE_MoveBlock : public SceneEvent {
   float m_x, m_y;
 };
 
-class MGE_SetPhysicsBlockPos : public SceneEvent {
- public:
-  MGE_SetPhysicsBlockPos(int p_eventTime);
-  MGE_SetPhysicsBlockPos(int p_eventTime, std::string p_blockID, float p_x, float p_y);
-  ~MGE_SetPhysicsBlockPos();
-
-  void doAction(Scene *p_pScene);
-  void serialize(DBuffer &Buffer);
-  void unserialize(DBuffer &Buffer);
-  static GameEventType SgetType();
-  GameEventType getType();
-
-  std::string toString();
-
- private:
-  std::string m_blockID;
-  float m_x, m_y;
-};
-
 class MGE_SetBlockPos : public SceneEvent {
  public:
   MGE_SetBlockPos(int p_eventTime);
@@ -833,6 +815,24 @@ class MGE_CameraMove : public SceneEvent {
   float m_moveX, m_moveY;
 };
 
+class MGE_CameraSetPos : public SceneEvent {
+ public:
+  MGE_CameraSetPos(int p_eventTime);
+  MGE_CameraSetPos(int p_eventTime, float p_X, float p_Y);
+  ~MGE_CameraSetPos();
+
+  void doAction(Scene *p_pScene);
+  void serialize(DBuffer &Buffer);
+  void unserialize(DBuffer &Buffer);
+  static GameEventType SgetType();
+  GameEventType getType();
+
+  std::string toString();
+
+ private:
+  float m_X, m_Y;
+};
+
 class MGE_CameraZoom : public SceneEvent {
  public:
   MGE_CameraZoom(int p_eventTime);
