@@ -43,6 +43,8 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "gui/specific/GUIXMoto.h"
 #include "Credits.h"
 #include "Replay.h"
+#include "GeomsManager.h"
+#include "XMDemo.h"
 
 #include "states/StateManager.h"
 #include "states/StateEditProfile.h"
@@ -56,10 +58,10 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "thread/UpgradeLevelsThread.h"
 
 #include "UserConfig.h"
-#include "Renderer.h"
 #include "net/NetServer.h"
 #include "net/NetClient.h"
 #include "net/NetActions.h"
+#include "net/ActionReader.h"
 #include "include/xm_SDL_net.h"
 
 #if !defined(WIN32)
@@ -200,11 +202,18 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
   /* ***** */
 
   if(v_xmArgs.isOptConfigPath()) {
-    FS::init("xmoto", "xmoto.bin", "xmoto.log", v_xmArgs.getOpt_configPath_path());
+    XMFS::init("xmoto", "xmoto.bin", "xmoto.log", v_xmArgs.getOpt_configPath_path());
   } else {
-    FS::init("xmoto", "xmoto.bin", "xmoto.log");
+    XMFS::init("xmoto", "xmoto.bin", "xmoto.log");
+  }
+  Logger::init("xmoto.log");
+
+  /* c xmoto files */
+  if(v_xmArgs.isOptBuildQueries()) {
+    LevelsManager::writeDefaultPackages(XM_SQLQUERIES_GEN_FILE);
+    quit();
+    return;
   }
-  Logger::init(FS::getUserDir() + "/xmoto.log");
 
   /* load config file, the session */
   XMSession::createDefaultConfig(m_userConfig);
@@ -225,8 +234,10 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
   } else {
     LogInfo("Systeme is littleendien");
   }
-  LogInfo("User directory: %s", FS::getUserDir().c_str());
-  LogInfo("Data directory: %s", FS::getDataDir().c_str());
+  LogInfo("User data   directory: %s", XMFS::getUserDir(FDT_DATA).c_str());
+  LogInfo("User config directory: %s", XMFS::getUserDir(FDT_CONFIG).c_str());
+  LogInfo("User cache  directory: %s", XMFS::getUserDir(FDT_CACHE).c_str());
+  LogInfo("System data directory: %s", XMFS::getSystemDataDir().c_str());
 
   if(v_xmArgs.isOptListLevels() || v_xmArgs.isOptListReplays() || v_xmArgs.isOptReplayInfos() || v_xmArgs.isOptServerOnly() || v_xmArgs.isOptUpdateLevelsOnly()) {
     v_useGraphics = false;
@@ -261,7 +272,7 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
   xmDatabase* pDb = xmDatabase::instance("main");
   pDb->init(DATABASE_FILE,
 	    XMSession::instance()->profile() == "" ? std::string("") : XMSession::instance()->profile(),
-	    FS::getDataDir(), FS::getUserDir(), FS::binCheckSum(),
+	    XMFS::getSystemDataDir(), XMFS::getUserDir(FDT_DATA), XMFS::binCheckSum(),
 	    v_xmArgs.isOptNoDBDirsCheck() == false,
 	    NULL); // v_useGraphics : NULL because drawlib is still not initialized
   if(XMSession::instance()->sqlTrace()) {
@@ -305,6 +316,7 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
     }
 
     SysMessage::instance()->setDrawLib(drawLib);
+    SysMessage::instance()->setConsoleSize(XMSession::instance()->consoleSize());
     
     drawLib->setNoGraphics(v_useGraphics == false);
     drawLib->setDontUseGLExtensions(XMSession::instance()->glExts() == false);
@@ -324,20 +336,28 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
 	    XMSession::instance()->resolutionHeight(),
 	    XMSession::instance()->bpp());
     /* */
-
   }
 
-  // allocate the statemanager instance so that if it fails, it's not in a thread (serverThread for example)
-  StateManager::instance();
+  if(v_useGraphics) {
+    // allocate the statemanager instance so that if it fails, it's not in a thread (serverThread for example)
+    StateManager::instance();
+  }
 
   /* Init sound system */
   if(v_useGraphics) {
     Sound::init(XMSession::instance());
   }
 
+  bool v_graphicAutomaticMode;
+  v_graphicAutomaticMode = v_useGraphics && (
+  v_xmArgs.isOptLevelID()   ||
+  v_xmArgs.isOptLevelFile() ||
+  v_xmArgs.isOptReplay()    ||
+  v_xmArgs.isOptDemo());
+
   // no command line need the network for the moment
   if(v_useGraphics || v_xmArgs.isOptServerOnly()) {
-    initNetwork(v_xmArgs.isOptServerOnly(), v_xmArgs.isOptServerOnly());
+    initNetwork(v_xmArgs.isOptServerOnly(), v_xmArgs.isOptServerOnly() || v_graphicAutomaticMode);
   }
 
   /* Init renderer */
@@ -346,11 +366,6 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
     switchTestThemeMode(XMSession::instance()->testTheme());
   }    
 
-  if(v_useGraphics) {
-    if(XMSession::instance()->gDebug())
-      GameRenderer::instance()->loadDebugInfo(XMSession::instance()->gDebugFile());
-  }
-
   /* load theme */
   if(pDb->themes_isIndexUptodate() == false) {
     ThemeChoicer::initThemesFromDir(pDb);
@@ -406,6 +421,30 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
   if(v_xmArgs.isOptReplay()) {
     m_PlaySpecificReplay = v_xmArgs.getOpt_replay_file();
   }
+  if(v_xmArgs.isOptDemo()) {
+    /* demo : download the level and the replay
+       load the level as external,
+       play the replay
+     */
+    try {
+      m_xmdemo = new XMDemo(v_xmArgs.getOpt_demo_file());
+      LogInfo("Loading demo file %s\n", v_xmArgs.getOpt_demo_file().c_str());
+
+      _UpdateLoadingScreen(GAMETEXT_DLLEVEL);
+      m_xmdemo->getLevel(XMSession::instance()->proxySettings());
+      _UpdateLoadingScreen(GAMETEXT_DLREPLAY);
+      m_xmdemo->getReplay(XMSession::instance()->proxySettings());
+
+      try {
+	LevelsManager::instance()->addExternalLevel(m_xmdemo->levelFile(), xmDatabase::instance("main"));
+      } catch(Exception &e) {
+	LogError("Can't add level %s as external level", m_xmdemo->levelFile().c_str());
+      }
+      m_PlaySpecificReplay = m_xmdemo->replayFile();
+    } catch(Exception &e) {
+      LogError("Unable to load the demo file");
+    }
+  }
  
   /* Should we clean the level cache? (can also be done when disabled) */
   if(v_xmArgs.isOptCleanCache()) {
@@ -461,12 +500,33 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
   // init physics
   // if(dInitODE2(0) == 0) { /* erreur */} ; // ode 0.10
   dInitODE();
-  m_isODEInitialized = true;
 
-  if(v_xmArgs.isOptServerOnly() == false) {
-    /* Initialize renderer */
-    GameRenderer::instance()->init(drawLib);
-  }  
+  // check ode configuration
+#if HAVEDGETCONFIGURATION == 1
+  LogInfo("Ode config: %s", dGetConfiguration());
+#ifdef dSINGLE
+  LogInfo("Ode config: X-Moto compiled with dSINGLE defined");
+  if(dCheckConfiguration("ODE_single_precision") != 1) {
+    throw Exception("XMoto compiled with ode single precision, but libode library wasn't.");
+  }
+#endif
+#ifdef dDOUBLE
+  LogInfo("Ode config: X-Moto compiled with dDOUBLE defined");
+  if(dCheckConfiguration("ODE_double_precision") != 1) {
+    throw Exception("XMoto compiled with ode as double precision, but libode library wasn't.");
+  }
+#endif
+#if !defined(dSINGLE) && !defined(dDOUBLE)
+  LogInfo("Ode config: X-Moto compiled with dSINGLE and dDOUBLE undefined, dSINGLE is used by default");
+  if(dCheckConfiguration("ODE_single_precision") != 1) {
+    throw Exception("XMoto compiled with ode single precision, but libode library wasn't.");
+  }
+#endif
+#else
+  // old ode version
+  LogWarning("You've an old ode library, xmoto is not able to check whether it's compatible or not");
+#endif
+  m_isODEInitialized = true;
 
   /* build handler */
   InputHandler::instance()->init(m_userConfig, pDb, XMSession::instance()->profile(), XMSession::instance()->enableJoysticks());
@@ -487,6 +547,7 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
   if(v_xmArgs.isOptServerOnly()) {
     try {
       m_standAloneServer = NetServer::instance();
+      NetServer::instance()->setStandAloneOptions();
       NetServer::instance()->start(false);
     } catch(Exception &e) {
       LogError((std::string("Exception: ") + e.getMsg()).c_str());
@@ -512,23 +573,21 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
       /* ======= PLAY SPECIFIC LEVEL ======= */
 
       if(XMSession::instance()->clientGhostMode() == false && NetClient::instance()->isConnected()) {
-	  StateManager::instance()->pushState(new StateWaitServerInstructions(StateManager::instance()->getUniqueId()));
+	  StateManager::instance()->pushState(new StateWaitServerInstructions());
       } else {
-	StateManager::instance()->pushState(new StatePreplayingGame(StateManager::instance()->getUniqueId(),
-								    m_PlaySpecificLevelId, false));
+	StateManager::instance()->pushState(new StatePreplayingGame(m_PlaySpecificLevelId, false));
       }
       LogInfo("Playing as '%s'...", XMSession::instance()->profile().c_str());
     }
     else if(m_PlaySpecificReplay != "") {
       /* ======= PLAY SPECIFIC REPLAY ======= */
-      StateManager::instance()->pushState(new StatePreplayingReplay(StateManager::instance()->getUniqueId(),
-								    m_PlaySpecificReplay, false));
+      StateManager::instance()->pushState(new StatePreplayingReplay(m_PlaySpecificReplay, false));
     }
     else {
       /* display what must be displayed */
       StateManager::instance()->pushState(new StateMainMenu());
       if(XMSession::instance()->clientGhostMode() == false && NetClient::instance()->isConnected()) {
-	  StateManager::instance()->pushState(new StateWaitServerInstructions(StateManager::instance()->getUniqueId()));
+	  StateManager::instance()->pushState(new StateWaitServerInstructions());
       }
     }
 
@@ -566,8 +625,8 @@ void GameApp::manageEvent(SDL_Event* Event) {
   /* What event? */
   switch(Event->type) {
   case SDL_KEYDOWN:
-    //printf("%i\n", Event->key.keysym.sym);
     utf8Char = unicode2utf8(Event->key.keysym.unicode);
+    //printf("%i - %i\n", Event->key.keysym.sym, utf8Char.c_str()[0]);
     StateManager::instance()->xmKey(INPUT_DOWN, XMKey(Event->key.keysym.sym, Event->key.keysym.mod, utf8Char));
     break;
   case SDL_KEYUP:
@@ -670,6 +729,7 @@ void GameApp::run_loop() {
 }
 
 void GameApp::run_unload() {
+
   if(Logger::isInitialized()) {
     LogInfo("UserUnload started at %.3f", GameApp::getXMTime());
   }
@@ -682,14 +742,18 @@ void GameApp::run_unload() {
     delete m_pWebLevels;
   }    
 
-  if(GameRenderer::instance() != NULL) {
-    GameRenderer::instance()->unprepareForNewLevel(); /* just to be sure, shutdown can happen quite hard */
-    GameRenderer::instance()->shutdown();
+  if(InputHandler::instance() != NULL) {
     InputHandler::instance()->uninit(); // uinit the input, but you can still save the config
   }
 
   uninitNetwork();
 
+  // detroy the demo
+  if(m_xmdemo != NULL) {
+    m_xmdemo->destroyFiles();
+    delete m_xmdemo;
+  }
+
   StateManager::destroy();
   
   if(Sound::isInitialized()) {
@@ -700,7 +764,6 @@ void GameApp::run_unload() {
     dCloseODE(); // uninit ODE
   }
 
-  GameRenderer::destroy();
   SysMessage::destroy();  
 
   if(Logger::isInitialized()) {
@@ -722,6 +785,7 @@ void GameApp::run_unload() {
 
   InputHandler::destroy();
   LevelsManager::destroy();
+  GeomsManager::destroy();
   Theme::destroy();
   XMSession::destroy("live");
   XMSession::destroy("file");
@@ -791,6 +855,7 @@ void GameApp::run_unload() {
     FontManager* v_fm;
     FontGlyph* v_fg;
     int v_fh;
+    RenderSurface v_screen(Vector2i(0,0), Vector2i(getDrawLib()->getDispWidth(), getDrawLib()->getDispHeight()));
 
     getDrawLib()->clearGraphics();
     getDrawLib()->resetGraphics();
@@ -798,17 +863,17 @@ void GameApp::run_unload() {
     v_fm = getDrawLib()->getFontBig();
     v_fg = v_fm->getGlyph(GAMETEXT_LOADING);
     v_fh = v_fg->realHeight();
-    v_fm->printString(v_fg,
-		      getDrawLib()->getDispWidth()/2 - 256,
-		      getDrawLib()->getDispHeight()/2 - 30,
+    v_fm->printString(getDrawLib(), v_fg,
+		      v_screen.getDispWidth()/2 - 256,
+		      v_screen.getDispHeight()/2 - 30,
 		      MAKE_COLOR(255,255,255, 255));
     
     if(NextTask != "") {
       v_fm = getDrawLib()->getFontSmall();
       v_fg = v_fm->getGlyph(NextTask);
-      v_fm->printString(v_fg,
-			getDrawLib()->getDispWidth()/2 - 256,
-			getDrawLib()->getDispHeight()/2 -30 + v_fh + 2,
+      v_fm->printString(getDrawLib(), v_fg,
+			v_screen.getDispWidth()/2 - 256,
+			v_screen.getDispHeight()/2 -30 + v_fh + 2,
 			MAKE_COLOR(255,255,255,255));      
     }
     getDrawLib()->flushGraphics();
@@ -873,6 +938,7 @@ void GameApp::uninitNetwork() {
 
   if(Logger::isInitialized()) {
     NetAction::logStats();
+    ActionReader::logStats();
   }
 
   SDLNet_Quit();
