@@ -32,6 +32,7 @@ public:
   LuaLibGame(Scene *i_pScene);
   ~LuaLibGame();
 
+  void loadScriptFile(const std::string& i_scriptFilename);
   void loadScript(const std::string& i_scriptCode, const std::string& i_scriptFilename);
   std::string getErrorMsg();
 
@@ -77,6 +78,7 @@ private:
   static int L_Game_SetEntityPos(lua_State *pL);
   static int L_Game_SetKeyHook(lua_State *pL);
   static int L_Game_GetKeyByAction(lua_State *pL);
+  static int L_Game_GetKeyByActionTech(lua_State *pL);
   static int L_Game_Log(lua_State *pL);
   static int L_Game_SetBlockCenter(lua_State *pL);
   static int L_Game_SetBlockRotation(lua_State *pL);
@@ -114,10 +116,11 @@ private:
   static int L_Game_GetPlayerVelocity(lua_State *pL);
   static int L_Game_GetPlayerSpeed(lua_State *pL);
   static int L_Game_GetPlayerAngle(lua_State *pL);
-	static int L_Game_SetCameraPosition(lua_State *pL);
+  static int L_Game_SetCameraPosition(lua_State *pL);
+  static int L_Game_GetPlayerProfileName(lua_State *pL);
   /* Script Timer Functions*/
   static int L_Game_StartTimer(lua_State *pL);  
-	static int L_Game_SetTimerDelay(lua_State *pL);  
+  static int L_Game_SetTimerDelay(lua_State *pL);  
   static int L_Game_StopTimer(lua_State *pL);
 	
 
