@@ -24,6 +24,9 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "VCommon.h"
 #include "xmscene/Scene.h"
 #include "gui/basic/GUI.h"
+#include "XMSession.h"
+#include "helpers/RenderSurface.h"
+#include "xmscene/BikeGhost.h"
 
 #ifdef ENABLE_OPENGL
 #include "include/xm_OpenGL.h"
@@ -33,6 +36,9 @@ class ParticlesSource;
 class Universe;
 class BlockVertex;
 class Camera;
+class Geom;
+class ConvexBlock;
+class LevelGeoms;
 
   /*===========================================================================
   Graphical debug info
@@ -42,38 +48,6 @@ class Camera;
     std::vector<std::string> Args;    
   };
 
-  /*===========================================================================
-  Static geometry
-  ===========================================================================*/
-  struct GeomCoord {
-    float x,y;
-  };
-
-  struct GeomPoly {
-    GeomPoly() {
-      nNumVertices = 0;
-      pVertices = pTexCoords = NULL;
-      nVertexBufferID = nTexCoordBufferID = 0;
-    }
-    
-    unsigned int nNumVertices;
-    GeomCoord *pVertices;
-    GeomCoord *pTexCoords;
-    
-    unsigned int nVertexBufferID,nTexCoordBufferID;
-  };
-
-  struct Geom {
-    Geom() {
-      pTexture = NULL;
-    }
-        
-    std::vector<GeomPoly *> Polys;    
-    Texture *pTexture;
-    
-    Vector2f Min,Max; /* AABB */
-  };
-
   /*===========================================================================
   Special effects overlay
   ===========================================================================*/
@@ -81,6 +55,7 @@ class Camera;
     public:
       SFXOverlay() {
         m_drawLib = NULL;
+	m_screen = NULL;
 #ifdef ENABLE_OPENGL	
         m_bUseShaders = false;
         m_VertShaderID = m_FragShaderID = m_ProgramID = 0;
@@ -90,11 +65,11 @@ class Camera;
       }
     
       /* Methods */
-      void init(DrawLib* i_drawLib, unsigned int nWidth, unsigned int nHeight);
+      void init(DrawLib* i_drawLib, RenderSurface* i_screen, unsigned int nWidth, unsigned int nHeight);
       void cleanUp(void);
       void beginRendering(void);
       void endRendering(void);
-      void fade(float f);
+      void fade(float f, unsigned int i_frameNumber);
       void present(void);
     
     private:
@@ -115,25 +90,25 @@ class Camera;
       bool _SetShaderSource(GLhandleARB ShaderID, const std::string &File);
 #endif
       
-      
-
       int m_nOverlayWidth,m_nOverlayHeight;
       DrawLib* m_drawLib;
+      RenderSurface* m_screen;
   };
 
   /*===========================================================================
     Game rendering class
     ===========================================================================*/
-class GameRenderer : public Singleton<GameRenderer> {
-  friend class Singleton<GameRenderer>;
+class GameRenderer {
 
 public:
-  void init(DrawLib* i_drawLib); /* only called at start-up, and not per-level */
-  void shutdown(void);
+  GameRenderer();
+  ~GameRenderer();
+
+  void init(DrawLib* i_drawLib, RenderSurface* i_screen); /* only called at start-up, and not per-level */
   void render(Scene* i_scene);
 
   void prepareForNewLevel(Universe* i_universe);
-  void unprepareForNewLevel(void);
+  void unprepareForNewLevel(Universe* i_universe);
 
   void loadDebugInfo(std::string File);
 
@@ -146,18 +121,21 @@ public:
   void setBestTime(const std::string& s) {m_bestTime = s;}
   void setWorldRecordTime(const std::string &s) {m_worldRecordTime = s;}
   void setShowMinimap(bool i_value);
+  void setRenderGhostTrail(bool i_value);
+  bool renderGhostTrail();
   void setShowTimePanel(bool i_value);
+  void setScreenShade(bool i_doShade_global, bool i_doShadeAnim_global, float i_shadeTime_global); 
   void hideReplayHelp();
   bool showEngineCounter() const;
   void setShowEngineCounter(bool i_value);
   bool showMinimap() const;
   void showReplayHelp(float p_speed, bool bAllowRewind);
-  void switchFollow(Scene* i_scene);
 
   void setShowGhostsText(bool i_value);
   bool showGhostsText() const;
+  void renderGameMessages(Scene* i_scene);
 
-  unsigned int currentRegistrationStage() const;
+  void setGraphicsLevel();
 
 private:
   void renderMiniMap(Scene* i_scene, int x,int y,int nWidth,int nHeight);
@@ -167,29 +145,35 @@ private:
       
   std::string getBestTime(void) {return m_bestTime;}
 
-  GameRenderer();
-  ~GameRenderer();
-
   DrawLib* m_drawLib;
+  RenderSurface m_screen; // this is a copy of the initial screen (in fact, a state can be changed and then, the screen deleted)
   std::vector<GraphDebugInfo *> m_DebugInfo;
-      
-  std::vector<Geom*> m_StaticGeoms;
-  std::vector<Geom*> m_DynamicGeoms;
-  std::vector<Geom*> m_edgeGeoms;
 
   std::string m_bestTime;
-  std::string m_replayHelp;
+  std::string m_replayHelp_l;
+  std::string m_replayHelp_r;
   std::string m_worldRecordTime;
 
+  // registering
+  unsigned int m_registeringValue;
+
+  GraphicsLevel m_graphicsLevel;
+
   bool m_allowGhostEffect; // ask to not do the ghost effect
 
   float m_previousEngineSpeed;
   float m_previousEngineLinVel;
 
   bool m_showMinimap;
+  bool m_renderGhostTrail;
   bool m_showEngineCounter;
   bool m_showTimePanel;
   bool m_showGhostsText;
+  
+  // for Screenshadowing in menus
+  bool m_doShade_global; 
+  bool m_doShadeAnim_global;
+  float m_nShadeTime_global;
 
   /* FBO overlay */
   SFXOverlay m_Overlay;
@@ -200,10 +184,9 @@ private:
   float m_sizeMultOfEntitiesToTake;
   float m_sizeMultOfEntitiesWhichMakeWin;
   int   m_nParticlesRendered;
-  std::string       m_currentEdgeEffect;
-  EdgeEffectSprite* m_currentEdgeSprite;
-
-
+  Sprite*           m_currentSkySprite;
+  Sprite*           m_currentSkySprite2;
+  
   float m_xScale;
   float m_yScale;
   float m_xScaleDefault;
@@ -237,8 +220,8 @@ private:
 		      Biker* i_biker,
 		      int i_90_rotation = 0
 		      );
-  void _RenderBlocks(Scene* i_scene);
-  void _RenderBlock(Block* block);
+  void _RenderStaticBlocks(Scene* i_scene);
+  void _RenderStaticBlock(Block* block);
   void _RenderBlockEdges(Block* block);
   void _RenderDynamicBlocks(Scene* i_scene, bool bBackground=false);
   void _RenderBackground(Scene* i_scene);
@@ -246,55 +229,32 @@ private:
   void _RenderLayer(Scene* i_scene, int layer);
   void _RenderSky(Scene* i_scene, float i_zoom, float i_offset, const TColor& i_color,
 		  float i_driftZoom, const TColor &i_driftColor, bool i_drifted);
-  void _RenderGameMessages(Scene* i_scene); 
+  void _RenderGameMessages(Scene* i_scene, bool renderOverShadow); 
   void _RenderGameStatus(Scene* i_scene);
   void _RenderParticles(Scene* i_scene, bool bFront=true);
   void _RenderParticleDraw(Vector2f P,Texture *pTexture,float fSize,float fAngle, TColor c);
   void _RenderParticle(Scene* i_scene, ParticlesSource *i_source, unsigned int sprite=0);
   void _RenderInGameText(Vector2f P,const std::string &Text,Color c = 0xffffffff, float i_xcentering = 0.0, float i_ycentering = 0.0);
-  void _RenderZone(Zone *i_zone);
+  void _RenderZone(Zone *i_zone, bool i_renderOnlyDeathZone);
 
   void _RenderGhost(Scene* i_scene, Biker* i_ghost, int i, float i_textOffset);
-
+  void _RenderGhostTrail(Scene* i_scene, AABB* i_screenBBox, float i_scale);
   void _RenderDebugInfo(void);
-
+  
+  void _RenderScreenShadow(Scene* i_scene);
   void _RenderAlphaBlendedSection(Texture *pTexture,const Vector2f &p0,const Vector2f &p1,const Vector2f &p2,const Vector2f &p3,
 				  const TColor& i_filterColor = TColor(255, 255, 255, 0));
   void _RenderAdditiveBlendedSection(Texture *pTexture,const Vector2f &p0,const Vector2f &p1,const Vector2f &p2,const Vector2f &p3);
   void _RenderAlphaBlendedSectionSP(Texture *pTexture,const Vector2f &p0,const Vector2f &p1,const Vector2f &p2,const Vector2f &p3);
-  void _RenderRectangle(const Vector2f& i_p1, const Vector2f& i_p2, const Color& i_color);
-  void _RenderCircle(unsigned int nSteps,Color CircleColor,const Vector2f &C,float fRadius);
-  void _deleteGeoms(std::vector<Geom *>& geom, bool useFree=false);
+  void _RenderRectangle(const Vector2f& i_p1, const Vector2f& i_p2, const Color& i_color, bool i_filled = false);
+  void _RenderCircle(unsigned int nSteps, const Color i_color,const Vector2f &C,float fRadius, bool i_filled = false);
 
   void renderTimePanel(Scene* i_scene);
   void renderReplayHelpMessage(Scene* i_scene);
 
   Texture* loadTexture(std::string textureName);
-  Texture* loadTextureEdge(std::string textureName);
-  int  edgeGeomExists(Block* pBlock, std::string texture);
   void initCameras(Universe* i_universe);
-  int  loadBlock(Block* pBlock, Universe* i_universe, unsigned int currentScene, int sameSceneAs, int blockIndex);
-  int  loadBlockGeom(Block* pBlock, std::vector<Geom *>* pGeoms, Texture* pTexture, Vector2f Center, Scene* pScene);
-  int  loadBlockEdge(Block* pBlock, Vector2f Center, Scene* pScene);
-  void calculateEdgePosition(Block* pBlock,
-			     BlockVertex* vertexA1,
-			     BlockVertex* vertexB1,
-			     BlockVertex* vertexC1,
-			     Vector2f     center,
-			     Vector2f& A1, Vector2f& B1,
-			     Vector2f& B2, Vector2f& A2,
-			     Vector2f& C1, Vector2f& C2,
-			     Vector2f oldC2, Vector2f oldB2, bool useOld,
-			     bool AisLast, bool& swapDone);
-  void calculateEdgeTexture(Block* pBlock,
-			    Vector2f A1,   Vector2f B1,
-			    Vector2f B2,   Vector2f A2,
-			    Vector2f& ua1, Vector2f& ub1,
-			    Vector2f& ub2, Vector2f& ua2);
-
-  void beginTexturesRegistration();
-  void endTexturesRegistration();
-  unsigned int m_curRegistrationStage;
+  unsigned int  loadBlock(LevelGeoms* i_levelGeoms, Block* pBlock, int blockIndex);
 };
 
 #endif
