@@ -22,6 +22,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *  Replay recording and management.
  */
 #include "Replay.h"
+#include "helpers/Log.h"
 #if defined(WIN32) 
   #include "zlib.h"
 #else
@@ -41,6 +42,24 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "xmscene/Block.h"
 #include "db/xmDatabase.h"
 
+// minimum value to consider a block has moved
+#define RMOVINGBLOCK_MIN_DIFFMOVE             0.1
+#define RMOVINGBLOCK_MIN_DIFFROTATION         0.1
+
+// if the block has moved, save it but at a frequency depending on the distance to the player
+#define RMOVINGBLOCK_MIN_FARDIFFTIME1         15
+#define RMOVINGBLOCK_MIN_FARPLAYERLENGTH1     5.0
+#define RMOVINGBLOCK_MIN_FARDIFFTIME2         80
+#define RMOVINGBLOCK_MIN_FARPLAYERLENGTH2     10.0
+#define RMOVINGBLOCK_MIN_FARDIFFTIME3         150
+//#define RMOVINGBLOCK_MIN_FARPLAYERLENGTH3 no values while it's all what is greater
+
+ // if the block has not moved a lot, but for a long time, save it
+#define MOVINGBLOCK_MIN_LONGDIFFTIME            500
+#define RMOVINGBLOCK_MIN_LONGDIFFMOVE             0.05
+#define RMOVINGBLOCK_MIN_LONGDIFFROTATION         0.05
+
+
   bool Replay::m_bEnableCompression = true;
 
   Replay::Replay() {
@@ -49,6 +68,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
     m_finishTime = 0;
     m_pcInputEventsData = NULL;
     m_nInputEventsDataSize = 0;
+    m_saved = false;
   }
       
   Replay::~Replay() {
@@ -79,10 +99,9 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
     }
   }
   
-  void Replay::finishReplay(bool bFinished, int finishTime, int i_format) {
+  void Replay::finishReplay(bool bFinished, int finishTime) {
     m_finishTime = finishTime;
     m_bFinished = bFinished;
-    saveReplay(i_format);
   }
   
   void Replay::createReplay(const std::string &FileName, const std::string &LevelID,
@@ -96,9 +115,13 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
     initOutput(1024);
   }
   
-  void Replay::saveReplay(int i_format) {
+  void Replay::saveReplayIfNot(int i_format) {
 
-    FileHandle *pfh = FS::openOFile(std::string("Replays/") + m_FileName);
+    if(m_saved) {
+      return;
+    }
+
+    FileHandle *pfh = XMFS::openOFile(FDT_DATA, std::string("Replays/") + m_FileName);
     if(pfh == NULL) {
       LogWarning("Failed to open replay file for output: %s",(std::string("Replays/") + m_FileName).c_str());
       return;
@@ -111,19 +134,20 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
       saveReplay_1(pfh);
       break;
 
-    case 2:
-      saveReplay_2(pfh);
+    case 3:
+      saveReplay_3(pfh);
       break;
 
     default:
-      FS::closeFile(pfh);
+      XMFS::closeFile(pfh);
       throw Exception("Invalid replay format");
     }
 
-    FS::closeFile(pfh);
+    XMFS::closeFile(pfh);
+    m_saved = true;
   }
 
-  void Replay::saveReplay_2(FileHandle *pfh) {
+  void Replay::saveReplay_3(FileHandle *pfh) {
     const char *pcData;
     int nDataSize;
     char *pcCompressedData;
@@ -134,13 +158,14 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
     /* keep header uncompressed to be faster to read just it */
 
     /* Header */
-    FS::writeByte(pfh,2); /* Version: 2 */
-    FS::writeString(pfh,m_LevelID);
-    FS::writeString(pfh,m_PlayerName);
-    FS::writeBool(pfh,m_bFinished);
-    FS::writeFloat_LE(pfh, GameApp::timeToFloat(m_finishTime));
-    FS::writeFloat_LE(pfh,m_fFrameRate);
-    FS::writeInt_LE(pfh,m_nStateSize);
+    XMFS::writeByte(pfh,3); /* Version: 3 */
+    XMFS::writeInt_LE(pfh,0x12345678); /* Endianness guard */
+    XMFS::writeString(pfh,m_LevelID);
+    XMFS::writeString(pfh,m_PlayerName);
+    XMFS::writeFloat_LE(pfh,m_fFrameRate);
+    XMFS::writeInt_LE(pfh,m_nStateSize);
+    XMFS::writeBool(pfh,m_bFinished);
+    XMFS::writeFloat_LE(pfh, GameApp::timeToFloat(m_finishTime));
 
     /* ***** ***** ***** ***** ***** **/
     /* compress all except the header */
@@ -156,38 +181,68 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
     v_replay.writeBuf(pcData, nDataSize);
     
     /* Chunks */
-    v_replay << m_Chunks.size();
+    v_replay << (unsigned int) m_Chunks.size();
     for(unsigned int i=0;i<m_Chunks.size();i++) {
       v_replay << m_Chunks[i]->nNumStates;
       v_replay.writeBuf(m_Chunks[i]->pcChunkData, m_nStateSize * m_Chunks[i]->nNumStates);
     }
 
+    /* Moving blocks */
+    int nstates = 0;
+    unsigned int nmovingBlocks = 0;
+
+    // save only moving blocks
+    for(unsigned int i=0; i<m_movingBlocksForSaving.size(); i++) {
+      if(m_movingBlocksForSaving[i].states.size() > 1) { // > 1 because if there is only one state, the block has not moved
+	nmovingBlocks++;
+      }
+    }
+    v_replay << nmovingBlocks;
+    for(unsigned int i=0; i<m_movingBlocksForSaving.size(); i++) {
+      if(m_movingBlocksForSaving[i].states.size() > 1) { // > 1 because if there is only one state, the block has not moved
+	v_replay << m_movingBlocksForSaving[i].name;
+	v_replay << m_movingBlocksForSaving[i].states.size();
+	for(unsigned int j=0; j<m_movingBlocksForSaving[i].states.size(); j++) {
+	  v_replay << m_movingBlocksForSaving[i].states[j].time;
+	  v_replay << m_movingBlocksForSaving[i].states[j].position.x;
+	  v_replay << m_movingBlocksForSaving[i].states[j].position.y;
+	  v_replay << m_movingBlocksForSaving[i].states[j].rotation;
+	  nstates++;
+	}
+      }
+    }
+
+    LogInfo("Replay moving block states size = %iKB (%i blocks, %i states, %i bytes/state)",
+	    nstates*sizeof(rmblockState)/1024, m_movingBlocksForSaving.size(), nstates, sizeof(rmblockState));
+
     /* zip and write into the file */
     pcData    = v_replay.convertOutputToInput();
     nDataSize = v_replay.numRemainingBytes();
     pcCompressedData = FileCompression::zcompress(pcData, nDataSize, nCompressedDataSize);
-    FS::writeInt_LE(pfh, nDataSize);
-    FS::writeInt_LE(pfh, nCompressedDataSize);
-    FS::writeBuf(pfh, (char *)pcCompressedData, nCompressedDataSize); 
+    XMFS::writeInt_LE(pfh, nDataSize);
+    XMFS::writeInt_LE(pfh, nCompressedDataSize);
+    XMFS::writeBuf(pfh, (char *)pcCompressedData, nCompressedDataSize); 
     free(pcCompressedData);
+    LogInfo("Replay - uncompressed = %iKB ; compressed = %iKB (ratio = %.2f%%)",
+	    nDataSize/1024, nCompressedDataSize/1024, (((float)nCompressedDataSize*100.0))/((float)nDataSize));
   }
 
   void Replay::saveReplay_1(FileHandle *pfh) {
     
     /* Write header */
-    FS::writeByte(pfh,1); /* Version: 1 */
-    FS::writeInt_LE(pfh,0x12345678); /* Endianness guard */
-    FS::writeString(pfh,m_LevelID);
-    FS::writeString(pfh,m_PlayerName);
-    FS::writeFloat_LE(pfh,m_fFrameRate);
-    FS::writeInt_LE(pfh,m_nStateSize);
-    FS::writeBool(pfh,m_bFinished);
-    FS::writeFloat_LE(pfh, GameApp::timeToFloat(m_finishTime));
+    XMFS::writeByte(pfh,1); /* Version: 1 */
+    XMFS::writeInt_LE(pfh,0x12345678); /* Endianness guard */
+    XMFS::writeString(pfh,m_LevelID);
+    XMFS::writeString(pfh,m_PlayerName);
+    XMFS::writeFloat_LE(pfh,m_fFrameRate);
+    XMFS::writeInt_LE(pfh,m_nStateSize);
+    XMFS::writeBool(pfh,m_bFinished);
+    XMFS::writeFloat_LE(pfh, GameApp::timeToFloat(m_finishTime));
     
     /* Events */
     const char *pcUncompressedEvents = convertOutputToInput();
     int nUncompressedEventsSize = numRemainingBytes();
-    FS::writeInt_LE(pfh,nUncompressedEventsSize);
+    XMFS::writeInt_LE(pfh,nUncompressedEventsSize);
     
     /* Compression? */
     if(m_bEnableCompression) {
@@ -198,29 +253,29 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
       int nZRet = compress2((Bytef *)pcCompressedEvents,&nDestLen,(Bytef *)pcUncompressedEvents,nSrcLen,9);
       if(nZRet != Z_OK) {
         /* Failed to compress, save raw events */
-        FS::writeBool(pfh,false); /* compression: false */
-        FS::writeBuf(pfh,(char *)pcUncompressedEvents,nUncompressedEventsSize);
+        XMFS::writeBool(pfh,false); /* compression: false */
+        XMFS::writeBuf(pfh,(char *)pcUncompressedEvents,nUncompressedEventsSize);
       }
       else {
         /* OK */        
-        FS::writeBool(pfh,true); /* compression: true */
-        FS::writeInt_LE(pfh,nDestLen);
-        FS::writeBuf(pfh,(char *)pcCompressedEvents,nDestLen);
+        XMFS::writeBool(pfh,true); /* compression: true */
+        XMFS::writeInt_LE(pfh,nDestLen);
+        XMFS::writeBuf(pfh,(char *)pcCompressedEvents,nDestLen);
       }
       delete [] pcCompressedEvents;
     }
     else {    
       /* No compression */
-      FS::writeBool(pfh,false); /* compression: false */
-      FS::writeBuf(pfh,(char *)pcUncompressedEvents,nUncompressedEventsSize);
+      XMFS::writeBool(pfh,false); /* compression: false */
+      XMFS::writeBuf(pfh,(char *)pcUncompressedEvents,nUncompressedEventsSize);
     }
     
     /* Chunks */
-    FS::writeInt_LE(pfh,m_Chunks.size());
+    XMFS::writeInt_LE(pfh,m_Chunks.size());
     
     /* Write chunks */    
     for(unsigned int i=0;i<m_Chunks.size();i++) {
-      FS::writeInt_LE(pfh,m_Chunks[i]->nNumStates);
+      XMFS::writeInt_LE(pfh,m_Chunks[i]->nNumStates);
       
       /* Compression enabled? */
       if(m_bEnableCompression) {
@@ -231,42 +286,55 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 	int nZRet = compress2((Bytef *)pcCompressed,&nDestLen,(Bytef *)m_Chunks[i]->pcChunkData,nSrcLen,9);
 	if(nZRet != Z_OK) {
 	  /* Failed to compress... Save uncompressed chunk then */
-	  FS::writeBool(pfh,false); /* compression: false */
-	  FS::writeBuf(pfh,m_Chunks[i]->pcChunkData,m_nStateSize * m_Chunks[i]->nNumStates);
+	  XMFS::writeBool(pfh,false); /* compression: false */
+	  XMFS::writeBuf(pfh,m_Chunks[i]->pcChunkData,m_nStateSize * m_Chunks[i]->nNumStates);
 	} else {
 	  /* Compressed ok */
-	  FS::writeBool(pfh,true); /* compression: true */
-	  FS::writeInt_LE(pfh,nDestLen);
-	  FS::writeBuf(pfh,(char *)pcCompressed,nDestLen);
+	  XMFS::writeBool(pfh,true); /* compression: true */
+	  XMFS::writeInt_LE(pfh,nDestLen);
+	  XMFS::writeBuf(pfh,(char *)pcCompressed,nDestLen);
 	}
 	delete [] pcCompressed;        
       } else {
-	FS::writeBool(pfh,false); /* compression: false */
-	FS::writeBuf(pfh,m_Chunks[i]->pcChunkData,m_nStateSize * m_Chunks[i]->nNumStates);
+	XMFS::writeBool(pfh,false); /* compression: false */
+	XMFS::writeBuf(pfh,m_Chunks[i]->pcChunkData,m_nStateSize * m_Chunks[i]->nNumStates);
       }
     }
   }
   
-void Replay::openReplay_2(FileHandle *pfh, bool bDisplayInformation) {
+void Replay::openReplay_3(FileHandle *pfh, bool bDisplayInformation) {
   DBuffer v_replay;
   int v_nDataSize;
   char *v_pcData;
   int v_nCompressedDataSize;
   char *v_pcCompressedData;
 
+  /* Little/big endian safety check */
+  if(XMFS::readInt_LE(pfh) != 0x12345678) {
+    LogWarning("Sorry, the replay you're trying to open are not endian-compatible with your computer!");
+    throw Exception("Unable to open the replay");        
+  }
+
   /* Header */
-  m_LevelID = FS::readString(pfh);
+  m_LevelID = XMFS::readString(pfh);
   if(bDisplayInformation) {
     printf("%-30s: %s\n", "Level Id", m_LevelID.c_str());
   }
 
-  m_PlayerName = FS::readString(pfh);
+  m_PlayerName = XMFS::readString(pfh);
   if(bDisplayInformation) {
     printf("%-30s: %s\n", "Player", m_PlayerName.c_str());
   }      
+
+  m_fFrameRate = XMFS::readFloat_LE(pfh);
+
+  m_nStateSize = XMFS::readInt_LE(pfh);
+  if(bDisplayInformation) {
+    printf("%-30s: %i\n", "State size", m_nStateSize);
+  } 
       
-  m_bFinished = FS::readBool(pfh);
-  m_finishTime = GameApp::floatToTime(FS::readFloat_LE(pfh));
+  m_bFinished = XMFS::readBool(pfh);
+  m_finishTime = GameApp::floatToTime(XMFS::readFloat_LE(pfh));
   if(bDisplayInformation) {
     if(m_bFinished) {
       printf("%-30s: %.2f (%f)\n", "Finish time", m_finishTime / 100.0, m_finishTime / 100.0);
@@ -275,16 +343,9 @@ void Replay::openReplay_2(FileHandle *pfh, bool bDisplayInformation) {
     }
   }
 
-  m_fFrameRate = FS::readFloat_LE(pfh);
-
-  m_nStateSize = FS::readInt_LE(pfh);
-  if(bDisplayInformation) {
-    printf("%-30s: %i\n", "State size", m_nStateSize);
-  } 
-
   /* zuncompressed */
-  v_nDataSize           = FS::readInt_LE(pfh);
-  v_nCompressedDataSize = FS::readInt_LE(pfh);
+  v_nDataSize           = XMFS::readInt_LE(pfh);
+  v_nCompressedDataSize = XMFS::readInt_LE(pfh);
 
   v_pcData = (char*) malloc(v_nDataSize);
   if(v_pcData == NULL) {
@@ -295,7 +356,7 @@ void Replay::openReplay_2(FileHandle *pfh, bool bDisplayInformation) {
     free(v_pcData);
     throw Exception("Unable to malloc for decompression");
   }
-  FS::readBuf(pfh, v_pcCompressedData, v_nCompressedDataSize);
+  XMFS::readBuf(pfh, v_pcCompressedData, v_nCompressedDataSize);
   try {
     FileCompression::zuncompress(v_pcCompressedData, v_nCompressedDataSize, v_pcData, v_nDataSize);
   } catch(Exception &e) {
@@ -344,39 +405,64 @@ void Replay::openReplay_2(FileHandle *pfh, bool bDisplayInformation) {
     v_replay.readBuf(Chunk->pcChunkData, m_nStateSize*Chunk->nNumStates);
     m_Chunks.push_back(Chunk);
   }
+
+  /* moving blocks */
+  unsigned int v_nmovingBlocks;
+  unsigned int  v_nstates;
+  rmtime t;
+  rmtimeState s;
+
+  v_replay >> v_nmovingBlocks;
+
+  for(unsigned int i=0; i<v_nmovingBlocks; i++) {
+    v_replay >> t.name;
+    t.block = NULL; // don't initialize now, the level is not loaded
+    t.readPos = 0;
+    m_movingBlocksForLoading.push_back(t);
+
+    v_replay >> v_nstates;
+    for(unsigned int j=0; j<v_nstates; j++) {
+      v_replay >> s.time;
+      v_replay >> s.position.x;
+      v_replay >> s.position.y;
+      v_replay >> s.rotation;
+
+      m_movingBlocksForLoading[m_movingBlocksForLoading.size()-1].states.push_back(s);
+    }
+  }
 }
 
 void Replay::openReplay_1(FileHandle *pfh, bool bDisplayInformation, int nVersion) {
   /* Little/big endian safety check */
-  if(FS::readInt_LE(pfh) != 0x12345678) {
+  if(XMFS::readInt_LE(pfh) != 0x12345678) {
     LogWarning("Sorry, the replay you're trying to open are not endian-compatible with your computer!");
     throw Exception("Unable to open the replay");        
   }
   
   /* Read level ID */
-  m_LevelID = FS::readString(pfh);
+  m_LevelID = XMFS::readString(pfh);
   if(bDisplayInformation) {
     printf("%-30s: %s\n", "Level Id", m_LevelID.c_str());
   }
 
   /* Read player name */
-  m_PlayerName = FS::readString(pfh);
+  m_PlayerName = XMFS::readString(pfh);
   if(bDisplayInformation) {
     printf("%-30s: %s\n", "Player", m_PlayerName.c_str());
   }      
 
   /* Read replay frame rate */
-  m_fFrameRate = FS::readFloat_LE(pfh);
+  m_fFrameRate = XMFS::readFloat_LE(pfh);
 
   /* Read state size */
-  m_nStateSize = FS::readInt_LE(pfh);
+  m_nStateSize = XMFS::readInt_LE(pfh);
   if(bDisplayInformation) {
     printf("%-30s: %i\n", "State size", m_nStateSize);
   } 
       
   /* Read finish time if any */
-  m_bFinished = FS::readBool(pfh);
-  m_finishTime = GameApp::floatToTime(FS::readFloat_LE(pfh));
+  m_bFinished = XMFS::readBool(pfh);
+  m_finishTime = GameApp::floatToTime(XMFS::readFloat_LE(pfh));
   if(bDisplayInformation) {
     if(m_bFinished) {
       printf("%-30s: %.2f (%f)\n", "Finish time", m_finishTime / 100.0, m_finishTime / 100.0);
@@ -388,22 +474,22 @@ void Replay::openReplay_1(FileHandle *pfh, bool bDisplayInformation, int nVersio
   /* Version 1 includes event data */
   if(nVersion == 1) {
     /* Read uncompressed size */
-    m_nInputEventsDataSize = FS::readInt_LE(pfh);
+    m_nInputEventsDataSize = XMFS::readInt_LE(pfh);
     if(bDisplayInformation) {
       printf("%-30s: %i\n", "Events data size", m_nInputEventsDataSize);
     } 
     m_pcInputEventsData = new char [m_nInputEventsDataSize];
         
     /* Compressed? */
-    if(FS::readBool(pfh)) {
+    if(XMFS::readBool(pfh)) {
       /* Compressed */          
-      int nCompressedEventsSize = FS::readInt_LE(pfh);
+      int nCompressedEventsSize = XMFS::readInt_LE(pfh);
       if(bDisplayInformation) {
 	printf("%-30s: %i\n", "Compressed events data size", nCompressedEventsSize);
       } 
           
       char *pcCompressedEvents = new char [nCompressedEventsSize];
-      FS::readBuf(pfh,pcCompressedEvents,nCompressedEventsSize);
+      XMFS::readBuf(pfh,pcCompressedEvents,nCompressedEventsSize);
           
       /* Unpack */
       uLongf nDestLen = m_nInputEventsDataSize;
@@ -422,7 +508,7 @@ void Replay::openReplay_1(FileHandle *pfh, bool bDisplayInformation, int nVersio
     }
     else {
       /* Not compressed */
-      FS::readBuf(pfh,m_pcInputEventsData,m_nInputEventsDataSize);
+      XMFS::readBuf(pfh,m_pcInputEventsData,m_nInputEventsDataSize);
     }
     
     /* Set up input stream */
@@ -430,7 +516,7 @@ void Replay::openReplay_1(FileHandle *pfh, bool bDisplayInformation, int nVersio
   }
   
   /* Read chunks */
-  unsigned int nNumChunks = FS::readInt_LE(pfh);
+  unsigned int nNumChunks = XMFS::readInt_LE(pfh);
   if(bDisplayInformation) {
     printf("%-30s: %i\n", "Number of chunks", nNumChunks);
   }
@@ -446,7 +532,7 @@ void Replay::openReplay_1(FileHandle *pfh, bool bDisplayInformation, int nVersio
     }  
   
     ReplayStateChunk* Chunk = new ReplayStateChunk();
-    Chunk->nNumStates = FS::readInt_LE(pfh);
+    Chunk->nNumStates = XMFS::readInt_LE(pfh);
 
     if(bDisplayInformation) {
       printf("   %-27s: %i\n", "Number of states", Chunk->nNumStates);
@@ -455,20 +541,20 @@ void Replay::openReplay_1(FileHandle *pfh, bool bDisplayInformation, int nVersio
     Chunk->pcChunkData = new char [Chunk->nNumStates * m_nStateSize];
         
     /* Compressed or not compressed? */
-    if(FS::readBool(pfh)) {
+    if(XMFS::readBool(pfh)) {
       if(bDisplayInformation) {
 	printf("   %-27s: %s\n", "Compressed data", "true");
       } 
 
       /* Compressed! - read compressed size */
-      int nCompressedSize = FS::readInt_LE(pfh);
+      int nCompressedSize = XMFS::readInt_LE(pfh);
       if(bDisplayInformation) {
 	printf("   %-27s: %i\n", "Compressed states size", nCompressedSize);
       }
 
       /* Read compressed data */
       unsigned char *pcCompressed = new unsigned char [nCompressedSize];
-      FS::readBuf(pfh,(char *)pcCompressed,nCompressedSize);
+      XMFS::readBuf(pfh,(char *)pcCompressed,nCompressedSize);
          
       /* Uncompress it */           
       uLongf nDestLen = Chunk->nNumStates * m_nStateSize;
@@ -490,7 +576,7 @@ void Replay::openReplay_1(FileHandle *pfh, bool bDisplayInformation, int nVersio
       } 
       
       /* Not compressed! */
-      FS::readBuf(pfh,Chunk->pcChunkData,m_nStateSize*Chunk->nNumStates);
+      XMFS::readBuf(pfh,Chunk->pcChunkData,m_nStateSize*Chunk->nNumStates);
     }
         
     m_Chunks.push_back(Chunk);
@@ -499,13 +585,13 @@ void Replay::openReplay_1(FileHandle *pfh, bool bDisplayInformation, int nVersio
 
   std::string Replay::openReplay(const std::string &FileName, std::string &Player, bool bDisplayInformation) {
     /* Try opening as if it is a full path */
-    FileHandle *pfh = FS::openIFile(FileName, true);
+    FileHandle *pfh = XMFS::openIFile(FDT_DATA, FileName, true);
     if(pfh == NULL) {
       /* Open file for input */
-      pfh = FS::openIFile(std::string("Replays/") + FileName);
+      pfh = XMFS::openIFile(FDT_DATA, std::string("Replays/") + FileName);
       if(pfh == NULL) {
         /* Try adding a .rpl extension */
-        pfh = FS::openIFile(std::string("Replays/") + FileName + std::string(".rpl"));
+        pfh = XMFS::openIFile(FDT_DATA, std::string("Replays/") + FileName + std::string(".rpl"));
         if(pfh == NULL) {    
           LogWarning("Failed to open replay file for input: %s",(std::string("Replays/") + FileName).c_str());
           throw Exception("Unable to open the replay");
@@ -514,7 +600,7 @@ void Replay::openReplay_1(FileHandle *pfh, bool bDisplayInformation, int nVersio
     }
 
     /* Read header */
-    int nVersion = FS::readByte(pfh); 
+    int nVersion = XMFS::readByte(pfh); 
     if(bDisplayInformation) {
       printf("%-30s: %i\n", "Replay file version", nVersion);
     }
@@ -527,22 +613,22 @@ void Replay::openReplay_1(FileHandle *pfh, bool bDisplayInformation, int nVersio
       try {
 	openReplay_1(pfh, bDisplayInformation, nVersion);
       } catch(Exception &e) {
-	FS::closeFile(pfh);
+	XMFS::closeFile(pfh);
 	throw e;
       }
       break;
 
-    case 2:
+    case 3:
       try {
-	openReplay_2(pfh, bDisplayInformation);
+	openReplay_3(pfh, bDisplayInformation);
       } catch(Exception &e) {
-	FS::closeFile(pfh);
+	XMFS::closeFile(pfh);
 	throw e;
       }
       break;
 
     default:
-      FS::closeFile(pfh);
+      XMFS::closeFile(pfh);
       LogWarning("Unsupported replay file version (%d): %s",nVersion,(std::string("Replays/") + FileName).c_str());
       throw Exception("Unable to open the replay (unsupported version)");
 
@@ -550,7 +636,7 @@ void Replay::openReplay_1(FileHandle *pfh, bool bDisplayInformation, int nVersio
     }
 
     /* Clean up */
-    FS::closeFile(pfh);
+    XMFS::closeFile(pfh);
     
     Player = m_PlayerName;
 
@@ -572,18 +658,127 @@ void Replay::openReplay_1(FileHandle *pfh, bool bDisplayInformation, int nVersio
     return m_LevelID;
   }
 
-  void Replay::storeBlocks(const std::vector<Block *>& i_blocks) {
-     for(unsigned int i=0; i<i_blocks.size(); i++) {
-       if(i_blocks[i]->isPhysics() == true) {
-	 /* store this block in the .rpl if the position changed */
-	 // 
-	 // i_blocks[i]->Id();
-	 // i_blocks[i]->DynamicPosition();
-	 // i_blocks[i]->DynamicRotation();
-       }
-     }
+bool Replay::isPhysicBlockToSave(Block* i_block, int i_time, const std::vector<rmblockState>& i_states,
+				 const std::vector<Biker*>& i_players, bool i_forceSaveAll) {
+  // the block must be physic
+  if(i_block->isPhysics() == false) {
+    return false;
   }
 
+  if(i_forceSaveAll) {
+    return true;
+  }
+
+  // the block must be saved the first time
+  if(i_states.size() == 0) {
+    return true;
+  }
+
+  // store only on changes and if timediff is enough
+  const rmblockState* v_previousState = &(i_states[i_states.size()-1]);
+
+  // main case : do not save under the RMOVINGBLOCK_MIN_DIFFTIME frequency
+  if(i_time < v_previousState->time + RMOVINGBLOCK_MIN_FARDIFFTIME1) {
+    return false;
+  }
+
+  // at this step the decision is taken according to the distance to the players
+  // compute the minimum distance between the block and the players
+  float v_playerDist = 0.0;
+  Vector2f plen;
+  if(i_players.size() > 0) {
+    plen = i_players[0]->getState()->CenterP - i_block->DynamicPosition();
+    v_playerDist = plen.length();
+  }
+  for(unsigned int p=1; p<i_players.size(); p++) {
+    plen = i_players[p]->getState()->CenterP - i_block->DynamicPosition();
+    if(plen.length() < v_playerDist) {
+      v_playerDist = plen.length();
+    }
+  }
+
+  // at this step, the block is stored only if it has moved
+  bool v_hasNormalMove =
+    v_previousState->position.x - i_block->DynamicPosition().x >   RMOVINGBLOCK_MIN_DIFFMOVE      ||
+    v_previousState->position.x - i_block->DynamicPosition().x < -(RMOVINGBLOCK_MIN_DIFFMOVE)     ||
+    v_previousState->position.y - i_block->DynamicPosition().y >   RMOVINGBLOCK_MIN_DIFFMOVE      ||
+    v_previousState->position.y - i_block->DynamicPosition().y < -(RMOVINGBLOCK_MIN_DIFFMOVE)     ||
+    v_previousState->rotation   - i_block->DynamicRotation()   >   RMOVINGBLOCK_MIN_DIFFROTATION  ||
+    v_previousState->rotation   - i_block->DynamicRotation()   < -(RMOVINGBLOCK_MIN_DIFFROTATION);
+
+  if(v_hasNormalMove) {
+    // save if the block is near the player
+    //
+    /* test moved higher whie it's the most current
+    if(i_time <= v_previousState->time + RMOVINGBLOCK_MIN_FARDIFFTIME1) {
+      return false;
+    }
+    */
+
+    // save if the block if far from the player, but less frequently
+    if(v_playerDist >= RMOVINGBLOCK_MIN_FARPLAYERLENGTH1 &&
+       i_time <= v_previousState->time + RMOVINGBLOCK_MIN_FARDIFFTIME2) {
+      return false;
+    }
+
+    // save if the block if very far from the player, but really less frequently
+    if(v_playerDist >= RMOVINGBLOCK_MIN_FARPLAYERLENGTH2 &&
+       i_time <= v_previousState->time + RMOVINGBLOCK_MIN_FARDIFFTIME3) {
+      return false;
+    }
+
+    return true;
+  }
+
+  // at this step normal moves are saved, but i suggest to save last block position when they have not moved from a long time
+  if(i_time <= v_previousState->time + MOVINGBLOCK_MIN_LONGDIFFTIME) {
+    return false;
+  }
+
+  bool v_hasMinimalMove =
+    v_previousState->position.x - i_block->DynamicPosition().x >   RMOVINGBLOCK_MIN_LONGDIFFMOVE      ||
+    v_previousState->position.x - i_block->DynamicPosition().x < -(RMOVINGBLOCK_MIN_LONGDIFFMOVE)     ||
+    v_previousState->position.y - i_block->DynamicPosition().y >   RMOVINGBLOCK_MIN_LONGDIFFMOVE      ||
+    v_previousState->position.y - i_block->DynamicPosition().y < -(RMOVINGBLOCK_MIN_LONGDIFFMOVE)     ||
+    v_previousState->rotation   - i_block->DynamicRotation()   >   RMOVINGBLOCK_MIN_LONGDIFFROTATION  ||
+    v_previousState->rotation   - i_block->DynamicRotation()   < -(RMOVINGBLOCK_MIN_LONGDIFFROTATION);
+  // save if the block has moved only a little from a long time
+  if(v_hasMinimalMove) {
+    return true;
+  }
+
+  return false;
+}
+
+void Replay::storeBlocks(int i_time, const std::vector<Block *>& i_blocks, const std::vector<Biker*>& i_players, bool i_forceSaveAll) {
+
+  // create a structure for each block (initialisation)
+  if(m_movingBlocksForSaving.size() < i_blocks.size()) {
+    for(unsigned int i=m_movingBlocksForSaving.size(); i<i_blocks.size(); i++) {
+      rmblock rb;
+      rb.name = i_blocks[i]->Id();
+      rb.lastMovingTime = i_time;
+      m_movingBlocksForSaving.push_back(rb);
+    }
+  }
+
+  // store the delta
+  for(unsigned int i=0; i<i_blocks.size(); i++) {
+    /* store this block in the .rpl if the position changed */
+    if(isPhysicBlockToSave(i_blocks[i], i_time, m_movingBlocksForSaving[i].states, i_players, i_forceSaveAll)) {
+      rmblockState rmbs;
+      rmbs.time = i_time;
+      rmbs.position = i_blocks[i]->DynamicPosition();
+      rmbs.rotation = i_blocks[i]->DynamicRotation();
+      m_movingBlocksForSaving[i].states.push_back(rmbs);
+    }
+  }
+}
+
+std::vector<rmtime>* Replay::getMovingBlocks() {
+  return &m_movingBlocksForLoading;
+}
+
   void Replay::storeState(const SerializedBikeState& state) {
     char *addr;
 
@@ -620,6 +815,31 @@ void Replay::openReplay_1(FileHandle *pfh, bool bDisplayInformation, int nVersio
     return nextState(1);
   }
 
+void Replay::rewindAtPosition(ReplayPosition i_rs) {
+  m_bEndOfFile = i_rs.bEndOfFile;
+  m_nCurChunk = i_rs.nCurChunk;
+  m_nCurState = i_rs.nCurState;
+}
+
+void Replay::rewindAtBeginning() {
+  ReplayPosition v_rs;
+  v_rs.bEndOfFile = false;
+  v_rs.nCurChunk = 0;
+  v_rs.nCurState = 0.0;
+
+  rewindAtPosition(v_rs);
+}
+
+ReplayPosition Replay::getReplayPosition() const {
+  ReplayPosition v_res;
+
+  v_res.bEndOfFile = m_bEndOfFile;
+  v_res.nCurChunk  = m_nCurChunk;
+  v_res.nCurState  = m_nCurState;
+
+  return v_res;
+}
+
 bool Replay::nextState(int p_frames) {
   m_bEndOfFile = false;
 
@@ -718,7 +938,7 @@ bool Replay::nextState(int p_frames) {
   }
 
   void Replay::deleteReplay(std::string ReplayName) {
-    FS::deleteFile(std::string("Replays/") + ReplayName + std::string(".rpl"));
+    XMFS::deleteFile(FDT_DATA, std::string("Replays/") + ReplayName + std::string(".rpl"));
   }
 
   ReplayInfo* Replay::getReplayInfos(const std::string p_ReplayName) {
@@ -726,17 +946,17 @@ bool Replay::nextState(int p_frames) {
        want to check this again */
 
       /* Try opening it */
-      FileHandle *pfh = FS::openIFile("Replays/" + p_ReplayName + ".rpl");
+      FileHandle *pfh = XMFS::openIFile(FDT_DATA, "Replays/" + p_ReplayName + ".rpl");
       if(pfh == NULL) {
-	pfh = FS::openIFile(p_ReplayName, true);
+	pfh = XMFS::openIFile(FDT_DATA, p_ReplayName, true);
 	if(pfh == NULL) {
 	  return NULL;
 	}
       }
       
-      int nVersion = FS::readByte(pfh);
-      if(nVersion != 0 && nVersion != 1 && nVersion != 2) {
-	FS::closeFile(pfh);
+      int nVersion = XMFS::readByte(pfh);
+      if(nVersion != 0 && nVersion != 1 && nVersion != 3) {
+	XMFS::closeFile(pfh);
 	return NULL;
       }
       
@@ -746,30 +966,18 @@ bool Replay::nextState(int p_frames) {
       bool bFinished = true;
       int finishTime = 0;
 
-      switch(nVersion) {
-      case 0:
-      case 1:
-	if(FS::readInt_LE(pfh) != 0x12345678) {   
-	  FS::closeFile(pfh);
-	  return NULL;
-	}
-	
-	LevelID       =    FS::readString(pfh);
-	Player        =    FS::readString(pfh);
-	/* fFrameRate = */ FS::readFloat_LE(pfh);
-	/* nStateSize = */ FS::readInt_LE(pfh);
-	bFinished     =    FS::readBool(pfh);
-	finishTime    = GameApp::floatToTime(FS::readFloat_LE(pfh));
-	break;
-
-      case 2:
-	LevelID       =    FS::readString(pfh);
-	Player        =    FS::readString(pfh);
-	bFinished     =    FS::readBool(pfh);
-	finishTime    = GameApp::floatToTime(FS::readFloat_LE(pfh));
-	break;
+      if(XMFS::readInt_LE(pfh) != 0x12345678) {   
+	XMFS::closeFile(pfh);
+	return NULL;
       }
 	
+      LevelID       =    XMFS::readString(pfh);
+      Player        =    XMFS::readString(pfh);
+      /* fFrameRate = */ XMFS::readFloat_LE(pfh);
+      /* nStateSize = */ XMFS::readInt_LE(pfh);
+      bFinished     =    XMFS::readBool(pfh);
+      finishTime    = GameApp::floatToTime(XMFS::readFloat_LE(pfh));
+	
       /* Set members */
       pRpl->Level = LevelID;
       pRpl->Name = p_ReplayName;
@@ -782,7 +990,7 @@ bool Replay::nextState(int p_frames) {
 	pRpl->finishTime = -1;                
       }
 
-      FS::closeFile(pfh); 
+      XMFS::closeFile(pfh); 
       return pRpl;
   }
 
