@@ -33,7 +33,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 	void UserConfig::loadFile(void) {
 	  XMLDocument ConfigDoc;
 	  
-	  ConfigDoc.readFromFile("config.dat");
+	  ConfigDoc.readFromFile(FDT_CONFIG, XM_CONFIGFILE);
 	  TiXmlDocument *pConfigData = ConfigDoc.getLowLevelAccess();
 	  if(pConfigData != NULL) {
 	    TiXmlElement *pUserConfigElem = pConfigData->FirstChildElement("userconfig");
@@ -51,10 +51,10 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 	      }
 	    }
 	    else 
-	      LogWarning("no configuration in 'config.dat'");
+	      LogWarning("no configuration in " XM_CONFIGFILE);
 	  }
 	  else
-	    LogWarning("failed to load or parse user configuration 'config.dat'");
+	    LogWarning("failed to load or parse user configuration " XM_CONFIGFILE);
 	}
 
   /*===========================================================================
@@ -62,26 +62,26 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   ===========================================================================*/
 	void UserConfig::saveFile(void) {
 	  /* Save configuration */
-	  FileHandle *pfh = FS::openOFile("config.dat");
+	  FileHandle *pfh = XMFS::openOFile(FDT_CONFIG, XM_CONFIGFILE);
 	  if(pfh != NULL) {
-	    FS::writeLine(pfh,"<?xml version=\"1.0\" encoding=\"utf-8\"?>");
-	    FS::writeLine(pfh,"");
-	    FS::writeLine(pfh,"<userconfig>");
+	    XMFS::writeLine(pfh,"<?xml version=\"1.0\" encoding=\"utf-8\"?>");
+	    XMFS::writeLine(pfh,"");
+	    XMFS::writeLine(pfh,"<userconfig>");
 	    
 	    for(unsigned int i=0;i<m_Vars.size();i++) {
 	      char cBuf[256];
 	      snprintf(cBuf, 256, "\t<var name=%-25s value=%s />",
 								 ("\"" + XML::str2xmlstr(m_Vars[i]->Name)  + "\"").c_str(),
 								 ("\"" + XML::str2xmlstr(m_Vars[i]->Value) + "\"").c_str());
-	      FS::writeLine(pfh,cBuf);
+	      XMFS::writeLine(pfh,cBuf);
 	    }
 	  
-	    FS::writeLine(pfh,"</userconfig>");
-	    FS::writeLine(pfh,"");
-	    FS::closeFile(pfh);
+	    XMFS::writeLine(pfh,"</userconfig>");
+	    XMFS::writeLine(pfh,"");
+	    XMFS::closeFile(pfh);
 	  }
 	  else
-	    LogWarning("failed to save user configuration 'config.dat'");
+	    LogWarning("failed to save user configuration " XM_CONFIGFILE);
 	}
 
   /*===========================================================================
