@@ -29,25 +29,25 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   /*===========================================================================
   Load XML document from disk
   ===========================================================================*/
-  void XMLDocument::readFromFile(std::string File,unsigned long *pnCRC32, bool i_includeCurrentDir) {
+void XMLDocument::readFromFile(FileDataType i_fdt, std::string File,unsigned long *pnCRC32, bool i_includeCurrentDir) {
     /* Clean up if a doc already is loaded */
     if(m_pXML) delete m_pXML;
     
     /* Load */
     std::string Line,Doc = "";    
     
-    FileHandle *pfh = FS::openIFile(File, i_includeCurrentDir);
+    FileHandle *pfh = XMFS::openIFile(i_fdt, File, i_includeCurrentDir);
     if(pfh==NULL) return;
 
     m_pXML = new TiXmlDocument;
     
-    while(FS::readNextLine(pfh,Line)) {
+    while(XMFS::readNextLine(pfh,Line)) {
       if(Line.length() > 0) {
         Doc.append(Line);
         Doc.append("\n");
       }
     }    
-    FS::closeFile(pfh);
+    XMFS::closeFile(pfh);
     
     /* Are we just going to need the CRC? */
     if(pnCRC32 != NULL) {
@@ -65,13 +65,13 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   /*===========================================================================
   Save XML document from disk
   ===========================================================================*/
-  void XMLDocument::writeToFile(std::string File) {
+  void XMLDocument::writeToFile(FileDataType i_fdt, std::string File) {
     /* Anything? */
     if(!m_pXML) return;
-    FileHandle *pfh = FS::openOFile(File);
+    FileHandle *pfh = XMFS::openOFile(i_fdt, File);
     if(pfh==NULL || pfh->Type!=FHT_STDIO) return;
     m_pXML->Print(pfh->fp);
-    FS::closeFile(pfh);
+    XMFS::closeFile(pfh);
   }
 
   std::string XML::str2xmlstr(std::string str) {
