@@ -22,13 +22,23 @@
 #include "helpers/FileCompression.h"
 #include "db/xmDatabase.h"
 #include "md5sum/md5file.h"
+#include <sstream>
 
 #define DEFAULT_WWW_MSGFILE "wwwMsg.xml"
 
-void WebRoom::downloadReplay(const std::string& i_url) {
-  std::string i_rplFilename = FS::getReplaysDir()
+bool WebRoom::downloadReplayExists(const std::string& i_url) {
+  std::string i_rplFilename = XMFS::getUserReplaysDir()
     + "/" 
-    + FS::getFileBaseName(i_url) 
+    + XMFS::getFileBaseName(i_url) 
+    + ".rpl";
+  
+  return XMFS::fileExists(FDT_DATA, i_rplFilename);
+}
+
+void WebRoom::downloadReplay(const std::string& i_url) {
+  std::string i_rplFilename = XMFS::getUserReplaysDir()
+    + "/" 
+    + XMFS::getFileBaseName(i_url) 
     + ".rpl";
 
   /* download xml file */
@@ -45,7 +55,7 @@ void WebRoom::downloadReplay(const std::string& i_url) {
 
 WebRoom::WebRoom(WWWAppInterface* p_WebRoomApp)
 {
-  m_userFilename_prefix = FS::getUserDir()
+  m_userFilename_prefix = XMFS::getUserDir(FDT_CACHE)
     + "/" 
     + DEFAULT_WEBHIGHSCORES_FILENAME_PREFIX;
   m_webhighscores_url = DEFAULT_WEBHIGHSCORES_URL;
@@ -57,18 +67,12 @@ WebRoom::~WebRoom()
 {
 }
 
-std::string WebRoom::getRoomId() const
-{
-  return m_roomId;
+void WebRoom::setProxy(const ProxySettings* pProxySettings) {
+  m_proxy_settings    = pProxySettings;
 }
 
-void WebRoom::setWebsiteInfos(const std::string& i_id_room,
-			      const std::string& i_webhighscores_url,
-			      const ProxySettings* pProxySettings)
-{
+void WebRoom::setHighscoresUrl(const std::string& i_webhighscores_url) {
   m_webhighscores_url = i_webhighscores_url;
-  m_roomId            = i_id_room;
-  m_proxy_settings    = pProxySettings;
 }
 
 /* use the id of the room so that if several people share the same room, it doesn't need to recheck if md5 is ok */
@@ -90,7 +94,7 @@ void WebRoom::update(const std::string& i_id_room)
 
 void WebRoom::upgrade(const std::string& i_id_room, xmDatabase *i_db)
 {
-  i_db->webhighscores_updateDB(m_userFilename_prefix + i_id_room + ".xml", m_webhighscores_url);
+  i_db->webhighscores_updateDB(FDT_CACHE, m_userFilename_prefix + i_id_room + ".xml", m_webhighscores_url);
 }
 
 size_t FSWeb::writeData(void *ptr, size_t size, size_t nmemb, FILE *stream)
@@ -139,8 +143,8 @@ void FSWeb::downloadFileBz2UsingMd5(const std::string &p_local_file,
   bool require_dwd = true;
 
   try {
-    if(FS::isFileReadable(p_local_file) == true) {
-      std::string v_md5Local  = FS::md5sum(p_local_file);
+    if(XMFS::isFileReadable(FDT_CACHE, p_local_file) == true) {
+      std::string v_md5Local  = XMFS::md5sum(FDT_CACHE, p_local_file);
       if(v_md5Local != "") {
 	std::string v_md5File = p_local_file + ".md5";
   
@@ -309,7 +313,7 @@ void FSWeb::uploadReplay(const std::string& p_replayFilename,
 
   FILE *v_destinationFile;
   std::string v_local_file;
-  v_local_file = FS::getUserDir() + "/" + DEFAULT_WWW_MSGFILE;
+  v_local_file = XMFS::getUserDir(FDT_CACHE) + "/" + DEFAULT_WWW_MSGFILE;
 
   struct curl_httppost *v_post, *v_last;
 
@@ -467,7 +471,7 @@ void FSWeb::sendVote(const std::string& p_id_level,
 
   FILE *v_destinationFile;
   std::string v_local_file;
-  v_local_file = FS::getUserDir() + "/" + DEFAULT_WWW_MSGFILE;
+  v_local_file = XMFS::getUserDir(FDT_CACHE) + "/" + DEFAULT_WWW_MSGFILE;
 
   struct curl_httppost *v_post, *v_last;
 
@@ -553,7 +557,7 @@ void FSWeb::uploadAnalyseMsg(const std::string& p_key,
   TiXmlNode * pChild;
   
   /* open the file */
-  v_Xml.readFromFile(p_filename);   
+  v_Xml.readFromFile(FDT_CACHE, p_filename);   
   v_XmlData = v_Xml.getLowLevelAccess();
   
   if(v_XmlData == NULL) {
@@ -704,7 +708,7 @@ void WebLevels::setWebsiteInfos(const std::string &p_url, const ProxySettings* p
 }
 
 std::string WebLevels::getXmlFileName() {
-  return FS::getUserDir() + "/" + DEFAULT_WEBLEVELS_FILENAME;
+  return XMFS::getUserDir(FDT_CACHE) + "/" + DEFAULT_WEBLEVELS_FILENAME;
 }
 
 void WebLevels::downloadXml()
@@ -723,25 +727,25 @@ void WebLevels::downloadXml()
 }
 
 std::string WebLevels::getDestinationDir() {
-  return FS::getLevelsDir() + "/" + DEFAULT_WEBLEVELS_DIR;
+  return XMFS::getUserLevelsDir() + "/" + DEFAULT_WEBLEVELS_DIR;
 }
 
 void WebLevels::createDestinationDirIfRequired() {
   std::string v_destination_dir = getDestinationDir();
 
-  if(FS::isDir(v_destination_dir) == false) {
+  if(XMFS::isDir(v_destination_dir) == false) {
     /* no mkdir() with microsoft C */
-    FS::mkArborescence(v_destination_dir + "/file");
+    XMFS::mkArborescence(v_destination_dir + "/file");
   }
 }
 
 std::string WebLevels::getDestinationFile(std::string p_url) {
-  return getDestinationDir() + "/" + FS::getFileBaseName(p_url) + ".lvl";
+  return getDestinationDir() + "/" + XMFS::getFileBaseName(p_url) + ".lvl";
 }
 
 void WebLevels::update(xmDatabase *i_db) {
   downloadXml();
-  i_db->weblevels_updateDB(getXmlFileName());
+  i_db->weblevels_updateDB(FDT_CACHE, getXmlFileName());
 }
 
 int FSWeb::f_curl_progress_callback_upload(void *clientp,
@@ -811,7 +815,7 @@ int WebLevels::nbLevelsToGet(xmDatabase *i_db) const {
   return i_db->levels_nbLevelsToDownload();
 }
 
-void WebLevels::upgrade(xmDatabase *i_db) {
+void WebLevels::upgrade(xmDatabase *i_db, int i_nb_levels) {
   char **v_result = NULL;
   unsigned int nrow;
   std::string v_levelId;
@@ -828,10 +832,17 @@ void WebLevels::upgrade(xmDatabase *i_db) {
 
   createDestinationDirIfRequired();
 
+  std::string v_sqlLimit;
+  std::ostringstream v_limit;
+  if(i_nb_levels != -1) {
+    v_limit << i_nb_levels;
+    v_sqlLimit = " LIMIT " + v_limit.str();
+  }
+
   v_result = i_db->readDB("SELECT a.id_level, a.name, a.fileUrl, b.filepath "
 			  "FROM weblevels AS a "
 			  "LEFT OUTER JOIN levels AS b ON a.id_level=b.id_level "
-			  "WHERE b.id_level IS NULL OR a.checkSum <> b.checkSum;",
+			  "WHERE b.id_level IS NULL OR a.checkSum <> b.checkSum order by creationDate DESC" + v_sqlLimit + ";",
 			  nrow);
   v_nb_levels_to_download = nrow;
   v_data.v_WebApp = m_WebLevelApp;
@@ -873,7 +884,7 @@ void WebLevels::upgrade(xmDatabase *i_db) {
 	std::string v_destFile;
 	
 	if(v_isAnUpdate) {
-	  if(FS::isInUserDir(v_filePath)) {
+	  if(XMFS::isInUserDir(FDT_DATA, v_filePath)) {
 	    v_destFile = v_filePath;
 	  } else {
 	    v_destFile = WebLevels::getDestinationFile(v_urlFile);
@@ -971,7 +982,7 @@ void WebThemes::updateTheme(xmDatabase* i_pDb, const std::string& i_id_theme, WW
     
     /* the destination file must be in the user dir */
     if(v_filePath != "") {
-      if(FS::isInUserDir(v_filePath)) {
+      if(XMFS::isInUserDir(FDT_DATA, v_filePath)) {
 	v_destinationFileXML = v_filePath;
       }
     }
@@ -979,8 +990,8 @@ void WebThemes::updateTheme(xmDatabase* i_pDb, const std::string& i_id_theme, WW
     if(v_destinationFileXML == "") {
       /* determine destination file */
       v_destinationFileXML = 
-	FS::getUserDir() + "/" + THEMES_DIRECTORY + "/" + 
-	FS::getFileBaseName(v_fileUrl) + ".xml";
+	XMFS::getUserDir(FDT_DATA) + "/" + THEMES_DIRECTORY + "/" + 
+	XMFS::getFileBaseName(v_fileUrl) + ".xml";
     }
   
     v_themeFile = v_destinationFileXML;
@@ -988,7 +999,7 @@ void WebThemes::updateTheme(xmDatabase* i_pDb, const std::string& i_id_theme, WW
     /* download the theme file */
     v_data.v_nb_files_performed   = 0;
     v_data.v_nb_files_to_download = 1;
-    FS::mkArborescence(v_destinationFileXML);
+    XMFS::mkArborescence(v_destinationFileXML);
     v_destinationFileXML_tmp = v_destinationFileXML + ".tmp";
     FSWeb::downloadFileBz2(v_destinationFileXML_tmp, v_fileUrl, FSWeb::f_curl_progress_callback_download, &v_data, XMSession::instance()->proxySettings());
     
@@ -1002,16 +1013,16 @@ void WebThemes::updateTheme(xmDatabase* i_pDb, const std::string& i_id_theme, WW
     /* download all the files required */
     Theme *v_theme = Theme::instance();
     std::vector<ThemeFile> *v_required_files;
-    v_theme->load(v_destinationFileXML_tmp);
+    v_theme->load(FDT_CACHE, v_destinationFileXML_tmp);
     v_required_files = v_theme->getRequiredFiles();
 
     // all files must be checked for md5sum
     int v_nb_files_to_download = 0;
     for(unsigned int i=0; i<v_required_files->size(); i++) {
-      if(FS::fileExists((*v_required_files)[i].filepath) == false) {
+      if(XMFS::fileExists(FDT_DATA, (*v_required_files)[i].filepath) == false) {
 	v_nb_files_to_download++;
       } else {
-	v_md5Local = FS::md5sum((*v_required_files)[i].filepath);
+	v_md5Local = XMFS::md5sum(FDT_DATA, (*v_required_files)[i].filepath);
 	v_md5Dist  = (*v_required_files)[i].filemd5;
 	if(v_md5Local != v_md5Dist && v_md5Dist != "") {
 	  v_nb_files_to_download++;
@@ -1034,21 +1045,21 @@ void WebThemes::updateTheme(xmDatabase* i_pDb, const std::string& i_id_theme, WW
 
       while(i<v_required_files->size() && i_askThreadToEnd == false) {
 	// download v_required_files[i]     
-	v_destinationFile = FS::getUserDir() + std::string("/") + (*v_required_files)[i].filepath;
+	v_destinationFile = XMFS::getUserDir(FDT_DATA) + std::string("/") + (*v_required_files)[i].filepath;
 	v_sourceFile = XMSession::instance()->webThemesURLBase() + std::string("/") + (*v_required_files)[i].filepath;
 	
 	/* check md5 sums */
 	v_md5Local = v_md5Dist = "";
-	if(FS::fileExists((*v_required_files)[i].filepath) == true) {
-	  v_md5Local = FS::md5sum((*v_required_files)[i].filepath);
+	if(XMFS::fileExists(FDT_DATA, (*v_required_files)[i].filepath) == true) {
+	  v_md5Local = XMFS::md5sum(FDT_DATA, (*v_required_files)[i].filepath);
 	  v_md5Dist  = (*v_required_files)[i].filemd5;
 	}
 	
 	/* if v_md5Dist == "", don't download ; it's a manually adding */
-	if(FS::fileExists((*v_required_files)[i].filepath) == false || (v_md5Local != v_md5Dist && v_md5Dist != "")) {
+	if(XMFS::fileExists(FDT_DATA, (*v_required_files)[i].filepath) == false || (v_md5Local != v_md5Dist && v_md5Dist != "")) {
 	  v_data.v_nb_files_performed = v_nb_files_performed;
 
-	  if(FS::fileExists((*v_required_files)[i].filepath) == false) {
+	  if(XMFS::fileExists(FDT_DATA, (*v_required_files)[i].filepath) == false) {
 	    LogInfo("The file %s must be downloaded because it is missing on the system", (*v_required_files)[i].filepath.c_str());
 	  } else {
 	    if(v_md5Local != v_md5Dist && v_md5Dist != "") {
@@ -1062,7 +1073,7 @@ void WebThemes::updateTheme(xmDatabase* i_pDb, const std::string& i_id_theme, WW
 	    i_WebLevelApp->setBeingDownloadedInformation((*v_required_files)[i].filepath);
 	  }
 
-	  FS::mkArborescence(v_destinationFile);
+	  XMFS::mkArborescence(v_destinationFile);
 	  
 	  FSWeb::downloadFile(v_destinationFile, v_sourceFile, FSWeb::f_curl_progress_callback_download, &v_data,
 			      XMSession::instance()->proxySettings());
@@ -1119,7 +1130,7 @@ void WebThemes::updateTheme(xmDatabase* i_pDb, const std::string& i_id_theme, WW
 }
 
 void WebThemes::updateThemeList(xmDatabase* i_pDb, WWWAppInterface* i_WebLevelApp) {
-  std::string v_destinationFile = FS::getUserDir() + "/" + DEFAULT_WEBTHEMES_FILENAME;
+  std::string v_destinationFile = XMFS::getUserDir(FDT_CACHE) + "/" + DEFAULT_WEBTHEMES_FILENAME;
   f_curl_download_data v_data;
   
   if(i_WebLevelApp != NULL) {
@@ -1134,7 +1145,7 @@ void WebThemes::updateThemeList(xmDatabase* i_pDb, WWWAppInterface* i_WebLevelAp
   v_data.v_nb_files_to_download = 1;
   FSWeb::downloadFileBz2UsingMd5(v_destinationFile, XMSession::instance()->webThemesURL(),
 				 FSWeb::f_curl_progress_callback_download, &v_data, XMSession::instance()->proxySettings());
-  i_pDb->webthemes_updateDB(v_destinationFile);
+  i_pDb->webthemes_updateDB(FDT_CACHE, v_destinationFile);
 }
 
 bool WebThemes::isUpdatable(xmDatabase* i_pDb, const std::string& i_id_theme) {
