@@ -38,6 +38,8 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "Theme.h"
 #include "XMSession.h"
 #include "XMBuild.h"
+#include "VFileIO_types.h"
+
 class ThemeChoice;
 
 #define DEFAULT_WEBHIGHSCORES_URL         "http://xmoto.tuxfamily.org/highscores.xml"
@@ -196,18 +198,18 @@ class WebRoom {
   void upgrade(const std::string& i_id_room, xmDatabase* i_db); /* throws exceptions */
 
   /* return NULL if no data found */
-  void setWebsiteInfos(const std::string& i_id_room,
-		       const std::string& i_webhighscores_url,
-		       const ProxySettings* pProxySettings);
-  std::string getRoomId() const;
+  void setProxy(const ProxySettings* pProxySettings);
+
+  // only required for update and upgrade
+  void setHighscoresUrl(const std::string& i_webhighscores_url);
 
   void downloadReplay(const std::string& i_url);
+  bool downloadReplayExists(const std::string& i_url);
 
  private:
   WWWAppInterface* m_WebRoomApp;
   std::string m_userFilename_prefix;
   std::string m_webhighscores_url;
-  std::string m_roomId;
   const ProxySettings* m_proxy_settings;
 };
 
@@ -220,7 +222,7 @@ class WebLevels {
   void update(xmDatabase* i_db); /* throws exceptions */
 
   /* download new levels */
-  void upgrade(xmDatabase* i_db); /* throws exceptions */
+  void upgrade(xmDatabase* i_db, int i_nb_levels = -1 /* -1 for all levels */); /* throws exceptions */
   
   /* Get names of new files downloaded OK */
   const std::vector<std::string> &getNewDownloadedLevels();
