@@ -22,6 +22,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "helpers/VExcept.h"
 #include "XMBuild.h"
 #include "VFileIO.h"
+#include "LevelsManager.h"
 #include <sstream>
 #include <stdlib.h>
 #include <cstdio>
@@ -53,6 +54,7 @@ XMArguments::XMArguments() {
   m_opt_levelID        = false;
   m_opt_levelFile      = false;
   m_opt_listLevels     = false;
+  m_opt_demo           = false;
   m_opt_profile        = false;
   m_opt_timedemo       = false;
   m_opt_testTheme      = false;
@@ -75,6 +77,7 @@ XMArguments::XMArguments() {
   m_opt_updateLevelsOnly        = false;
   m_opt_clientConnectAtStartup  = false;
   m_opt_adminMode               = false;
+  m_opt_buildQueries            = false;
 }
 
 void XMArguments::parse(int i_argc, char **i_argv) {
@@ -280,10 +283,12 @@ void XMArguments::parse(int i_argc, char **i_argv) {
       i++;
     } else if(v_opt == "--admin") { // hidden option to control website from the game ; keep undocumented
       m_opt_adminMode = true;
+    } else if(v_opt == "--buildQueries") { // hidden option to control website from the game ; keep undocumented
+      m_opt_buildQueries = true;
     } else {
       /* check if the parameter is a file */
       v_arg = i_argv[i];
-      std::string v_extension = FS::getFileExtension(v_arg);
+      std::string v_extension = XMFS::getFileExtension(v_arg);
       
       if(v_extension == "rpl") {        /* replay file */
 	m_opt_replay = true;
@@ -291,6 +296,9 @@ void XMArguments::parse(int i_argc, char **i_argv) {
       } else if(v_extension == "lvl") { /* level file */
 	m_opt_levelFile = true;
 	m_levelFile_file = v_arg;
+      } else if (v_extension == "dmo") {
+	m_opt_demo = true;
+	m_demo_file = v_arg;
       } else {
 	throw Exception("Invalid option \"" + v_opt + "\"");
       }
@@ -430,6 +438,14 @@ std::string XMArguments::getOpt_levelFile_file() const {
   return m_levelFile_file;
 }
 
+bool XMArguments::isOptDemo() const {
+  return m_opt_demo;
+}
+
+std::string XMArguments::getOpt_demo_file() const {
+  return m_demo_file;
+}
+
 bool XMArguments::isOptDebug() const {
   return m_opt_debug;
 }
@@ -590,6 +606,10 @@ bool XMArguments::isOptAdminMode() const {
   return m_opt_adminMode;
 }
 
+bool XMArguments::isOptBuildQueries() const {
+  return m_opt_buildQueries;
+}
+
 void XMArguments::help(const std::string& i_cmd) {
   printf("X-Moto %s\n", XMBuild::getVersionString().c_str());
   printf("usage:  %s [options]\n"
@@ -636,11 +656,12 @@ void XMArguments::help(const std::string& i_cmd) {
   printf("\t--defaultTheme THEME\n\t\tDefault theme for new profiles created.\n");
   printf("\t--noDBDirsCheck\n\t\tDon't check that system and user dirs changed at startup.\n");
   printf("\t--server\n\t\tRun X-Moto as a server only (no gui).\n");
-  printf("\t--updateServerOnly\n\t\tOnly update levels (no gui).\n");
+  printf("\t--updateLevelsOnly\n\t\tOnly update levels (no gui).\n");
   printf("\t--connectAtStartup\n\t\tConnect the client to the server at startup.\n");
   printf("\t-h, -?, -help, --help\n\t\tDisplay this message.\n");
   printf("\t--pack [BIN] [DIR]\n\t\tBuild the BIN package from directory DIR.\n");
   printf("\t--unpack [BIN] [DIR] [no_list]\n\t\tUnpack the BIN package into the dir DIR.\n");
+  printf("\t--buildQueries\n\t\tBuild sql queries into file " XM_SQLQUERIES_GEN_FILE ".\n");
   //If both sdlgfx and opengl are available give the user the posibility
   //to select the draw library
 #ifdef ENABLE_SDLGFX
