@@ -26,6 +26,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include <vector>
 #include "xmDatabaseUpdateInterface.h"
 #include "../helpers/MultiSingleton.h"
+#include "../VFileIO_types.h"
 
 class Level;
 
@@ -39,12 +40,13 @@ private:
 public:
   void init(const std::string& i_dbFileUTF8,
 	    const std::string& i_profile,
-	    const std::string& i_gameDir,
-	    const std::string& i_userDir,
+	    const std::string& i_gameDataDir,
+	    const std::string& i_userDataDir,
 	    const std::string& i_binPackCheckSum,
 	    bool i_dbDirsCheck,
 	    XmDatabaseUpdateInterface *i_interface = NULL);
   void init(const std::string& i_dbFileUTF8, bool i_readOnly = false);
+  int  getXmDbVersion();
 
   /* RULE:
      all write access must be done from class xmDatabase
@@ -100,14 +102,14 @@ public:
 		  const std::string& i_filepath, const std::string& i_name,
 		  const std::string& i_checkSum,
 		  const std::string& i_author, const std::string& i_description,
-		  const std::string& i_date, const std::string& i_packName, const std::string& i_packNum,
+		  const std::string& i_date,
 		  const std::string& i_music, bool i_isScripted, bool i_isPhysics,
 		  bool i_isToReload);
   void levels_update(const std::string& i_id_level,
 		     const std::string& i_filepath, const std::string& i_name,
 		     const std::string& i_checkSum,
 		     const std::string& i_author, const std::string& i_description,
-		     const std::string& i_date, const std::string& i_packName, const std::string& i_packNum,
+		     const std::string& i_date,
 		     const std::string& i_music, bool i_isScripted, bool i_isPhysics,
 		     bool i_isToReload);
   void levels_add_end();
@@ -146,11 +148,11 @@ public:
 			      const std::string& i_timeStamp, int i_finishTime);
 
   /* web */
-  std::string webhighscores_updateDB(const std::string& i_webhighscoresFile,
+  std::string webhighscores_updateDB(FileDataType i_fdt, const std::string& i_webhighscoresFile,
 				     const std::string& i_websource);
-  void weblevels_updateDB(const std::string& i_weblevelsFile);
-  void webrooms_updateDB(const std::string& i_webroomsFile);
-  void webthemes_updateDB(const std::string& i_webThemesFile);
+  void weblevels_updateDB(FileDataType i_fdt, const std::string& i_weblevelsFile);
+  void webrooms_updateDB(FileDataType i_fdt, const std::string& i_webroomsFile);
+  void webthemes_updateDB(FileDataType i_fdt, const std::string& i_webThemesFile);
   std::string webrooms_getName(const std::string& i_id_room);
   int webrooms_getHighscoreTime(const std::string& i_id_room,
 				const std::string& i_id_level); /* or a negativ value if not */
@@ -215,23 +217,23 @@ public:
   static void user_xm_userCrappy(sqlite3_context* i_context, int i_nArgs, sqlite3_value** i_values);
   static void user_xm_userChildrenCompliant(sqlite3_context* i_context, int i_nArgs, sqlite3_value** i_values);
   static void user_xm_replaceStart(sqlite3_context* i_context, int i_nArgs, sqlite3_value** i_values);
+  static void user_xm_profile(sqlite3_context* i_context, int i_nArgs, sqlite3_value** i_values);
 
   /* function used to synchronise with the last xmoto version */
-  int  getXmDbVersion();
   void upgradeXmDbToVersion(int i_fromVersion,
 			    const std::string& i_profile,
 			    XmDatabaseUpdateInterface *i_interface = NULL);
   void updateXmDbVersion(int i_newVersion);
-  std::string getXmDbGameDir();
-  std::string getXmDbUserDir();
+  std::string getXmDbGameDataDir();
+  std::string getXmDbUserDataDir();
   std::string getXmDbBinPackCheckSum();
-  void setXmDbGameDir(const std::string& i_gameDir);
-  void setXmDbUserDir(const std::string& i_userDir);
+  void setXmDbGameDataDir(const std::string& i_gameDataDir);
+  void setXmDbUserDataDir(const std::string& i_userDataDir);
   void setXmDbBinPackCheckSum(const std::string& i_binPackChecksum);
 
   /* update db directories without reanalysing all the files on the disk */
-  void updateXMDirectories(const std::string& i_oldGameDir, const std::string& i_newGameDir,
-			   const std::string& i_oldUserDir, const std::string& i_newUserDir);
+  void updateXMDirectories(const std::string& i_oldGameDataDir, const std::string& i_newGameDataDir,
+			   const std::string& i_oldUserDataDir, const std::string& i_newUserDataDir);
 
   std::string setXmDbSiteKey();
   
