@@ -58,7 +58,7 @@ void xmDatabase::levels_updateDB(const std::vector<Level *>&    i_levels,
   for(unsigned int i=0; i<i_levels.size(); i++) {
     levels_add(i_levels[i]->Id()      , i_levels[i]->FileName(), i_levels[i]->Name(),
 	       i_levels[i]->Checksum(), i_levels[i]->Author()  , i_levels[i]->Description(),
-	       i_levels[i]->Date()    , i_levels[i]->Pack()    , i_levels[i]->PackNum(),
+	       i_levels[i]->Date()    ,
 	       i_levels[i]->Music()   , i_levels[i]->isScripted(), i_levels[i]->isPhysics(),
 	       i_isToReload
 	       );
@@ -115,7 +115,7 @@ void xmDatabase::updateDB_favorite(const std::string& i_profile,
   const char *pc;
   std::string v_levelId;
   
-  v_favoriteLevelsXml.readFromFile("favoriteLevels.xml");
+  v_favoriteLevelsXml.readFromFile(FDT_DATA, "favoriteLevels.xml");
   v_favoriteLevelsXmlData = v_favoriteLevelsXml.getLowLevelAccess();
 
   if(v_favoriteLevelsXmlData == NULL) {
@@ -177,8 +177,6 @@ void xmDatabase::levels_add(const std::string& i_id_level,
 			    const std::string& i_author,
 			    const std::string& i_description,
 			    const std::string& i_date,
-			    const std::string& i_packName,
-			    const std::string& i_packNum,
 			    const std::string& i_music,
 			    bool i_isScripted, bool i_isPhysics,
 			    bool i_isToReload) {
@@ -187,7 +185,7 @@ void xmDatabase::levels_add(const std::string& i_id_level,
 
   simpleSql("INSERT INTO levels(id_level,"
 	    "filepath, name, checkSum, author, description, "
-	    "date_str, packName, packNum, music, isScripted, isPhysics, isToReload, loaded, loadingCacheFormatVersion) "
+	    "date_str, music, isScripted, isPhysics, isToReload, loaded, loadingCacheFormatVersion) "
 	    "VALUES(\"" + protectString(i_id_level) + "\", " +
 	    "\"" + protectString(i_filepath)        + "\", " +
 	    "\"" + protectString(i_name)            + "\", " +
@@ -195,8 +193,6 @@ void xmDatabase::levels_add(const std::string& i_id_level,
 	    "\"" + protectString(i_author)          + "\", " +
 	    "\"" + protectString(i_description)     + "\", " +
 	    "\"" + protectString(i_date)            + "\", " +
-	    "\"" + protectString(i_packName)        + "\", " +
-	    "\"" + protectString(i_packNum)         + "\", " +
 	    "\"" + protectString(i_music)           + "\", " +
 	    std::string(i_isScripted  ? "1" : "0")  + ", "   +
 	    std::string(i_isPhysics   ? "1" : "0")  + ", "   +
@@ -208,7 +204,7 @@ void xmDatabase::levels_update(const std::string& i_id_level,
 			       const std::string& i_filepath, const std::string& i_name,
 			       const std::string& i_checkSum,
 			       const std::string& i_author, const std::string& i_description,
-			       const std::string& i_date, const std::string& i_packName, const std::string& i_packNum,
+			       const std::string& i_date,
 			       const std::string& i_music, bool i_isScripted, bool i_isPhysics,
 			       bool i_isToReload) {
   std::ostringstream v_cacheFV;
@@ -220,9 +216,7 @@ void xmDatabase::levels_update(const std::string& i_id_level,
 	    protectString(i_checkSum) 	 + "\", author=\"" +
 	    protectString(i_author)   	 + "\", description=\"" +
 	    protectString(i_description) + "\", date_str=\"" +
-	    protectString(i_date)        + "\", packName=\"" +
-	    protectString(i_packName)    + "\", packNum=\"" +
-	    protectString(i_packNum)     + "\", music=\"" +
+	    protectString(i_date)        + "\", music=\"" +
 	    protectString(i_music)       + "\", isScripted=" +
 	    std::string(i_isScripted  ? "1" : "0") + ", isPhysics=" +
 	    std::string(i_isPhysics   ? "1" : "0") + ", isToReload=" +
@@ -235,11 +229,11 @@ void xmDatabase::levels_cleanNoWWWLevels() {
   char **v_result;
   unsigned int nrow;
   std::string v_name, v_filepath, v_id_level;
-  std::string v_savePath = FS::getUserDir() + std::string("/Trash/Levels");
+  std::string v_savePath = XMFS::getUserDir(FDT_DATA) + std::string("/Trash/Levels");
   std::string v_basename;
 
   // make directory for levels
-  FS::mkArborescenceDir(v_savePath);
+  XMFS::mkArborescenceDir(v_savePath);
 
   v_result = readDB("SELECT a.id_level, a.name, a.filepath "
 		    "FROM levels AS a LEFT OUTER JOIN weblevels AS b ON a.id_level=b.id_level "
@@ -250,13 +244,13 @@ void xmDatabase::levels_cleanNoWWWLevels() {
     v_name     = getResult(v_result, 3, i, 1);
     v_filepath = getResult(v_result, 3, i, 2);
 
-    if(FS::isInUserDir(v_filepath)) { // remove only files of the user dir
+    if(XMFS::isInUserDir(FDT_DATA, v_filepath)) { // remove only files of the user dir
       LogInfo("Removing level %s (%s)", v_name.c_str(), v_filepath.c_str());
       try {
 	simpleSql("DELETE FROM levels WHERE id_level=\"" + protectString(v_id_level) + "\";");
 
-	v_basename  = FS::getFileBaseName(v_filepath);
-	if(FS::renameUserFile(v_filepath, v_savePath + "/" + v_basename + ".lvl") == false) {
+	v_basename  = XMFS::getFileBaseName(v_filepath);
+	if(XMFS::renameUserFile(v_filepath, v_savePath + "/" + v_basename + ".lvl") == false) {
 	  LogWarning("Unable to move the file into the trash");
 	}
       } catch(Exception &e) {
@@ -281,7 +275,7 @@ bool xmDatabase::levels_add_fast(const std::string& i_filepath, std::string& o_l
     v_cond = "isToReload=1 AND ";
   }
 
-  v_checksum = FS::md5sum(i_filepath);
+  v_checksum = XMFS::md5sum(FDT_DATA, i_filepath);
 
   v_result = readDB("SELECT name FROM levels "
 		    "WHERE " + v_cond   +
