@@ -19,6 +19,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 =============================================================================*/
 
 #include "xmDatabase.h"
+#include "../GameText.h"
 #include <sstream>
 #include <math.h>
 
@@ -66,3 +67,30 @@ bool xmDatabase::replays_exists(const std::string& i_name) {
   read_DB_free(v_result);
   return nrow == 1;
 }
+
+void xmDatabase::replays_print() {
+  char **v_result;
+  unsigned int nrow;
+    
+  printf("\nReplay                    Level                     Player\n");
+  printf("-----------------------------------------------------------------------\n");
+    
+  v_result = readDB("SELECT a.name, a.id_profile, b.name "
+		    "FROM replays AS a LEFT OUTER JOIN levels AS b "
+		    "ON a.id_level = b.id_level;", nrow);
+  if(nrow == 0) {
+    printf("(none)\n");
+  } else {
+    std::string v_levelName;
+    
+    for(unsigned int i=0; i<nrow; i++) {
+      if(getResult(v_result, 3, i, 2) == NULL) {
+	v_levelName = GAMETEXT_UNKNOWN;
+      } else {
+	v_levelName = getResult(v_result, 3, i, 2);
+      }
+      printf("%-25s %-25s %-25s\n", getResult(v_result, 3, i, 0), v_levelName.c_str(), getResult(v_result, 3, i, 1));
+    }
+  }
+  read_DB_free(v_result);
+}
