@@ -28,6 +28,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "../../Game.h"
 #include "../../helpers/Log.h"
 #include "../../helpers/utf8.h"
+#include "../../helpers/RenderSurface.h"
 
 #ifdef ENABLE_OPENGL
 #include "../../include/xm_OpenGL.h"
@@ -79,6 +80,7 @@ bool UIWindow::isUglyMode() {
   
   void UIWindow::initW(UIWindow *pParent,int x,int y,std::string Caption,int nWidth,int nHeight) {
     _InitWindow();
+    m_screen = pParent->m_screen;
 
     setPosition(x,y,nWidth,nHeight);
     setCaption(Caption);
@@ -125,6 +127,10 @@ bool UIWindow::isUglyMode() {
       m_Children[i]->enableWindow(bState);
   }
 
+RenderSurface* UIWindow::getScreen() {
+  return m_screen;
+}
+
   void UIWindow::makeActive(void) {
     /* Find root */
     UIRoot *pRoot = NULL;
@@ -218,7 +224,43 @@ bool UIWindow::isUglyMode() {
   
   /*===========================================================================
   Message boxes
-  ===========================================================================*/  
+  ===========================================================================*/
+  void UIMsgBox::initMsgBox(UIWindow *pParent,int x,int y,std::string Caption,int nWidth,int nHeight) {
+    initW(pParent,x,y,Caption,nWidth,nHeight);
+
+    setStyle(UI_FRAMESTYLE_TRANS);
+    m_textInputFont = NULL;
+    m_bTextInput = false;
+    m_nNumButtons = 0;
+  }
+
+  UIMsgBox::UIMsgBox(UIWindow *pParent,int x,int y,std::string Caption,int nWidth,int nHeight) {
+    initMsgBox(pParent, x, y, Caption, nWidth, nHeight);
+  }
+
+  UIMsgBox::UIMsgBox(UIWindow *pParent, std::vector<std::string>& wordcompletionlist ,int x,int y,std::string Caption,int nWidth,int nHeight) {
+   initMsgBox(pParent, x, y, Caption, nWidth, nHeight);
+   for (int i = 0, n = wordcompletionlist.size(); i < n; i++) {
+     addCompletionWord(wordcompletionlist[i]);
+   }
+ }
+
+std::string UIMsgBox::getCustom1() const {
+  return m_custom1;
+}
+
+void UIMsgBox::setCustom1(const std::string& i_str) {
+  m_custom1 = i_str;
+}
+      
+std::string UIMsgBox::getCustom2() const {
+  return m_custom2;
+}
+
+void UIMsgBox::setCustom2(const std::string& i_str) {
+  m_custom2 = i_str;
+}
+
   UIMsgBoxButton UIMsgBox::getClicked(void) {
     /* Go through buttons... anything clicked? */
     for(unsigned int i=0;i<m_nNumButtons;i++) {
@@ -233,6 +275,10 @@ bool UIWindow::isUglyMode() {
 	  return UI_MSGBOX_NO;
         if(m_pButtons[i]->getCaption() == GAMETEXT_YES_FOR_ALL)
 	  return UI_MSGBOX_YES_FOR_ALL;
+        if(m_pButtons[i]->getCaption() == m_custom1)
+	  return UI_MSGBOX_CUSTOM1;
+        if(m_pButtons[i]->getCaption() == m_custom2)
+	  return UI_MSGBOX_CUSTOM2;
       }
     }
     return UI_MSGBOX_NOTHING;
@@ -294,12 +340,26 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
       }
     }
 
+   if(i_button == UI_MSGBOX_CUSTOM1) {
+      if(m_pButtons[i]->getCaption() == m_custom1) {
+	m_pButtons[i]->makeActive();
+	return;
+      }
+    }
+
+   if(i_button == UI_MSGBOX_CUSTOM2) {
+      if(m_pButtons[i]->getCaption() == m_custom2) {
+	m_pButtons[i]->makeActive();
+	return;
+      }
+    }
+
   }
 }
 
   void UIMsgBox::paint(void) {
     /* Should the OK button be disabled? (if any) */
-    if(m_bTextInput && m_TextInput.empty()) {
+    if(m_bTextInput && m_TextInput_real.empty()) {
       for(unsigned int i=0; i<m_nNumButtons; i++) {
         if(m_pButtons[i]->getCaption() == GAMETEXT_OK)
 	  m_pButtons[i]->enableWindow(false);
@@ -318,10 +378,11 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
     /* Should we do some text? */
     if(m_bTextInput && m_textInputFont!=NULL) {      
       setFont(m_textInputFont);
-      putText(16,120,m_TextInput + std::string("|"));
+      putText(16,120,m_TextInput_fake + std::string("|"));
     }
   }
   
+
   bool UIMsgBox::keyDown(int nKey, SDLMod mod, const std::string& i_utf8Char) {
     switch(nKey) {
       case SDLK_ESCAPE:
@@ -329,7 +390,8 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
           setClicked(GAMETEXT_NO);
         return true;
       case SDLK_RETURN:
-        if(!m_bTextInput || !m_TextInput.empty()) {
+	m_TextInput_real = m_TextInput_fake; // valid completion if in competion
+        if(!m_bTextInput || !m_TextInput_real.empty()) {
           setClicked(GAMETEXT_OK);
           return true;
         }
@@ -339,13 +401,21 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
 
           switch(nKey) {
             case SDLK_BACKSPACE:
-	      if(m_TextInput != "") {
-		m_TextInput = utf8::utf8_delete(m_TextInput, utf8::utf8_length(m_TextInput));
+	      m_TextInput_real = m_TextInput_fake; // valid completion if in competion
+	      if(m_TextInput_real != "") {
+		m_TextInput_fake = utf8::utf8_delete(m_TextInput_fake, utf8::utf8_length(m_TextInput_fake));
+		m_TextInput_real = m_TextInput_fake;
 	      }
               return true;
+            case SDLK_TAB:
+            	showMatch();
+            	return true;
             default:
 	      if(utf8::utf8_length(i_utf8Char) == 1) { // alt/... and special keys must not be kept
-		m_TextInput = utf8::utf8_concat(m_TextInput, i_utf8Char);
+		if(i_utf8Char[0] != '\n') { // you can generate ascii 10 with ctrl+j or keyboard having new line key
+		  m_TextInput_fake = utf8::utf8_concat(m_TextInput_fake, i_utf8Char);
+		  m_TextInput_real = m_TextInput_fake;
+		}
 	      }
               return true;
           }
@@ -357,9 +427,11 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
     return false;
   }
   
-  UIMsgBox *UIWindow::msgBox(std::string Text,UIMsgBoxButton Buttons,bool bTextInput,bool bQuery, bool i_verticallyLarge) {
+  UIMsgBox *UIWindow::msgBox(std::string Text,UIMsgBoxButton Buttons, const std::string& i_help,
+			     const std::string& i_custom1, const std::string& i_custom2,
+			     bool bTextInput,bool bQuery, bool i_verticallyLarge) {
     unsigned int nNumButtons = 0;
-    
+
     if(Buttons & UI_MSGBOX_OK)
       nNumButtons++;
     if(Buttons & UI_MSGBOX_CANCEL)
@@ -370,11 +442,16 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
       nNumButtons++;
     if(Buttons & UI_MSGBOX_YES_FOR_ALL)
       nNumButtons++;
+    if(Buttons & UI_MSGBOX_CUSTOM1)
+      nNumButtons++;
+    if(Buttons & UI_MSGBOX_CUSTOM2)
+      nNumButtons++;
 
     /* Determine size of contents */
     FontGlyph* v_fg = m_curFont->getGlyph(Text);
 
     const unsigned int nButtonSize = 57;
+    int v_help_height = i_help == "" ? 0 : 20+8;
     int w;
 
     if(i_verticallyLarge) {
@@ -382,7 +459,7 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
     } else {
       w = (v_fg->realWidth() > nNumButtons*115 ? (v_fg->realWidth()) : nNumButtons*115) + 16 + 100;
     }
-    int h = v_fg->realHeight() + nButtonSize + 24 + 100;
+    int h = v_fg->realHeight() + nButtonSize + 16 + v_help_height + 100;
     
     if(bTextInput) h+=40;    
     
@@ -390,6 +467,9 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
     UIMsgBox *pMsgBox;
     if(bQuery) pMsgBox = (UIMsgBox *)new UIQueryKeyBox(this,getPosition().nWidth/2 - w/2,getPosition().nHeight/2 - h/2,"",w,h);
     else pMsgBox = new UIMsgBox(this,getPosition().nWidth/2 - w/2,getPosition().nHeight/2 - h/2,"",w,h);
+
+    pMsgBox->setCustom1(i_custom1);
+    pMsgBox->setCustom2(i_custom2);
     
     pMsgBox->makeActive();    
     if(bTextInput)
@@ -407,57 +487,171 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
     int nStaticY=0;
     if(bTextInput) nStaticY=40;
     UIStatic *pText = new UIStatic(pMsgBox,8,8,Text,pMsgBox->getPosition().nWidth-16,
-                                   pMsgBox->getPosition().nHeight-24-nButtonSize-nStaticY);   
+                                   pMsgBox->getPosition().nHeight-24-nButtonSize-nStaticY);
     pText->setFont(getFont()); /* inherit font */      
     pText->setBackgroundShade(true); /* make text more easy to read */
     
-    /* Create buttons */
-    int nCX = pMsgBox->getPosition().nWidth/2 - (nNumButtons*115)/2;
-    int nCY = pMsgBox->getPosition().nHeight-16-nButtonSize;
-    
+    /* Create buttons */    
     UIButton *pButton;
+    int v_buttonWidth;
+
+    bool v_useCustom = ((Buttons & UI_MSGBOX_CUSTOM1) || (Buttons & UI_MSGBOX_CUSTOM2));
+
+    // make them a bit larger when there are custom
+    if(v_useCustom) {
+      v_buttonWidth = 150;
+    } else {
+      v_buttonWidth = 115;
+    }
+
+    int nCX = pMsgBox->getPosition().nWidth/2 - (nNumButtons*v_buttonWidth)/2;
+    int nCY = pMsgBox->getPosition().nHeight-16-v_help_height-nButtonSize;
+
     if(Buttons & UI_MSGBOX_OK) {
-      pButton = new UIButton(pMsgBox,nCX,nCY,GAMETEXT_OK,115,57);
+      pButton = new UIButton(pMsgBox,nCX,nCY,GAMETEXT_OK,v_buttonWidth,57);
       pButton->setFont(getFont());
-      pButton->setType(UI_BUTTON_TYPE_SMALL);
+      if(v_useCustom == false) pButton->setType(UI_BUTTON_TYPE_SMALL);
       pMsgBox->addButton(pButton);
-      nCX+=115;
+      nCX+=v_buttonWidth;
     }
     if(Buttons & UI_MSGBOX_CANCEL) {
-      pButton = new UIButton(pMsgBox,nCX,nCY,GAMETEXT_CANCEL,115,57);
+      pButton = new UIButton(pMsgBox,nCX,nCY,GAMETEXT_CANCEL,v_buttonWidth,57);
       pButton->setFont(getFont());
-      pButton->setType(UI_BUTTON_TYPE_SMALL);
+      if(v_useCustom == false) pButton->setType(UI_BUTTON_TYPE_SMALL);
       pMsgBox->addButton(pButton);
-      nCX+=115;
+      nCX+=v_buttonWidth;
     }
     if(Buttons & UI_MSGBOX_YES) {
-      pButton = new UIButton(pMsgBox,nCX,nCY,GAMETEXT_YES,115,57);
+      pButton = new UIButton(pMsgBox,nCX,nCY,GAMETEXT_YES,v_buttonWidth,57);
       pButton->setFont(getFont());
-      pButton->setType(UI_BUTTON_TYPE_SMALL);
+      if(v_useCustom == false) pButton->setType(UI_BUTTON_TYPE_SMALL);
       pMsgBox->addButton(pButton);
-      nCX+=115;
+      nCX+=v_buttonWidth;
+    }
+    if(Buttons & UI_MSGBOX_CUSTOM1) {
+      pButton = new UIButton(pMsgBox,nCX,nCY,pMsgBox->getCustom1(),v_buttonWidth,57);
+      pButton->setFont(getFont());
+      pMsgBox->addButton(pButton);
+      nCX+=v_buttonWidth;
+    }
+    if(Buttons & UI_MSGBOX_CUSTOM2) {
+      pButton = new UIButton(pMsgBox,nCX,nCY,pMsgBox->getCustom2(),v_buttonWidth,57);
+      pButton->setFont(getFont());
+      pMsgBox->addButton(pButton);
+      nCX+=v_buttonWidth;
     }
     if(Buttons & UI_MSGBOX_NO) {
-      pButton = new UIButton(pMsgBox,nCX,nCY,GAMETEXT_NO,115,57);
+      pButton = new UIButton(pMsgBox,nCX,nCY,GAMETEXT_NO,v_buttonWidth,57);
       pButton->setFont(getFont());
-      pButton->setType(UI_BUTTON_TYPE_SMALL);
+      if(v_useCustom == false) pButton->setType(UI_BUTTON_TYPE_SMALL);
       pMsgBox->addButton(pButton);
-      nCX+=115;
+      nCX+=v_buttonWidth;
     }
     if(Buttons & UI_MSGBOX_YES_FOR_ALL) {
-      pButton = new UIButton(pMsgBox,nCX,nCY,GAMETEXT_YES_FOR_ALL,115,57);
+      pButton = new UIButton(pMsgBox,nCX,nCY,GAMETEXT_YES_FOR_ALL,v_buttonWidth,57);
       pButton->setFont(getFont());
-      pButton->setType(UI_BUTTON_TYPE_SMALL);
+      if(v_useCustom == false) pButton->setType(UI_BUTTON_TYPE_SMALL);
       pMsgBox->addButton(pButton);
-      nCX+=115;
+      nCX+=v_buttonWidth;
     }    
 
+    /* Display some help */
+    if(i_help != "") {
+      UIStatic *pText = new UIStatic(pMsgBox,
+				     8, pMsgBox->getPosition().nHeight-20-8,
+				     i_help,
+				     pMsgBox->getPosition().nWidth-16, 20);
+      pText->setFont(m_drawLib->getFontSmall());
+      pText->setNormalColor(MAKE_COLOR(255,255,0,255));
+    }
+
     /* Set msgbox flag */
     m_bActiveMsgBox = true;
 
     return pMsgBox;                                
   }
 
+  UIMsgBox *UIWindow::msgBox(std::string Text, std::vector<std::string>& wordcompletionlist,UIMsgBoxButton Buttons, const std::string& i_help,
+			     const std::string& i_custom1, const std::string& i_custom2,
+			     bool bTextInput,bool bQuery,bool i_verticallyLarge) {
+    UIMsgBox *pMsgBox = this->msgBox(Text, Buttons, i_help, i_custom1, i_custom2, bTextInput, bQuery, i_verticallyLarge);
+	  pMsgBox->addCompletionWord(wordcompletionlist);
+	  return pMsgBox;
+  }
+
+ /*
+  * overlodaed functions for adding completion words
+  */
+
+  void UIMsgBox::addCompletionWord(std::string& word) {
+	  m_completionWords.push_back(word);
+  }
+
+  void UIMsgBox::addCompletionWord(std::vector<std::string> &list) {
+	  for(int i = 0, n = list.size(); i < n; i++) {
+		  this->addCompletionWord(list[i]);
+	  }
+  }
+
+  /*
+   *
+   * completion word function
+   */
+
+  void UIMsgBox::showMatch() {
+	  int last_word_f_pos = m_TextInput_real.rfind(" ") + 1;
+	  std::string last_word_f = m_TextInput_fake.substr(last_word_f_pos);
+	  for (int i = 0, n = last_word_f.size(); i < n; i++) {
+		  last_word_f[i] = tolower(last_word_f[i]);
+	  }
+	  std::vector<std::string> matches = findMatches();
+	  for (int i = 0, n = matches.size(); i < n; i++) {
+		  std::string match = matches[i];
+		  for (int j = 0, k = match.size(); j < k; j++) {
+			  match[j] = tolower(match[j]);
+		  }
+		  if(match.find(last_word_f) == 0) {
+			  if(i == (n - 1)) {
+				  std::string s;
+				  s = utf8::utf8_substring(m_TextInput_fake, 0, last_word_f_pos);
+				  s += matches[0];
+				  m_TextInput_fake = s;
+				  break;
+			  } else {
+				  std::string s;
+				  s = utf8::utf8_substring(m_TextInput_fake, 0, last_word_f_pos);
+				  s += matches[i + 1];
+				  m_TextInput_fake = s;
+			  }
+		  }
+	  }
+  }
+
+  std::vector<std::string> UIMsgBox::findMatches() {
+	  std::vector<std::string> matchesList;
+	  int pos_find = m_TextInput_real.rfind(" ") + 1;
+	  std::string last_word = m_TextInput_real.substr(pos_find);
+	  std::string last_word_normal = last_word;
+	  for (int i = 0, n = last_word.size(); i < n; i++) {
+		  last_word[i] = tolower(last_word[i]);
+	  }
+
+	  for (int i = 0, n = m_completionWords.size(); i < n; i++) {
+	    std::string completionWord = m_completionWords[i];
+	    for (int j = 0, k = completionWord.size(); j < k; j++) {
+	      completionWord[j] = tolower(completionWord[j]);
+	    }
+	    if (completionWord.find(last_word) == 0) {
+	      matchesList.push_back(m_completionWords[i]);
+	    }
+	  }
+	  matchesList.push_back(last_word_normal); // allow to come back before completion
+
+	  std::vector<std::string> empty;
+	  empty.push_back("");
+	  return !matchesList.empty() ? matchesList : empty;
+  }
+
   /*===========================================================================
   Base painting
   ===========================================================================*/
@@ -487,7 +681,7 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
     
     FontManager* v_fm = m_curFont;
     FontGlyph* v_fg = v_fm->getGlyph(Text);
-    v_fm->printStringGrad(v_fg,
+    v_fm->printStringGrad(GameApp::instance()->getDrawLib(), v_fg,
 			  getAbsPosX()+x + (int)(((float)v_fg->realWidth()) * i_xper),
 			  getAbsPosY()+y + (int)(((float)v_fg->realHeight()) * i_yper),
 			  c0, c1, c2, c3, i_perCentered);
@@ -695,12 +889,18 @@ FRAME_BR (187,198) (8x8)
   /*===========================================================================
   Root window
   ===========================================================================*/
-UIRoot::UIRoot()
+UIRoot::UIRoot(RenderSurface* i_screen)
 {
   m_pApp = GameApp::instance();
   m_bShowContextMenu = true;
   m_lastHover = NULL;
+
   _InitWindow();
+  m_screen = new RenderSurface(*i_screen); // get a copy in case the original is removed
+}
+
+UIRoot::~UIRoot() {
+  delete m_screen;
 }
 
   void UIRoot::_ClipRect(UIRect *pRect,UIRect *pClipWith) {
@@ -754,8 +954,8 @@ UIRoot::UIRoot()
     /* Clip to full screen */
     Screen.nX = 0;
     Screen.nY = 0;
-    Screen.nWidth = m_drawLib->getDispWidth();
-    Screen.nHeight = m_drawLib->getDispHeight();
+    Screen.nWidth = m_screen->getDispWidth();
+    Screen.nHeight = m_screen->getDispHeight();
       
     /* Draw root's children */
 #ifdef ENABLE_OPENGL
@@ -778,20 +978,20 @@ UIRoot::UIRoot()
       m_drawLib->startDraw(DRAW_MODE_POLYGON);
       //glColor4f(0,0,0,0);//fully transparent??
       m_drawLib->setColorRGBA(0,0,0,0);
-      m_drawLib->glVertexSP(0,m_drawLib->getDispHeight()-nContextHelpHeight);
-      m_drawLib->glVertexSP(m_drawLib->getDispWidth(),m_drawLib->getDispHeight()-nContextHelpHeight);
+      m_drawLib->glVertexSP(0,m_screen->getDispHeight()-nContextHelpHeight);
+      m_drawLib->glVertexSP(m_screen->getDispWidth(),m_screen->getDispHeight()-nContextHelpHeight);
       //glColor4f(0,0,0,0.7);
       m_drawLib->setColorRGBA(0,0,0,255 * 7 / 100);
-      m_drawLib->glVertexSP(m_drawLib->getDispWidth(),m_drawLib->getDispHeight());
-      m_drawLib->glVertexSP(0,m_drawLib->getDispHeight());
+      m_drawLib->glVertexSP(m_screen->getDispWidth(),m_screen->getDispHeight());
+      m_drawLib->glVertexSP(0,m_screen->getDispHeight());
       m_drawLib->endDraw();
         
       if(!m_CurrentContextHelp.empty()) {
         /* Print help string */
 	setFont(m_drawLib->getFontSmall());
 	setTextSolidColor(MAKE_COLOR(255,255,0,255));
-	putText(m_drawLib->getDispWidth()  -5,
-		m_drawLib->getDispHeight() -1,
+	putText(m_screen->getDispWidth()  -5,
+		m_screen->getDispHeight() -1,
 		m_CurrentContextHelp, -1.0, -1.0);
       }
     }
@@ -1001,11 +1201,9 @@ bool UIRoot::_RootJoystickButtonDownEvent(UIWindow *pWindow, Uint8 i_joyNum, Uin
               
               /* By the way, does it want to offer context-help? (only 
                  if mouse have actually moved) */
-              if(getApp()->haveMouseMoved()) {
-                std::string SubCHelp = pWindow->subContextHelp(wx,wy);
-                if(SubCHelp != "") m_CurrentContextHelp = SubCHelp;
-                else m_CurrentContextHelp = pWindow->getContextHelp();
-              }
+	      std::string SubCHelp = pWindow->subContextHelp(wx,wy);
+	      if(SubCHelp != "") m_CurrentContextHelp = SubCHelp;
+	      else m_CurrentContextHelp = pWindow->getContextHelp();
               
               break;
             }
