@@ -22,10 +22,19 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "../helpers/Log.h"
 #include "../helpers/VExcept.h"
 #include "../XMSession.h"
+#include <sstream>
 #include "NetActions.h"
+#include "helpers/Net.h"
 
 #define XM_MAX_PACKET_SIZE_DIGITS 6 // limit the size of a command : n digits
 
+unsigned int ActionReader::m_biggestTCPPacketReceived = 0;
+unsigned int ActionReader::m_biggestUDPPacketReceived = 0;
+unsigned int ActionReader::m_nbTCPPacketsReceived     = 0;
+unsigned int ActionReader::m_nbUDPPacketsReceived     = 0;
+unsigned int ActionReader::m_TCPPacketsSizeReceived   = 0;
+unsigned int ActionReader::m_UDPPacketsSizeReceived   = 0;
+
 ActionReader::ActionReader() {
     m_tcpPacketOffset     = 0;
     m_tcpNotEnoughData    = false;
@@ -35,6 +44,35 @@ ActionReader::ActionReader() {
 ActionReader::~ActionReader() {
 }
 
+/* std::string version */
+std::string ActionReader::getStats() {
+  std::string v_stats;
+
+  std::ostringstream v_nbTCPReceived, v_nbUDPReceived;
+
+  v_nbTCPReceived << ActionReader::m_nbTCPPacketsReceived;
+  v_nbUDPReceived << ActionReader::m_nbUDPPacketsReceived;
+
+  v_stats += "number of TCP packets received : " + v_nbTCPReceived.str()                                          + "\n";
+  v_stats += "biggest TCP packet received : "    + XMNet::getFancyBytes(ActionReader::m_biggestTCPPacketReceived) + "\n";
+  v_stats += "size of TCP packets received : "   + XMNet::getFancyBytes(ActionReader::m_TCPPacketsSizeReceived)   + "\n";
+  v_stats += "number of UDP packets received : " + v_nbUDPReceived.str()                                       	  + "\n";
+  v_stats += "biggest UDP packet received : "    + XMNet::getFancyBytes(ActionReader::m_biggestUDPPacketReceived) + "\n";
+  v_stats += "size of UDP packets received : "   + XMNet::getFancyBytes(ActionReader::m_UDPPacketsSizeReceived);
+
+  return v_stats;
+}
+
+/* log version */
+void ActionReader::logStats() {
+  LogInfo("%-31s : %u", "net: number of TCP packets received", ActionReader::m_nbTCPPacketsReceived);
+  LogInfo("%-31s : %s", "net: biggest TCP packet received"   , XMNet::getFancyBytes(ActionReader::m_biggestTCPPacketReceived).c_str());
+  LogInfo("%-31s : %s", "net: size of TCP packets received"  , XMNet::getFancyBytes(ActionReader::m_TCPPacketsSizeReceived).c_str());
+  LogInfo("%-31s : %u", "net: number of UDP packets received", ActionReader::m_nbUDPPacketsReceived);
+  LogInfo("%-31s : %s", "net: biggest UDP packet received"   , XMNet::getFancyBytes(ActionReader::m_biggestUDPPacketReceived).c_str());
+  LogInfo("%-31s : %s", "net: size of UDP packets received"  , XMNet::getFancyBytes(ActionReader::m_UDPPacketsSizeReceived).c_str());
+}
+
 bool ActionReader::TCPReadAction(TCPsocket* i_tcpsd, NetAction** i_netAction) {
   int nread;
   unsigned int v_cmdStart;
@@ -51,6 +89,12 @@ bool ActionReader::TCPReadAction(TCPsocket* i_tcpsd, NetAction** i_netAction) {
     m_tcpPacketOffset += nread;
     LogDebug("Data received (%u bytes available)", m_tcpPacketOffset);
     m_tcpNotEnoughData = false; // you don't know if the buffer is full enough
+
+    if(nread > ActionReader::m_biggestTCPPacketReceived) {
+      ActionReader::m_biggestTCPPacketReceived = nread;
+    }
+    ActionReader::m_nbTCPPacketsReceived++;
+    ActionReader::m_TCPPacketsSizeReceived += nread;
   }
 
   m_tcpPossiblyInBuffer = false;
@@ -125,6 +169,12 @@ NetAction* ActionReader::UDPReadAction(Uint8* data, int len) {
   unsigned int v_size;
   unsigned int v_cmdStart;
 
+  if(len > ActionReader::m_biggestUDPPacketReceived) {
+    ActionReader::m_biggestUDPPacketReceived = len;
+  }
+  ActionReader::m_nbUDPPacketsReceived++;
+  ActionReader::m_UDPPacketsSizeReceived += len;
+
   if( (v_size = ActionReader::getSubPacketSize(data, len, v_cmdStart)) > 0) {
     return NetAction::newNetAction(((char*)data)+v_cmdStart, v_size);
   }
