@@ -27,7 +27,16 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "../xmscene/BasicSceneStructs.h"
 #include "BasicStructures.h"
 
-#define XM_NET_PROTOCOL_VERSION 1
+#define XM_NET_PROTOCOL_VERSION 4
+/*
+DELTA 1->2:
+clientInfos : add xmversion string
+DELTA 2->3:
+add NA_udpBindValidation
+DELTA 3->4:
+add chatMessagePP (private/public)
+*/
+
 #define NETACTION_MAX_PACKET_SIZE 1024 * 2 // bytes
 #define NETACTION_MAX_SUBSRC 4 // maximum 4 players by client
 #define XM_NET_MAX_EVENTS_SHOT_SIZE 1024
@@ -38,12 +47,16 @@ class DBuffer;
 
 enum NetActionType {
   TNA_clientInfos,
-  TNA_udpBind,
   TNA_udpBindQuery,
+  TNA_udpBind,
+  TNA_udpBindValidation,
   TNA_chatMessage,
+  TNA_chatMessagePP,
   TNA_serverError,
   TNA_frame,
   TNA_changeName,
+  TNA_clientsNumber,
+  TNA_clientsNumberQuery,
   TNA_playingLevel,
   TNA_changeClients,
   TNA_playerControl,
@@ -63,7 +76,7 @@ struct NetInfosClient {
 
 class NetAction {
   public:
-  NetAction();
+  NetAction(bool i_forceTcp);
   virtual ~NetAction();
   virtual std::string actionKey()    = 0;
   virtual NetActionType actionType() = 0;
@@ -77,7 +90,6 @@ class NetAction {
   static NetAction* newNetAction(void* data, unsigned int len);
   static void logStats();
   static std::string getStats();
-  static std::string getFancyBytes(unsigned int i_bytes);
 
   protected:
   void send(TCPsocket* i_tcpsd, UDPsocket* i_udpsd, UDPpacket* i_sendPacket, IPaddress* i_udpRemoteIP, const void* subPacketData, int subPacketLen);
@@ -99,12 +111,14 @@ class NetAction {
 
   private:
   static char m_buffer[NETACTION_MAX_PACKET_SIZE];
-  static unsigned int m_biggestTCPPacket;
-  static unsigned int m_biggestUDPPacket;
+  static unsigned int m_biggestTCPPacketSent;
+  static unsigned int m_biggestUDPPacketSent;
   static unsigned int m_nbTCPPacketsSent;
   static unsigned int m_nbUDPPacketsSent;
   static unsigned int m_TCPPacketsSizeSent;
   static unsigned int m_UDPPacketsSizeSent;
+
+  bool m_forceTCP; // by default, xmoto try to use UDP when available ; for some actions, TCP can be forced
 };
 
 class NA_udpBind : public NetAction {
@@ -140,9 +154,30 @@ class NA_udpBindQuery : public NetAction {
   private:
 };
 
-/* the client send its key by tcp (clientInfos) */
-/* then, the server ask via tcp (udpBindQuery) the client to send an udp packet (udpBind) */
-/* the server bind */
+class NA_udpBindValidation : public NetAction {
+  public:
+  NA_udpBindValidation();
+  NA_udpBindValidation(void* data, unsigned int len);
+  virtual ~NA_udpBindValidation();
+  std::string actionKey()    { return ActionKey; }
+  NetActionType actionType() { return NAType; }
+  static std::string ActionKey;
+  static NetActionType NAType;
+
+  void send(TCPsocket* i_tcpsd, UDPsocket* i_udpsd, UDPpacket* i_sendPacket, IPaddress* i_udpRemoteIP);  
+
+  private:
+};
+
+/*
+CLI : >clientInfos (send the udp key)
+SRV : >udpBindQuery
+CLI : <udpBindQuery ; >udpBind
+SRV : <udpBind (ok, i can *receive* udp from client) >udpBindValidation ; >udpBind
+CLI : <udpBindValidation (ok, i can *send* udp to the server)
+CLI : <udpBind (ok, i can *receive* udp from the server) ; >udpBindValidation
+SRV : <udpBindValidation (ok, i can *send* udp to the client)
+*/
 
 class NA_clientInfos : public NetAction {
   public:
@@ -158,15 +193,40 @@ class NA_clientInfos : public NetAction {
 
   int protocolVersion() const;
   std::string udpBindKey() const;
+  std::string xmversion() const;
 
   private:
   int m_protocolVersion;
   std::string m_udpBindKey;
+  std::string m_xmversion;
+};
+
+/* replace chatMessage ; it allows public and private messages */
+class NA_chatMessagePP : public NetAction {
+  public:
+  NA_chatMessagePP(const std::string& i_msg, const std::string &i_me /* if empty, change nothing */, const std::vector<int>& i_private_people /* empty vector for public message -- require XM_NET_PROTOCOL_VERSION */);
+  NA_chatMessagePP(void* data, unsigned int len);
+  virtual ~NA_chatMessagePP();
+  std::string actionKey()    { return ActionKey; }
+  NetActionType actionType() { return NAType; }
+  static std::string ActionKey;
+  static NetActionType NAType;
+
+  void send(TCPsocket* i_tcpsd, UDPsocket* i_udpsd, UDPpacket* i_sendPacket, IPaddress* i_udpRemoteIP);
+
+  std::string getMessage();
+  const std::vector<int>& privatePeople() const; // empty for public message
+
+  private:
+  std::string m_msg;
+  std::vector<int> m_private_people;
+
+  void ttransform(const std::string& i_me);
 };
 
 class NA_chatMessage : public NetAction {
   public:
-  NA_chatMessage(const std::string& i_msg);
+  NA_chatMessage(const std::string& i_msg, const std::string &i_me);
   NA_chatMessage(void* data, unsigned int len);
   virtual ~NA_chatMessage();
   std::string actionKey()    { return ActionKey; }
@@ -180,6 +240,8 @@ class NA_chatMessage : public NetAction {
 
   private:
   std::string m_msg;
+
+  void ttransform(const std::string& i_me);
 };
 
 class NA_serverError : public NetAction {
@@ -236,6 +298,38 @@ class NA_changeName : public NetAction {
   std::string m_name;
 };
 
+class NA_clientsNumber : public NetAction {
+  public:
+  NA_clientsNumber(int i_number);
+  NA_clientsNumber(void* data, unsigned int len);
+  virtual ~NA_clientsNumber();
+  std::string actionKey()    { return ActionKey; }
+  NetActionType actionType() { return NAType; }
+  static std::string ActionKey;
+  static NetActionType NAType;
+
+  void send(TCPsocket* i_tcpsd, UDPsocket* i_udpsd, UDPpacket* i_sendPacket, IPaddress* i_udpRemoteIP);
+
+  int getNumber();
+
+  private:
+  int m_number;
+};
+
+// query the number of clients for the munin plugin
+class NA_clientsNumberQuery : public NetAction {
+  public:
+  NA_clientsNumberQuery();
+  NA_clientsNumberQuery(void* data, unsigned int len);
+  virtual ~NA_clientsNumberQuery();
+  std::string actionKey()    { return ActionKey; }
+  NetActionType actionType() { return NAType; }
+  static std::string ActionKey;
+  static NetActionType NAType;
+
+  void send(TCPsocket* i_tcpsd, UDPsocket* i_udpsd, UDPpacket* i_sendPacket, IPaddress* i_udpRemoteIP);
+};
+
 class NA_playingLevel : public NetAction {
   public:
   NA_playingLevel(const std::string& i_levelId);
