@@ -35,6 +35,7 @@ class NetAction;
 class NetGhost;
 class Universe;
 class xmDatabase;
+class VirtualNetLevelsList;
 
 class NetOtherClient {
  public:
@@ -74,7 +75,7 @@ class NetClient : public Singleton<NetClient> {
   void connect(const std::string& i_server, int i_port);
   void disconnect();
   bool isConnected();
-  void send(NetAction* i_netAction, int i_subsrc);
+  void send(NetAction* i_netAction, int i_subsrc, bool i_forceUdp = false);
   TCPsocket* tcpSocket();
   UDPsocket* udpSocket();
   UDPpacket* sendPacket();
@@ -90,13 +91,22 @@ class NetClient : public Singleton<NetClient> {
   bool isPlayInitialized();
   void endPlay();
 
+  void getOtherClientsNameList(std::vector<std::string>& io_list, const std::string& i_suffix);
+  void addChatTransformations(std::vector<std::string>& io_clientList, const std::string i_suffix);
+  std::string getDisplayMessage(const std::string& i_msg, const std::string& i_author);
+
   std::vector<NetOtherClient*>& otherClients();
+  void fillPrivatePeople(const std::string& i_msg, const std::string& i_private_suffix, std::vector<int>& io_private_people,
+			 std::vector<std::string>& o_unknown_players);
 
   int getOwnFrameFPS() const;
+  VirtualNetLevelsList* getOtherClientLevelsList(xmDatabase* pDb);
 
   private:
   bool m_isConnected;
   IPaddress serverIp;
+  bool m_serverReceivesUdp;
+  bool m_serverSendsUdp;
   TCPsocket m_tcpsd;
   UDPsocket m_udpsd;
   ClientListenerThread* m_clientListenerThread;
@@ -121,6 +131,8 @@ class NetClient : public Singleton<NetClient> {
   int m_lastOwnFPS;
   int m_currentOwnFramesNb;
   int m_currentOwnFramesTime;
+
+  VirtualNetLevelsList* m_otherClientsLevelsList;
 };
 
 #endif
