@@ -37,7 +37,9 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "../../xmscene/Level.h"
 #include "../../xmscene/BikeController.h"
 
-#define XM_SERVER_UPLOADING_FPS_PLAYER   30
+#define XM_SERVER_SLAVE_MODE_MIN_PROTOCOL_VERSION 1
+
+#define XM_SERVER_UPLOADING_FPS_PLAYER   40
 #define XM_SERVER_UPLOADING_FPS_OPLAYERS 15
 #define XM_SERVER_PLAYER_INACTIV_TIME_MAX  1000
 #define XM_SERVER_PLAYER_INACTIV_TIME_PREV  300
@@ -50,6 +52,9 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #define XM_SERVER_DEFAULT_BANNER "Welcome on this server"
 #define XM_SERVER_UNPLAYING_SLEEP 10
 
+// limit multi private message to avoid people spamming everybody and making think it's private
+#define XM_SERVER_MAXIMUM_MULTI_PRIVATE_MESSAGE 3
+
 NetSClient::NetSClient(unsigned int i_id, TCPsocket i_tcpSocket, IPaddress *i_tcpRemoteIP) {
     m_id   = i_id;
     m_mode = NETCLIENT_GHOST_MODE;
@@ -59,12 +64,14 @@ NetSClient::NetSClient(unsigned int i_id, TCPsocket i_tcpSocket, IPaddress *i_tc
     m_tcpSocket   = i_tcpSocket;
     m_tcpRemoteIP = *i_tcpRemoteIP;
     m_isUdpBinded = false;
+    m_isUdpBindedValidated = false;
     tcpReader = new ActionReader();
     m_lastActivTime        =  0;
     m_lastInactivTimeAlert = -1;
     m_points = 0;
     m_isAdminConnected = false;
     m_lastGhostFrameTime = 0;
+    m_protocolVersion = -1; // not set
 }
 
 NetSClient::~NetSClient() {
@@ -115,6 +122,9 @@ bool NetSClient::isUdpBinded() const {
   return m_isUdpBinded;
 }
 
+bool NetSClient::isUdpBindedValidated() const {
+  return m_isUdpBindedValidated;
+}
 void NetSClient::bindUdp(IPaddress i_udpIPAdress) {
   m_udpRemoteIP = i_udpIPAdress;
 
@@ -123,6 +133,10 @@ void NetSClient::bindUdp(IPaddress i_udpIPAdress) {
   m_isUdpBinded = true;
 }
 
+void NetSClient::validUdpBind() {
+  m_isUdpBindedValidated = true;
+}
+
 void NetSClient::unbindUdp() {
   m_isUdpBinded = false;
 }
@@ -135,6 +149,22 @@ std::string NetSClient::udpBindKey() const {
   return m_udpBindKey;
 }
 
+void NetSClient::setProtocolVersion(int i_protocolVersion) {
+  m_protocolVersion = i_protocolVersion;
+}
+
+int NetSClient::protocolVersion() const {
+  return m_protocolVersion;
+}
+
+void NetSClient::setXmVersion(const std::string& i_xmversion) {
+  m_xmversion = i_xmversion;
+}
+
+std::string NetSClient::xmversion() const {
+  return m_xmversion;
+}
+
 void NetSClient::setName(const std::string& i_name) {
   m_name = i_name;
 }
@@ -279,7 +309,9 @@ int ServerThread::realThreadFunction() {
   }
 
   m_acceptConnections = true;
-  StateManager::instance()->sendAsynchronousMessage("SERVER_STATUS_CHANGED");
+  if(StateManager::exists()) {
+    StateManager::instance()->sendAsynchronousMessage("SERVER_STATUS_CHANGED");
+  }
 
   // manage server
   while(m_askThreadToEnd == false) {
@@ -316,7 +348,9 @@ int ServerThread::realThreadFunction() {
   SDLNet_FreeSocketSet(m_set);
   m_set = NULL;
 
-  StateManager::instance()->sendAsynchronousMessage("SERVER_STATUS_CHANGED");
+  if(StateManager::exists()) {
+    StateManager::instance()->sendAsynchronousMessage("SERVER_STATUS_CHANGED");
+  }
   LogInfo("server: ending normally");
   return 0;
 }
@@ -329,7 +363,7 @@ std::string ServerThread::SP2_determineLevel() {
   // don't allow own levels (isToReload=1)
   v_result = m_pDb->readDB("SELECT id_level "
                            "FROM levels "
-                           "WHERE isToReload=0 AND isScripted=0 AND isPhysics=0 "
+                           "WHERE isToReload=0 AND isScripted=0 AND isPhysics=0 " // warning, addforcetoplayer event cannot be over network game cause player id is serialized
                            "ORDER BY RANDOM() LIMIT 1;",
 			 nrow);
   if(nrow == 0) {
@@ -356,6 +390,7 @@ void ServerThread::SP2_initPlaying() {
 
   try {
     for(unsigned int i=0; i<m_universe->getScenes().size(); i++) {
+
       v_numPlayer = 0;
       m_universe->getScenes()[i]->loadLevel(m_pDb, v_id_level);
       if(m_universe->getScenes()[i]->getLevelSrc()->isXMotoTooOld()) {
@@ -379,10 +414,10 @@ void ServerThread::SP2_initPlaying() {
 	  v_localNetId++;
 	}
       }
-      m_universe->getScenes()[i]->playLevel();
+      m_universe->getScenes()[i]->playInitLevel();
     }
   } catch(Exception &e) {
-    LogWarning("Server: Unable to load level %s",  v_id_level.c_str());
+    LogWarning("server: Unable to load level %s",  v_id_level.c_str());
     throw Exception("Unable to load level " + v_id_level);
   }
 
@@ -527,10 +562,10 @@ void ServerThread::SP2_updateScenePlaying() {
       nPhysSteps++;
     }
 
-    // if the delay is too long, reinitialize
-    if(m_fLastPhysTime + PHYS_STEP_SIZE/100.0 < GameApp::getXMTime()) {
-      m_fLastPhysTime = GameApp::getXMTime();
-    }
+    // if the delay is too long, reinitialize -- don't skip in server mode
+    //if(m_fLastPhysTime + PHYS_STEP_SIZE/100.0 < GameApp::getXMTime()) {
+    //  m_fLastPhysTime = GameApp::getXMTime();
+    //}
     SP2_sendSceneEvents(m_DBuffer);
   } else {
     /* send the first frame regularly, so that the client received it once ready */
@@ -538,6 +573,8 @@ void ServerThread::SP2_updateScenePlaying() {
       m_firstFrameSent = GameApp::getXMTimeInt();
       v_firstFrame = true;
     }
+    // initialize the physics time
+    m_fLastPhysTime = GameApp::getXMTime();
   }
 
   // send to each client his frame and the frame of the others
@@ -548,19 +585,21 @@ void ServerThread::SP2_updateScenePlaying() {
 	if(m_clients[i]->isMarkedToPlay()) {
 	  v_scene = m_universe->getScenes()[m_clients[i]->getNumScene()];
 	  
-	  v_scene->getSerializedBikeState(v_scene->Players()[m_clients[i]->getNumPlayer()]->getState(),
-					  v_scene->getTime(), &BikeState, v_scene->getPhysicsSettings());
-	  NA_frame na(&BikeState);
-	  try {
-	    if(v_firstFrame ||
-	       m_currentFrame%(100/XM_SERVER_UPLOADING_FPS_PLAYER) == 0) {
-	      sendToClient(&na, i, -1, 0);
+	  if(v_scene->Players()[m_clients[i]->getNumPlayer()]->isDead() == false) {
+	    v_scene->getSerializedBikeState(v_scene->Players()[m_clients[i]->getNumPlayer()]->getState(),
+					    v_scene->getTime(), &BikeState, v_scene->getPhysicsSettings());
+	    NA_frame na(&BikeState);
+	    try {
+	      if(v_firstFrame ||
+		 m_currentFrame%(100/XM_SERVER_UPLOADING_FPS_PLAYER) == 0) {
+		sendToClient(&na, i, -1, 0);
+	      }
+	      if(v_firstFrame ||
+		 m_currentFrame%(100/XM_SERVER_UPLOADING_FPS_OPLAYERS) == 0) {
+		sendToAllClientsMarkedToPlay(&na, m_clients[i]->id(), 0, i);
+	      }
+	    } catch(Exception &e) {
 	    }
-	    if(v_firstFrame ||
-	       m_currentFrame%(100/XM_SERVER_UPLOADING_FPS_OPLAYERS) == 0) {
-	      sendToAllClientsMarkedToPlay(&na, m_clients[i]->id(), 0, i);
-	    }
-	  } catch(Exception &e) {
 	  }
 	}
       }
@@ -780,9 +819,11 @@ void ServerThread::removeClient(unsigned int i) {
   m_clients.erase(m_clients.begin() + i);
 }
 
-void ServerThread::sendToClient(NetAction* i_netAction, unsigned int i, int i_src, int i_subsrc) {
+void ServerThread::sendToClient(NetAction* i_netAction, unsigned int i, int i_src, int i_subsrc, bool i_forceUdp) {
   i_netAction->setSource(i_src, i_subsrc);
-  if(m_clients[i]->isUdpBinded()) {
+  if(i_forceUdp) {
+    i_netAction->send(NULL, &m_udpsd, m_udpPacket, m_clients[i]->udpRemoteIP());
+  } else if(m_clients[i]->isUdpBinded() && m_clients[i]->isUdpBindedValidated()) {
     i_netAction->send(m_clients[i]->tcpSocket(), &m_udpsd, m_udpPacket, m_clients[i]->udpRemoteIP());
   } else {
     i_netAction->send(m_clients[i]->tcpSocket(), NULL, NULL, NULL);
@@ -818,6 +859,22 @@ void ServerThread::sendToAllClientsMarkedToPlay(NetAction* i_netAction, int i_sr
   }
 }
 
+void ServerThread::sendToAllClientsHavingProtocol(int i_protocol, NetAction* i_netAction_lt, NetAction* i_netAction_ge, int i_src, int i_subsrc, int i_except) {
+  for(unsigned int i=0; i<m_clients.size(); i++) {
+    try {
+      if((int)i != i_except) {
+	if(m_clients[i]->protocolVersion() < i_protocol) {
+	  sendToClient(i_netAction_lt, i, i_src, i_subsrc);
+	} else {
+	  sendToClient(i_netAction_ge, i, i_src, i_subsrc);
+	}
+      }
+    } catch(Exception &e) {
+      // don't remove the client while removeclient function can call sendToAllClients ...
+    }
+  }
+}
+
 void ServerThread::acceptClient() {
   TCPsocket csd;
   IPaddress *tcpRemoteIP;
@@ -924,6 +981,24 @@ void ServerThread::manageClientUDP() {
 	      if(m_clients[i]->udpBindKey() == ((NA_udpBind*)v_netAction)->key()) {
 		//LogInfo("UDP bind key received via UDP: %s", ((NA_udpBind*)v_netAction)->key().c_str());
 		m_clients[i]->bindUdp(m_udpPacket->address);
+		if(m_clients[i]->protocolVersion() >= 3) { // don't send if the version is lower because the client will not understand -- udp could not work in that case
+		  LogInfo("server: i can receive udp from the client %i", i);
+
+		  NA_udpBindValidation nabv;
+		  try {
+		    sendToClient(&nabv, i, -1, 0);
+		  } catch(Exception &e) {
+		  }
+
+		  NA_udpBind nab("XMS");
+		  try {
+		    // send the packet 3 times to get more change it arrives
+		    for(unsigned int j=0; j<3; j++) {
+		      sendToClient(&nab, i, -1, 0, true);
+		    }
+		  } catch(Exception &e) {
+		  }
+		}
 		break; // stop : only one client
 	      }
 	    }
@@ -953,6 +1028,7 @@ bool ServerThread::manageAction(NetAction* i_netAction, unsigned int i_client) {
   case TNA_udpBindQuery:
   case TNA_serverError:
   case TNA_changeClients:
+  case TNA_clientsNumber:
   case TNA_prepareToPlay:
   case TNA_prepareToGo:
   case TNA_killAlert:
@@ -964,10 +1040,28 @@ bool ServerThread::manageAction(NetAction* i_netAction, unsigned int i_client) {
     }
     break;
 
+  case TNA_clientsNumberQuery:
+    {
+      // only clients having a name
+      int n=0;
+      for(unsigned int i=0; i<m_clients.size(); i++) {
+	if(m_clients[i]->name() != "") {
+	  n++;
+	}
+      }
+
+      NA_clientsNumber nacn(n);
+      try {
+	sendToClient(&nacn, i_client, -1, 0);
+      } catch(Exception &e) {
+      }
+    }
+    break;
+
   case TNA_clientInfos:
     {
       // check protocol version
-      if(((NA_clientInfos*)i_netAction)->protocolVersion() != XM_NET_PROTOCOL_VERSION) {
+      if(((NA_clientInfos*)i_netAction)->protocolVersion() > XM_NET_PROTOCOL_VERSION) {
 	NA_serverError na(UNTRANSLATED_GAMETEXT_SERVER_PROTOCOL_VERSION_INCOMPATIBLE);
 	na.setSource(-1, 0);
 	try {
@@ -981,7 +1075,9 @@ bool ServerThread::manageAction(NetAction* i_netAction, unsigned int i_client) {
       //LogInfo("Protocol version of client %i is %i", i_client, ((NA_clientInfos*)i_netAction)->protocolVersion());
       //LogInfo("UDP bind key of client %i is %s", i_client, ((NA_clientInfos*)i_netAction)->udpBindKey().c_str());
       m_clients[i_client]->setUdpBindKey(((NA_clientInfos*)i_netAction)->udpBindKey());
-      
+      m_clients[i_client]->setProtocolVersion(((NA_clientInfos*)i_netAction)->protocolVersion());
+      m_clients[i_client]->setXmVersion(((NA_clientInfos*)i_netAction)->xmversion());
+
       // query bind udp
       NA_udpBindQuery naq;
       try {
@@ -989,7 +1085,7 @@ bool ServerThread::manageAction(NetAction* i_netAction, unsigned int i_client) {
       } catch(Exception &e) {
       }
 
-      // send the current clients list to the client only when the client has a name
+      // send the current clients list to the client only when the client has a name (or TNA_clientsNumber)
       NA_changeClients nacc;
       NetInfosClient nic;
       try {
@@ -1005,10 +1101,57 @@ bool ServerThread::manageAction(NetAction* i_netAction, unsigned int i_client) {
       }
     }
     break;
-      
-  case TNA_chatMessage:
+
+  case TNA_udpBindValidation:
     {
-      sendToAllClients(i_netAction, m_clients[i_client]->id(), i_netAction->getSubSource(), i_client);
+      if(m_clients[i_client]->isUdpBindedValidated() == false) {
+	LogInfo("server: the client %i can receive udp from me", i_client);
+	m_clients[i_client]->validUdpBind();
+      }
+    }
+    break;
+
+  case TNA_chatMessage: /* old message format, convert into the new one */
+    {
+      // retrieve message
+      std::string v_msg = ((NA_chatMessage*)i_netAction)->getMessage();
+      std::vector<int> v_pp;
+      NA_chatMessagePP napp(v_msg, "", v_pp); 
+      sendToAllClientsHavingProtocol(4, i_netAction, &napp, m_clients[i_client]->id(), i_netAction->getSubSource(), i_client);
+    }
+    break;
+
+  case TNA_chatMessagePP:
+    {
+      NA_chatMessagePP* na = ((NA_chatMessagePP*)i_netAction);
+      NA_chatMessage na_old = NA_chatMessage(((NA_chatMessagePP*)i_netAction)->getMessage(), "");
+
+      if(na->privatePeople().size() <= XM_SERVER_MAXIMUM_MULTI_PRIVATE_MESSAGE) {
+	if(na->privatePeople().size() == 0) {
+	  // public
+	  sendToAllClientsHavingProtocol(4, &na_old, na, m_clients[i_client]->id(), i_netAction->getSubSource(), i_client);
+	} else {
+	  // private
+	  for(unsigned int i=0; i<na->privatePeople().size(); i++) {
+	    for(unsigned int j=0; j<m_clients.size(); j++) {
+	      if(((int)m_clients[j]->id()) == na->privatePeople()[i]) {
+		if(m_clients[j]->protocolVersion() < 4) {
+		  sendToClient(&na_old, j, m_clients[i_client]->id(), i_netAction->getSubSource());
+		} else {
+		  sendToClient(na, j, m_clients[i_client]->id(), i_netAction->getSubSource());
+		}
+	      }
+	    }
+	  }
+	}
+      } else {
+	std::ostringstream v_str;
+	v_str << "Private messages to maximum " << XM_SERVER_MAXIMUM_MULTI_PRIVATE_MESSAGE << " people are allowed";
+	try {
+	  sendMsgToClient(i_client, v_str.str());
+	} catch(Exception &e) {
+	}
+      }
     }
     break;
     
@@ -1107,6 +1250,25 @@ bool ServerThread::manageAction(NetAction* i_netAction, unsigned int i_client) {
 
   case TNA_clientMode:
     {
+      // 
+      if(((NA_clientMode*)i_netAction)->mode() == NETCLIENT_SLAVE_MODE) {
+	// in this mode, the client must satisfy the minimum version allowed by the server
+	if(m_clients[i_client]->protocolVersion() < XM_SERVER_SLAVE_MODE_MIN_PROTOCOL_VERSION) {
+	  std::ostringstream v_str;
+	  v_str << "Protocol version " << XM_SERVER_SLAVE_MODE_MIN_PROTOCOL_VERSION << " is required on this mode.";
+	  v_str << " Your version is " << ((NA_clientMode*)i_netAction)->mode() << ".\n";
+	  v_str << " Update X-Moto or play simple ghost mode.";
+	  
+	  NA_serverError na(v_str.str());
+	  try {
+	    sendToClient(&na, i_client, -1, 0);
+	  } catch(Exception &e) {
+	    /* ok, no pb */
+	  }
+	  
+	  return false;
+	}
+      }
       m_clients[i_client]->setMode(((NA_clientMode*)i_netAction)->mode());
     }
     break;
@@ -1128,8 +1290,6 @@ void ServerThread::manageSrvCmd(unsigned int i_client, const std::string& i_cmd)
   char **v_result;
   unsigned int nrow;
 
-  LogInfo("server cmd: %s", i_cmd.c_str());
-
   utf8::utf8_split(i_cmd, " ", v_args);
 
   if(v_args.size() < 1) {
@@ -1144,6 +1304,7 @@ void ServerThread::manageSrvCmd(unsigned int i_client, const std::string& i_cmd)
       v_answer += "logout: disconnect from the server\n";
       v_answer += "changepassword <password>: change your password\n";
       v_answer += "lsplayers: list connected players\n";
+      v_answer += "lsxmversions: list xmoto versions used by players\n";
       v_answer += "lsbans: list banned players\n";
 
       std::ostringstream v_n;
@@ -1154,6 +1315,7 @@ void ServerThread::manageSrvCmd(unsigned int i_client, const std::string& i_cmd)
       v_answer += "addadmin <id player> <password>: add player <id player> as admin\n";
       v_answer += "rmadmin <id admin>: remove admin\n";
       v_answer += "stats: server statistics\n";
+      v_answer += "msg <msg>: message to players\n";
     }
 
   } else if(v_args[0] == "banner") {
@@ -1229,11 +1391,6 @@ void ServerThread::manageSrvCmd(unsigned int i_client, const std::string& i_cmd)
 		 m_pDb->getResult(v_result, 2, i, 0),
 		 m_pDb->getResult(v_result, 2, i, 1));
 	v_answer += v_adminstr;
-	if(i % 3 == 2) {
-	  v_answer += "\n";
-	}
-      }
-      if(nrow %3 != 2) {
 	v_answer += "\n";
       }
       m_pDb->read_DB_free(v_result);
@@ -1265,11 +1422,41 @@ void ServerThread::manageSrvCmd(unsigned int i_client, const std::string& i_cmd)
     if(v_args.size() != 1) {
       v_answer += "lsplayers: invalid arguments\n";
     } else {
-      char v_clientstr[38];
+      char v_clientstr[48];
+      std::string v_mode;
       for(unsigned int i=0; i<m_clients.size(); i++) {
-	snprintf(v_clientstr, 38, "%5u: %-12s %-17s",
+	switch(m_clients[i]->mode()) {
+	case NETCLIENT_GHOST_MODE:
+	  v_mode = "GHOST";
+	  break;
+	case NETCLIENT_SLAVE_MODE:
+	  v_mode = "SLAVE";
+	  break;
+	default:
+	  v_mode = "UNKWN";
+	}
+
+	snprintf(v_clientstr, 48, "%5u: %-12s %-5s %-21s",
 		 m_clients[i]->id(), m_clients[i]->name().c_str(),
-		 ("(" + XMNet::getIp(m_clients[i]->tcpRemoteIP()) + ")").c_str());
+		 v_mode.c_str(),
+		 ("(" + XMNet::getIp(m_clients[i]->tcpRemoteIP()) + "/" + 
+		  (m_clients[i]->isUdpBindedValidated() ? "UDP" : "TCP")
+		  + ")").c_str());
+	v_answer += v_clientstr;
+	v_answer += "\n";
+      }
+    }
+
+  } else if(v_args[0] == "lsxmversions") {
+    if(v_args.size() != 1) {
+      v_answer += "lsxmversions: invalid arguments\n";
+    } else {
+      char v_clientstr[55];
+      for(unsigned int i=0; i<m_clients.size(); i++) {
+	snprintf(v_clientstr, 55, "%5u: %-12s (%-26s ; %3i)",
+		 m_clients[i]->id(), m_clients[i]->name().c_str(),
+		 m_clients[i]->xmversion().c_str(),
+		 m_clients[i]->protocolVersion());
 	v_answer += v_clientstr;
 	if(i % 3 == 2) {
 	  v_answer += "\n";
@@ -1336,6 +1523,28 @@ void ServerThread::manageSrvCmd(unsigned int i_client, const std::string& i_cmd)
     } else {
       v_answer += "start time : " + m_startTimeStr + "\n";
       v_answer += NetAction::getStats() + "\n";
+      v_answer += ActionReader::getStats() + "\n";
+    }
+
+  } else if(v_args[0] == "msg") {
+    if(v_args.size() < 2) {
+      v_answer += "msg: invalid arguments\n";
+    } else {
+      std::string v_msg;
+      for(unsigned int i=1; i<v_args.size(); i++) {
+	v_msg += v_args[i];
+	if(i<v_args.size()-1) {
+	  v_msg += " ";
+	}
+      }
+
+      std::vector<int> v_private_people;
+      NA_chatMessagePP na(v_msg.c_str(), "server", v_private_people);
+      NA_chatMessage na_old(v_msg.c_str(), "server");
+      try {
+	sendToAllClientsHavingProtocol(4, &na_old, &na, -1, 0, i_client);
+      } catch(Exception &e) {
+      }
     }
 
   } else {
@@ -1397,3 +1606,21 @@ void ServerThread::SP2_addPointsToClient(unsigned int i_client, unsigned int i_p
 bool ServerThread::acceptConnections() const {
   return m_acceptConnections;
 }
+
+void ServerThread::sendMsgToClient(unsigned int i_client, const std::string& i_msg) {
+  std::vector<int> v_private_people;
+ 
+  if(m_clients[i_client]->protocolVersion() < 4) {
+    NA_chatMessage na(i_msg, "server");
+    try {
+      sendToClient(&na, i_client, -1, 0);
+    } catch(Exception &e) {
+    }
+  } else {
+    NA_chatMessagePP na(i_msg, "server", v_private_people);
+    try {
+      sendToClient(&na, i_client, -1, 0);
+    } catch(Exception &e) {
+    }
+  }
+}
