@@ -43,6 +43,8 @@ class NetSClient {
   IPaddress* udpRemoteIP();
   bool isUdpBinded() const;
   void bindUdp(IPaddress i_udpIPAdress);
+  void validUdpBind();
+  bool isUdpBindedValidated() const;
   void unbindUdp();
 
   ActionReader* tcpReader;
@@ -50,6 +52,12 @@ class NetSClient {
   void setUdpBindKey(const std::string& i_key);
   std::string udpBindKey() const;
 
+  void setProtocolVersion(int i_protocolVersion);
+  int protocolVersion() const;
+
+  void setXmVersion(const std::string& i_xmversion);
+  std::string xmversion() const;
+
   void setName(const std::string& i_name);
   std::string name() const;
   void setMode(NetClientMode i_mode);
@@ -90,8 +98,11 @@ class NetSClient {
   IPaddress m_tcpRemoteIP;
   IPaddress m_udpRemoteIP;
   bool m_isUdpBinded;
+  bool m_isUdpBindedValidated;
   std::string m_udpBindKey;
   std::string m_name;
+  int m_protocolVersion;
+  std::string m_xmversion;
   std::string m_playingLevelId;
   int m_lastActivTime;
   int m_lastInactivTimeAlert;
@@ -149,7 +160,9 @@ class ServerThread : public XMThread {
   void sendToAllClients(NetAction* i_netAction, int i_src, int i_subsrc, int i_except = -1);
   void sendToAllClientsHavingMode(NetClientMode i_mode, NetAction* i_netAction, int i_src, int i_subsrc, int i_except = -1);
   void sendToAllClientsMarkedToPlay(NetAction* i_netAction, int i_src, int i_subsrc, int i_except = -1);
-  void sendToClient(NetAction* i_netAction, unsigned int i, int i_src, int i_subsrc);
+  void sendToAllClientsHavingProtocol(int i_protocol, NetAction* i_netAction_lt, NetAction* i_netAction_ge, int i_src, int i_subsrc, int i_except);
+  void sendToClient(NetAction* i_netAction, unsigned int i, int i_src, int i_subsrc, bool i_forceUdp = false);
+  void sendMsgToClient(unsigned int i_client, const std::string& i_msg);
   void removeClient(unsigned int i);
   unsigned int nbClientsInMode(NetClientMode i_mode);
 
