@@ -33,14 +33,13 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "../net/NetClient.h"
 #include "StateHelp.h"
 #include "../Renderer.h"
+#include "../Input.h"
 
 #define MENU_SHADING_TIME 0.3
 #define MENU_SHADING_VALUE 150
 
 GameState::GameState(bool drawStateBehind,
-		     bool updateStatesBehind,
-		     bool i_doShade,
-		     bool i_doShadeAnim)
+		     bool updateStatesBehind)
 {
   m_isHide             = false;
   m_drawStateBehind    = drawStateBehind;
@@ -57,10 +56,6 @@ GameState::GameState(bool drawStateBehind,
   m_updatePeriod       = 0;
   m_updateCounter      = 0;
 
-  // shade
-  m_doShade     = i_doShade;
-  m_doShadeAnim = i_doShadeAnim;
-
   m_showCursor = true;
 
   StateManager::instance()->registerAsObserver("CLIENT_DISCONNECTED_BY_ERROR", this);
@@ -83,6 +78,14 @@ GameState::~GameState()
   SDL_DestroyMutex(m_commandsMutex);
 }
 
+void GameState::setScreen(const RenderSurface& i_screen) {
+  m_screen = i_screen;
+}
+
+RenderSurface* GameState::getScreen() {
+  return &m_screen;
+}
+
 bool GameState::doUpdate()
 {
   m_updateCounter += 1.0;
@@ -95,20 +98,13 @@ bool GameState::doUpdate()
 }
 
 void GameState::enter() {
-  m_nShadeTime = GameApp::getXMTime();
-  if(XMSession::instance()->ugly() == true)
-    GameApp::instance()->displayCursor(showCursor());
 }
 
 void GameState::enterAfterPop()
 {
-  if(XMSession::instance()->ugly() == true)
-    GameApp::instance()->displayCursor(showCursor());
 }
 
 bool GameState::render() {
-  //  left the shade call here to not do damage on other things I even not know about
-  GameRenderer::instance()->setScreenShade(m_doShade, m_doShadeAnim, m_nShadeTime);
   return renderOverShadow();
 }
 
@@ -116,22 +112,61 @@ bool GameState::renderOverShadow() {
   return true;
 }
 
-std::string GameState::getId() const {
-  return m_id;
+void GameState::setStateId(const std::string& i_id) {
+  m_stateId = i_id;
 }
 
-void GameState::setId(const std::string& i_id) {
-  m_id = i_id;
+std::string GameState::getStateId() const {
+  return m_stateId;
+}
+
+void GameState::setStateType(const std::string& i_type) {
+  m_stateType = i_type;
+}
+
+std::string GameState::getStateType() const {
+  return m_stateType;
 }
 
 void GameState::sendFromMessageBox(const std::string& i_id, UIMsgBoxButton i_button, const std::string& i_input) {
   if(i_id == "CHATMESSAGE"){
     if(i_button == UI_MSGBOX_OK) {
-      NA_chatMessage na(i_input);
-      try {
-	NetClient::instance()->send(&na, 0);
-	SysMessage::instance()->addConsoleLine(XMSession::instance()->profile() + ": " + i_input);
-      } catch(Exception &e) {
+      std::vector<int> v_private_people;
+      std::vector<std::string> v_not_found_people;
+      NetClient::instance()->fillPrivatePeople(i_input, "*: ", v_private_people, v_not_found_people);
+
+      // if the message seems private (people name found) but cannot be associated to players => it becomes public ; don't sent it, just alert the sender
+      if(v_private_people.size() == 0 && v_not_found_people.size() > 0) {
+	std::string v_msg;
+	v_msg = GAMETEXT_MESSAGE_NOT_SENT_UNKNOWN_PLAYERS;
+	for(unsigned int i=0; i<v_not_found_people.size(); i++) {
+	  v_msg += " " + v_not_found_people[i] + " ;";
+	}
+	SysMessage::instance()->displayError(v_msg);
+      } else {
+	NA_chatMessagePP na(i_input, XMSession::instance()->profile(), v_private_people);
+	try {
+	  NetClient::instance()->send(&na, 0);	
+	  
+	  /* and not i_input because na can have transformations */
+	  if(v_private_people.size() == 0) { /* public */
+	    SysMessage::instance()->addConsoleLine(NetClient::instance()->getDisplayMessage(na.getMessage(), XMSession::instance()->profile()));
+	  } else { /* private */
+	    SysMessage::instance()->addConsoleLine(NetClient::instance()->getDisplayMessage(na.getMessage(), XMSession::instance()->profile()), CLT_PRIVATE);
+	  }
+	} catch(Exception &e) {
+	}
+
+	// display unknown players, but send the message to the others
+	if(v_not_found_people.size() > 0) {
+	  std::string v_msg;
+	  v_msg = GAMETEXT_UNKNOWN_PLAYERS;
+	  for(unsigned int i = 0; i<v_not_found_people.size(); i++) {
+	    v_msg += " " + v_not_found_people[i] + " ;";
+	  }
+	  SysMessage::instance()->displayError(v_msg);
+	}
+
       }
     }
   }
@@ -202,28 +237,28 @@ void GameState::addCommand(std::string cmd, std::string args)
 void GameState::xmKey(InputEventType i_type, const XMKey& i_xmkey) {
   GameApp* gameApp = GameApp::instance();
 
-  if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_F1, KMOD_NONE)) {
+  if(i_type == INPUT_DOWN && i_xmkey == (*InputHandler::instance()->getGlobalKey(INPUT_HELP))) {
     if (StateManager::instance()->isThereASuchState("StateHelp") == false) StateManager::instance()->pushState(new StateHelp(true,false,true, false));
     return;
   }
 
-  if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_F12, KMOD_NONE)) {
+  if(i_type == INPUT_DOWN && i_xmkey == (*InputHandler::instance()->getGlobalKey(INPUT_SCREENSHOT))) {
     gameApp->gameScreenshot();
     return; 
   }
 
-  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_F8, KMOD_NONE)) {
+  else if(i_type == INPUT_DOWN && i_xmkey == (*InputHandler::instance()->getGlobalKey(INPUT_SWITCHWWWACCESS))) {
     gameApp->enableWWW(XMSession::instance()->www() == false);
     StateManager::instance()->sendAsynchronousMessage("CHANGE_WWW_ACCESS");
     return;
   }
 
-  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_F7, KMOD_NONE)) {
+  else if(i_type == INPUT_DOWN && i_xmkey == (*InputHandler::instance()->getGlobalKey(INPUT_SWITCHFPS))) {
     gameApp->enableFps(XMSession::instance()->fps() == false);
     return;
   }
 
-  else if(i_type == INPUT_DOWN && i_xmkey == InputHandler::instance()->getSwitchUglyMode()) {
+  else if(i_type == INPUT_DOWN && i_xmkey == (*InputHandler::instance()->getGlobalKey(INPUT_SWITCHUGLYMODE))) {
     gameApp->switchUglyMode(XMSession::instance()->ugly() == false);
     if(XMSession::instance()->ugly()) {
       SysMessage::instance()->displayText(SYS_MSG_UGLY_MODE_ENABLED);
@@ -231,32 +266,44 @@ void GameState::xmKey(InputEventType i_type, const XMKey& i_xmkey) {
       SysMessage::instance()->displayText(SYS_MSG_UGLY_MODE_DISABLED);
     }
     
-    if(XMSession::instance()->ugly() == true)
-      GameApp::instance()->displayCursor(showCursor());
-    
     return;
   }
 
-  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_F10, KMOD_NONE)) {
-    gameApp->switchTestThemeMode(XMSession::instance()->testTheme() == false);
-    if(XMSession::instance()->testTheme()) {
+   else if(i_type == INPUT_DOWN && i_xmkey == (*InputHandler::instance()->getGlobalKey(INPUT_SWITCHGFXQUALITYMODE))) {
+    /* Toggle GFX Modes */
+    if(XMSession::instance()->gameGraphics() == GFX_LOW) {
+      XMSession::instance()->setGameGraphics(GFX_MEDIUM);
+      SysMessage::instance()->displayText(SYS_MSG_GFX_MEDIUM_ACTIVATED);
+    }
+    else if(XMSession::instance()->gameGraphics() == GFX_MEDIUM) {
+      XMSession::instance()->setGameGraphics(GFX_HIGH);
+      SysMessage::instance()->displayText(SYS_MSG_GFX_HIGH_ACTIVATED);
+    }
+    else {
+      XMSession::instance()->setGameGraphics(GFX_LOW);
+      SysMessage::instance()->displayText(SYS_MSG_GFX_LOW_ACTIVATED);
+    }
+    return;
+  }
+
+  else if(i_type == INPUT_DOWN && i_xmkey == (*InputHandler::instance()->getGlobalKey(INPUT_SWITCHGFXMODE))) {
+    /* F11 toggles TestThemeMode, UglyOver Mode and Normal Mode */
+    if(XMSession::instance()->testTheme() == false && XMSession::instance()->uglyOver() == false) {
+      gameApp->switchTestThemeMode(true);
       SysMessage::instance()->displayText(SYS_MSG_THEME_MODE_ENABLED);
-    } else {
-      SysMessage::instance()->displayText(SYS_MSG_THEME_MODE_DISABLED);
-    }
-    return;
-  }
-
-  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_F11, KMOD_NONE)) {
-    gameApp->switchUglyOverMode(XMSession::instance()->uglyOver() == false);
-    if(XMSession::instance()->uglyOver()) {
+    } 
+    else if(XMSession::instance()->testTheme() == true) {
+      gameApp->switchTestThemeMode(false);
+      gameApp->switchUglyOverMode(true);
       SysMessage::instance()->displayText(SYS_MSG_UGLY_OVER_MODE_ENABLED);
-    } else {
-      SysMessage::instance()->displayText(SYS_MSG_UGLY_OVER_MODE_DISABLED);
+    }
+    else {
+      gameApp->switchUglyOverMode(false);
+      SysMessage::instance()->displayText(SYS_MSG_NORMAL_MODE_ENABLED);
     }
     return;
   }
-
+ 
   else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_RETURN, KMOD_LALT)) {
     gameApp->getDrawLib()->toogleFullscreen();
     XMSession::instance()->setWindowed(XMSession::instance()->windowed() == false);
@@ -292,26 +339,59 @@ void GameState::xmKey(InputEventType i_type, const XMKey& i_xmkey) {
   }
 
   // net chat
-  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_c, KMOD_LCTRL) && NetClient::instance()->isConnected()) {
-    if(StateManager::instance()->isThereASuchStateId("CHATMESSAGE") == false) { // do not open several chat box
-      StateMessageBox* v_msgboxState = new StateMessageBox(NULL, std::string(GAMETEXT_CHATMESSAGE) + ":",
-							   UI_MSGBOX_OK|UI_MSGBOX_CANCEL, true, "", false, true, false, true);
-      v_msgboxState->setId("CHATMESSAGE");
+  else if(i_type == INPUT_DOWN && i_xmkey == (*InputHandler::instance()->getGlobalKey(INPUT_CHAT)) &&
+	  NetClient::instance()->isConnected()) {
+    if(StateManager::instance()->isThereASuchStateType("CHATMESSAGE") == false) { // do not open several chat box
+
+      // add player names
+      std::vector<std::string> clientList;
+      NetClient::instance()->getOtherClientsNameList(clientList, "*: ");
+      NetClient::instance()->getOtherClientsNameList(clientList, ": ");
+
+      // add transformations like /me
+      NetClient::instance()->addChatTransformations(clientList, " ");
+
+      StateMessageBox* v_msgboxState = new StateMessageBox(NULL, clientList, std::string(GAMETEXT_CHATMESSAGE) + ":", UI_MSGBOX_OK|UI_MSGBOX_CANCEL, true, "", false, true, false, true);
+      v_msgboxState->setMsgBxId("CHATMESSAGE");
+      v_msgboxState->setStateType("CHATMESSAGE");
+      v_msgboxState->setHelp(GAMETEXT_CHATBOXHELP);
       StateManager::instance()->pushState(v_msgboxState);
       return; 
     }
   }
 
   else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_s, (SDLMod) (KMOD_LCTRL | KMOD_LALT)) && NetClient::instance()->isConnected()) {
-    if(StateManager::instance()->isThereASuchStateId("SERVERCONSOLE") == false) { // do not open several console
+    if(StateManager::instance()->isThereASuchState("StateServerConsole") == false) { // do not open several console
       StateServerConsole* v_console = new StateServerConsole(true, true);
-      v_console->setId("SERVERCONSOLE");
       StateManager::instance()->pushState(v_console);
       return; 
     }
   }
 
-  else if(i_type == INPUT_DOWN && i_xmkey == InputHandler::instance()->getShowConsole()) {
+  else if(i_type == INPUT_DOWN && i_xmkey == (*InputHandler::instance()->getGlobalKey(INPUT_SHOWCONSOLE))) {
     SysMessage::instance()->showConsole();
   }
+
+  else if(i_type == INPUT_DOWN && i_xmkey == (*InputHandler::instance()->getGlobalKey(INPUT_CONSOLEHISTORYPLUS))) {
+    SysMessage::instance()->alterConsoleSize(+1);
+    XMSession::instance()->setConsoleSize(SysMessage::instance()->consoleSize());
+    SysMessage::instance()->showConsole();
+  }
+
+  else if(i_type == INPUT_DOWN && i_xmkey == (*InputHandler::instance()->getGlobalKey(INPUT_CONSOLEHISTORYMINUS))) {
+    SysMessage::instance()->alterConsoleSize(-1);
+    XMSession::instance()->setConsoleSize(SysMessage::instance()->consoleSize());
+    SysMessage::instance()->showConsole();
+  }
+
+  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_t, KMOD_LCTRL)) {
+    XMSession::instance()->setEnableTrailCam(!XMSession::instance()->enableTrailCam());
+    StateManager::instance()->sendAsynchronousMessage("CHANGE_TRAILCAM");
+
+    if(XMSession::instance()->enableTrailCam()) {
+      SysMessage::instance()->displayText(SYS_MSG_TRAILCAM_ACTIVATED);
+    } else {
+      SysMessage::instance()->displayText(SYS_MSG_TRAILCAM_DEACTIVATED);
+    }
+  }
 }
