@@ -30,7 +30,6 @@ class Universe;
 class StateDeadMenu : public StateMenu {
   public:
   StateDeadMenu(Universe* i_universe,
-		bool i_doShadeAnim,
 		bool drawStateBehind    = true,
 		bool updateStatesBehind = false
 		);
@@ -54,7 +53,7 @@ class StateDeadMenu : public StateMenu {
   private:
   /* GUI */
   static UIRoot* m_sGUI;
-  static void createGUIIfNeeded();
+  static void createGUIIfNeeded(RenderSurface* i_screen);
 
   std::string m_replayName; // to save temporarly the replay name
 };
