@@ -34,8 +34,7 @@ StatePause::StatePause(Universe* i_universe,
 		       bool drawStateBehind,
 		       bool updateStatesBehind):
   StateMenu(drawStateBehind,
-	    updateStatesBehind,
-	    true)
+	    updateStatesBehind)
 {
   m_name  = "StatePause";
   m_universe = i_universe;
@@ -71,7 +70,7 @@ void StatePause::enter()
 
   Sound::togglePauseMusic();
   
-  createGUIIfNeeded();
+  createGUIIfNeeded(&m_screen);
   m_GUI = m_sGUI;
 
   /* reset the playnext button */
@@ -131,18 +130,18 @@ void StatePause::checkEvents() {
     pQuitButton->setClicked(false);
 
     StateMessageBox* v_msgboxState = new StateMessageBox(this, GAMETEXT_QUITMESSAGE, UI_MSGBOX_YES|UI_MSGBOX_NO);
-    v_msgboxState->setId("QUIT");
+    v_msgboxState->setMsgBxId("QUIT");
     StateManager::instance()->pushState(v_msgboxState);
   }
 }
 
 void StatePause::xmKey(InputEventType i_type, const XMKey& i_xmkey) {
-  if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_ESCAPE, KMOD_NONE)) {
+  if(i_type == INPUT_DOWN && i_xmkey == (*InputHandler::instance()->getGlobalKey(INPUT_PLAYINGPAUSE))) {
     /* quit this state */
     m_requestForEnd = true;
   }
 
-  else if(i_type == INPUT_DOWN && i_xmkey == InputHandler::instance()->getSwitchFavorite()) {
+  else if(i_type == INPUT_DOWN && i_xmkey == (*InputHandler::instance()->getGlobalKey(INPUT_SWITCHFAVORITE))) {
     if(m_universe != NULL) {
       if(m_universe->getScenes().size() > 0) { // just add the first world
 	GameApp::instance()->switchLevelToFavorite(m_universe->getScenes()[0]->getLevelSrc()->Id(), true);
@@ -151,7 +150,7 @@ void StatePause::xmKey(InputEventType i_type, const XMKey& i_xmkey) {
     }
   }
 
-  else if(i_type == INPUT_DOWN && i_xmkey == InputHandler::instance()->getSwitchBlacklist()) {
+  else if(i_type == INPUT_DOWN && i_xmkey == (*InputHandler::instance()->getGlobalKey(INPUT_SWITCHBLACKLIST))) {
     if(m_universe != NULL) {
       if(m_universe->getScenes().size() > 0) { // just blacklist the first world
 	GameApp::instance()->switchLevelToBlacklist(m_universe->getScenes()[0]->getLevelSrc()->Id(), true);
@@ -172,7 +171,7 @@ void StatePause::clean() {
   }
 }
 
-void StatePause::createGUIIfNeeded() {
+void StatePause::createGUIIfNeeded(RenderSurface* i_screen) {
   UIButton *v_button;
   UIFrame  *v_frame;
 
@@ -181,16 +180,16 @@ void StatePause::createGUIIfNeeded() {
 
   DrawLib* drawLib = GameApp::instance()->getDrawLib();
 
-  m_sGUI = new UIRoot();
+  m_sGUI = new UIRoot(i_screen);
   m_sGUI->setFont(drawLib->getFontSmall()); 
   m_sGUI->setPosition(0, 0,
-		      drawLib->getDispWidth(),
-		      drawLib->getDispHeight());
+		      i_screen->getDispWidth(),
+		      i_screen->getDispHeight());
   
   
   v_frame = new UIFrame(m_sGUI,
-			drawLib->getDispWidth()/2  - 400/2,
-			drawLib->getDispHeight()/2 - 540/2,
+			i_screen->getDispWidth()/2  - 400/2,
+			i_screen->getDispHeight()/2 - 540/2,
 			"", 400, 540);
   v_frame->setID("PAUSE_FRAME");
   v_frame->setStyle(UI_FRAMESTYLE_MENU);
