@@ -43,7 +43,7 @@ private:
 
   /* GUI */
   static UIRoot* m_sGUI;
-  static void createGUIIfNeeded();
+  static void createGUIIfNeeded(RenderSurface* i_screen);
   void updateGUI();
 };
 
