@@ -25,7 +25,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 class StateWaitServerInstructions : public GameState {
 public:
-  StateWaitServerInstructions(const std::string& i_id);
+  StateWaitServerInstructions();
   virtual ~StateWaitServerInstructions();
 
   virtual bool render();
