@@ -32,6 +32,9 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "../helpers/Log.h"
 #include "PhysicsSettings.h"
 
+#define MINIMUM_VELOCITY_TO_GET_MAXIMUM_DEATH_SOUND 80.0
+#define MINIMUM_SOUND_VOLUME 0.2
+
 BikeState::BikeState(PhysicsSettings* i_physicsSettings) {
   m_bikeParameters = new BikeParameters(i_physicsSettings);
   m_bikeAnchors    = new BikeAnchors();
@@ -187,6 +190,7 @@ Biker::Biker(PhysicsSettings* i_physicsSettings,
   m_physicsSettings = i_physicsSettings;
   m_bikeState   = new BikeState(m_physicsSettings);
   m_localNetId = -1;
+  m_nbRenderedFrames = 0;
 
   /* sound engine */
   try {
@@ -373,6 +377,10 @@ Vector2f Biker::getWheelSpinDir() {
   return m_WheelSpinDir;
 }
 
+void Biker::initWheelDetach() {
+  m_wheelDetach = false;
+}
+
 void Biker::initToPosition(Vector2f i_position, DriveDir i_direction, Vector2f i_gravity) {
 }
 
@@ -391,6 +399,29 @@ void Biker::setFinished(bool i_value, int i_finishTime) {
 void Biker::setDead(bool i_value, int i_deadTime) {
   m_dead = i_value;
   m_deadTime = i_deadTime;
+ 
+  if(m_dead == false) {
+    return;
+  }
+ 
+  /* Play the DIE!!! sound */
+  try {
+    float v_deathVolume;
+    float v_maxVelocity = 0.0;
+
+    if(this->getBikeLinearVel() > v_maxVelocity) {
+      v_maxVelocity = this->getBikeLinearVel();
+    }
+    // make deathVolume dependant of the velocity of the fastest of the players
+    v_deathVolume = v_maxVelocity / MINIMUM_VELOCITY_TO_GET_MAXIMUM_DEATH_SOUND + MINIMUM_SOUND_VOLUME;
+    if(v_deathVolume > 1.0) {
+      v_deathVolume = 1.0;
+    }
+    Sound::playSampleByName(Theme::instance()->getSound("Headcrash")->FilePath(), v_deathVolume);
+  } 
+  catch(Exception &e) {
+  }
+
 }
 
 bool Biker::isFinished() const {
@@ -667,6 +698,18 @@ float Biker::changeDirPer() const {
   return m_changeDirPer;
 }
 
+unsigned int Biker::getNbRenderedFrames() const {
+  return m_nbRenderedFrames;
+}
+
+void Biker::addNbRenderedFrames() {
+  m_nbRenderedFrames++;
+}
+
+bool Biker::isStateInitialized() const {
+  return true; // by default, consider that the biker is iniialized
+}
+
 void Biker::clearStates() {
   /* BIKE_S */
   m_bikeState->CenterP = Vector2f(0,0);
@@ -719,4 +762,5 @@ void Biker::clearStates() {
 
 BikeController* Biker::getControler() {
   return NULL;
+
 }
