@@ -98,7 +98,7 @@ Block::Block(std::string i_id) {
   m_dynamicPositionCenter = Vector2f(0.0, 0.0);
   m_texture          = XM_DEFAULT_BLOCK_TEXTURE;
   m_isBBoxDirty      = true;
-  m_geom             = -1;
+  m_geom             = NULL;
   m_layer            = -1;
   m_edgeDrawMethod   = angle;
   m_edgeAngle        = DEFAULT_EDGE_ANGLE;
@@ -112,6 +112,7 @@ Block::Block(std::string i_id) {
   m_blendColor.setBlue(255);
   m_blendColor.setAlpha(255);
   m_edgeDefaultColor = DEFAULT_EDGE_BLENDCOLOR; // assign it or get random colors if no edge material is added
+  m_sprite = NULL;
 
   m_previousSavedPosition = DynamicPosition();
   m_previousSavedRotation = DynamicRotation();
@@ -136,8 +137,6 @@ Block::~Block() {
   }
   m_collisionLines.clear();
 
-  m_edgeGeoms.clear();
-  
  // for(unsigned int i=0; i<edgeGeomBlendColor.size(); i++) delete edgeGeomBlendColor[i];
   m_edgeMaterial.clear();
   
@@ -304,37 +303,6 @@ void Block::updatePhysics(int i_time, int timeStep, CollisionSystem* io_collisio
   }
   
   cpBodyResetForces(mBody);
-
-  /* record */
-  /* don't save any physic event for the moment
-  float v_diffRotation;
-
-  if(i_recorder != NULL) {
-    v_diffPosition = m_previousSavedPosition-DynamicPosition();
-    v_diffRotation = m_previousSavedRotation-DynamicRotation();
-    
-    // save position only if the block moved
-    if(v_diffPosition.length() > PHYS_MOVE_POSITION_MINIMUM) {
-      SceneEvent *pEvent = new MGE_SetBlockPos(i_time,
-					       Id(),
-					       DynamicPosition().x,
-					       DynamicPosition().y);
-      pEvent->serialize(*i_recorder);
-      delete pEvent;
-      m_previousSavedPosition = DynamicPosition();
-    }
-    
-    // save rotation only if the block moved
-    if(fabs(v_diffRotation) > PHYS_MOVE_ROTATION_MINIMUM) {
-      SceneEvent *pEvent = new MGE_SetBlockRotation(i_time,
-						    Id(),
-						    DynamicRotation());
-      pEvent->serialize(*i_recorder);
-      delete pEvent;
-      m_previousSavedRotation = DynamicRotation();
-    }
-  }
-  */
 }
 
 int Block::loadToPlay(CollisionSystem* io_collisionSystem, ChipmunkWorld* i_chipmunkWorld, PhysicsSettings* i_physicsSettings) {
@@ -449,14 +417,49 @@ int Block::loadToPlay(CollisionSystem* io_collisionSystem, ChipmunkWorld* i_chip
     io_collisionSystem->addBlockInLayer(this, m_layer);
   }
 
+  // check for forbidden combinations
+  //if(isDynamic() == false && m_layer != -1) {
+  //  // thrown exception
+  //  throw Exception("ERROR::Block "+this->Id()+" is dynamic but is in a layer.");
+  //}
+
   /* Compute */
   std::vector<BSPPoly *> &v_BSPPolys = v_BSPTree.compute();      
   
+
+  /* Load Textures */  
+  //first, lets see if there are animated textures, that come with 0.5.3
+  float scale = TextureScale() * 0.25;
+  Sprite* pSprite = Theme::instance()->getSprite(SPRITE_TYPE_ANIMATION_TEXTURE,
+						 this->getTexture());
+  //if there aren't, keep theme files compatible to old xmoto versions
+  if(pSprite == NULL) {
+    pSprite = Theme::instance()->getSprite(SPRITE_TYPE_TEXTURE,
+						 this->getTexture());  
+  }
+  
+  if(pSprite != NULL) 
+  {
+    m_sprite = pSprite;
+    try{
+      int v_textureSize = m_sprite->getTextureSize();
+      if(v_textureSize < 256 && v_textureSize != 0)
+	// divide by texturesize to prevent upscaling of textures < 256x256
+	scale *= (256/v_textureSize);
+    } catch(Exception& e) {
+      ;
+    }
+  }
+  else {
+    LogWarning("Texture %s could not be loaded in class Block", this->getTexture().c_str());
+  }
+
   /* Create convex blocks */
   for(unsigned int i=0; i<v_BSPPolys.size(); i++) {
-    addPoly(v_BSPPolys[i], io_collisionSystem);
+    addPoly(v_BSPPolys[i], io_collisionSystem, scale);
   }
 
+
   if(i_chipmunkWorld != NULL) {
     if(isPhysics()) {
       if(getCollisionMethod() == Circle && getCollisionRadius() != 0.0f) {
@@ -534,27 +537,9 @@ int Block::loadToPlay(CollisionSystem* io_collisionSystem, ChipmunkWorld* i_chip
   return v_BSPTree.getNumErrors();  
 }
 
-void Block::addPoly(BSPPoly* i_poly, CollisionSystem* io_collisionSystem) {
-  ConvexBlock* v_block = new ConvexBlock(this);
-  float scale = TextureScale() * 0.25;
-  Sprite* pSprite = Theme::instance()->getSprite(SPRITE_TYPE_TEXTURE,
-						 this->getTexture());
-  
-  if(pSprite != NULL) 
-  {
-    try{
-      Texture* v_texture = pSprite->getTexture();
-      if(v_texture != NULL) {
-	int v_textureSize = v_texture->nWidth;
-	if(v_textureSize < 256 && v_textureSize != 0)
-	  // divide by texturesize to prevent upscaling of textures < 256x256
-	  scale *= (256/v_textureSize);
-      }
-    } catch(Exception& e) {
-      ;
-    }
-  }
 
+void Block::addPoly(BSPPoly* i_poly, CollisionSystem* io_collisionSystem, float scale) {
+  ConvexBlock* v_block = new ConvexBlock(this);
   for(unsigned int i=0; i<i_poly->Vertices().size(); i++) {	
     v_block->addVertex(i_poly->Vertices()[i],
 		       Vector2f((InitialPosition().x + i_poly->Vertices()[i].x) * scale,
@@ -563,6 +548,7 @@ void Block::addPoly(BSPPoly* i_poly, CollisionSystem* io_collisionSystem) {
   m_convexBlocks.push_back(v_block);
 }
 
+
 void Block::unloadToPlay() {
   for(unsigned int i=0; i<m_convexBlocks.size(); i++) {
     delete m_convexBlocks[i];
@@ -572,18 +558,20 @@ void Block::unloadToPlay() {
   for(unsigned int i=0; i<m_collisionLines.size(); i++) {
     delete m_collisionLines[i];
   }
-  m_collisionLines.clear();
-  
+  m_collisionLines.clear();  
 }
 
+
 bool Block::isPhysics() const {
   return m_physics;
 }
 
+
 bool Block::isLayer() const {
   return m_isLayer;
 }
 
+
 void Block::setInitialPosition(const Vector2f& i_initialPosition) {
   m_initialPosition  = i_initialPosition;
   m_isBBoxDirty = true;
@@ -871,121 +859,116 @@ Block* Block::readFromXml(XMLDocument* i_xmlSource, TiXmlElement *pElem) {
 }
 
 void Block::saveBinary(FileHandle *i_pfh) {
-      FS::writeString(i_pfh,   Id());
-      FS::writeBool(i_pfh,     isBackground());
-      FS::writeBool(i_pfh,     isDynamic());
-      FS::writeBool(i_pfh,     isPhysics());
-      FS::writeBool(i_pfh,     isLayer());
-      FS::writeInt_LE(i_pfh,   getLayer());
-      FS::writeString(i_pfh,   getTexture());
-      FS::writeFloat_LE(i_pfh, TextureScale());
+      XMFS::writeString(i_pfh,   Id());
+      XMFS::writeBool(i_pfh,     isBackground());
+      XMFS::writeBool(i_pfh,     isDynamic());
+      XMFS::writeBool(i_pfh,     isPhysics());
+      XMFS::writeBool(i_pfh,     isLayer());
+      XMFS::writeInt_LE(i_pfh,   getLayer());
+      XMFS::writeString(i_pfh,   getTexture());
+      XMFS::writeFloat_LE(i_pfh, TextureScale());
       
-      FS::writeInt_LE(i_pfh, m_blendColor.Red());
-      FS::writeInt_LE(i_pfh, m_blendColor.Green());
-      FS::writeInt_LE(i_pfh, m_blendColor.Blue());
-      FS::writeInt_LE(i_pfh, m_blendColor.Alpha());
+      XMFS::writeInt_LE(i_pfh, m_blendColor.Red());
+      XMFS::writeInt_LE(i_pfh, m_blendColor.Green());
+      XMFS::writeInt_LE(i_pfh, m_blendColor.Blue());
+      XMFS::writeInt_LE(i_pfh, m_blendColor.Alpha());
       
-      FS::writeFloat_LE(i_pfh, InitialPosition().x);
-      FS::writeFloat_LE(i_pfh, InitialPosition().y);
-      FS::writeFloat_LE(i_pfh, GripPer20());
-      FS::writeFloat_LE(i_pfh, Mass());
-      FS::writeFloat_LE(i_pfh, Friction());
-      FS::writeFloat_LE(i_pfh, Elasticity());
-      FS::writeInt_LE(i_pfh,   getEdgeDrawMethod());
-      FS::writeFloat_LE(i_pfh, edgeAngle());
+      XMFS::writeFloat_LE(i_pfh, InitialPosition().x);
+      XMFS::writeFloat_LE(i_pfh, InitialPosition().y);
+      XMFS::writeFloat_LE(i_pfh, GripPer20());
+      XMFS::writeFloat_LE(i_pfh, Mass());
+      XMFS::writeFloat_LE(i_pfh, Friction());
+      XMFS::writeFloat_LE(i_pfh, Elasticity());
+      XMFS::writeInt_LE(i_pfh,   getEdgeDrawMethod());
+      XMFS::writeFloat_LE(i_pfh, edgeAngle());
       
-      FS::writeInt_LE(i_pfh, m_edgeMaterial.size());
+      XMFS::writeInt_LE(i_pfh, m_edgeMaterial.size());
       for(unsigned int i=0; i<m_edgeMaterial.size(); i++) {
-        FS::writeString(i_pfh, m_edgeMaterial[i].name);
-        FS::writeString(i_pfh, m_edgeMaterial[i].texture);
-        FS::writeInt_LE(i_pfh, m_edgeMaterial[i].color.Red());
-        FS::writeInt_LE(i_pfh, m_edgeMaterial[i].color.Green());
-        FS::writeInt_LE(i_pfh, m_edgeMaterial[i].color.Blue());
-        FS::writeInt_LE(i_pfh, m_edgeMaterial[i].color.Alpha());
-        FS::writeFloat_LE(i_pfh, m_edgeMaterial[i].scale);
-        FS::writeFloat_LE(i_pfh, m_edgeMaterial[i].depth);
+        XMFS::writeString(i_pfh, m_edgeMaterial[i].name);
+        XMFS::writeString(i_pfh, m_edgeMaterial[i].texture);
+        XMFS::writeInt_LE(i_pfh, m_edgeMaterial[i].color.Red());
+        XMFS::writeInt_LE(i_pfh, m_edgeMaterial[i].color.Green());
+        XMFS::writeInt_LE(i_pfh, m_edgeMaterial[i].color.Blue());
+        XMFS::writeInt_LE(i_pfh, m_edgeMaterial[i].color.Alpha());
+        XMFS::writeFloat_LE(i_pfh, m_edgeMaterial[i].scale);
+        XMFS::writeFloat_LE(i_pfh, m_edgeMaterial[i].depth);
       }
       
-      FS::writeInt_LE(i_pfh,   getCollisionMethod());
-      FS::writeFloat_LE(i_pfh, getCollisionRadius());
+      XMFS::writeInt_LE(i_pfh,   getCollisionMethod());
+      XMFS::writeFloat_LE(i_pfh, getCollisionRadius());
 
-      FS::writeShort_LE(i_pfh, Vertices().size());
+      XMFS::writeShort_LE(i_pfh, Vertices().size());
         
       for(unsigned int j=0; j<Vertices().size();j++) {
-        FS::writeFloat_LE(i_pfh, Vertices()[j]->Position().x);
-        FS::writeFloat_LE(i_pfh, Vertices()[j]->Position().y);
-        FS::writeString(i_pfh,   Vertices()[j]->EdgeEffect());
+        XMFS::writeFloat_LE(i_pfh, Vertices()[j]->Position().x);
+        XMFS::writeFloat_LE(i_pfh, Vertices()[j]->Position().y);
+        XMFS::writeString(i_pfh,   Vertices()[j]->EdgeEffect());
       }   
 }
 
 Block* Block::readFromBinary(FileHandle *i_pfh) {
-  Block *pBlock = new Block(FS::readString(i_pfh));
+  Block *pBlock = new Block(XMFS::readString(i_pfh));
 
-  pBlock->setBackground(FS::readBool(i_pfh));
-  pBlock->setDynamic(FS::readBool(i_pfh));
-  pBlock->setPhysics(FS::readBool(i_pfh));
-  pBlock->setIsLayer(FS::readBool(i_pfh));
-  pBlock->setLayer(FS::readInt_LE(i_pfh));
-  pBlock->setTexture(FS::readString(i_pfh));
-  pBlock->setTextureScale(FS::readFloat_LE(i_pfh));
+  pBlock->setBackground(XMFS::readBool(i_pfh));
+  pBlock->setDynamic(XMFS::readBool(i_pfh));
+  pBlock->setPhysics(XMFS::readBool(i_pfh));
+  pBlock->setIsLayer(XMFS::readBool(i_pfh));
+  pBlock->setLayer(XMFS::readInt_LE(i_pfh));
+  pBlock->setTexture(XMFS::readString(i_pfh));
+  pBlock->setTextureScale(XMFS::readFloat_LE(i_pfh));
 
   TColor v_blendColor = TColor(255,255,255,255);
-  v_blendColor.setRed(FS::readInt_LE(i_pfh));
-  v_blendColor.setGreen(FS::readInt_LE(i_pfh));
-  v_blendColor.setBlue(FS::readInt_LE(i_pfh));
-  v_blendColor.setAlpha(FS::readInt_LE(i_pfh));
+  v_blendColor.setRed(XMFS::readInt_LE(i_pfh));
+  v_blendColor.setGreen(XMFS::readInt_LE(i_pfh));
+  v_blendColor.setBlue(XMFS::readInt_LE(i_pfh));
+  v_blendColor.setAlpha(XMFS::readInt_LE(i_pfh));
   pBlock->setBlendColor(v_blendColor);
 
   Vector2f v_Position;
-  v_Position.x = FS::readFloat_LE(i_pfh);
-  v_Position.y = FS::readFloat_LE(i_pfh);
+  v_Position.x = XMFS::readFloat_LE(i_pfh);
+  v_Position.y = XMFS::readFloat_LE(i_pfh);
   pBlock->setInitialPosition(v_Position);
-  pBlock->setGripPer20(FS::readFloat_LE(i_pfh));
-  pBlock->setMass(FS::readFloat_LE(i_pfh));
-  pBlock->setFriction(FS::readFloat_LE(i_pfh));
-  pBlock->setElasticity(FS::readFloat_LE(i_pfh));
-  pBlock->setEdgeDrawMethod((EdgeDrawMethod)FS::readInt_LE(i_pfh));
-  pBlock->setEdgeAngle(FS::readFloat_LE(i_pfh));
+  pBlock->setGripPer20(XMFS::readFloat_LE(i_pfh));
+  pBlock->setMass(XMFS::readFloat_LE(i_pfh));
+  pBlock->setFriction(XMFS::readFloat_LE(i_pfh));
+  pBlock->setElasticity(XMFS::readFloat_LE(i_pfh));
+  pBlock->setEdgeDrawMethod((EdgeDrawMethod)XMFS::readInt_LE(i_pfh));
+  pBlock->setEdgeAngle(XMFS::readFloat_LE(i_pfh));
   
-  int geomSize = FS::readInt_LE(i_pfh);  //get size of geoms and read out as many times
+  int geomSize = XMFS::readInt_LE(i_pfh);  //get size of geoms and read out as many times
   for( int i=0; i<geomSize; i++) {
     EdgeMaterial v_geomMat;
-    v_geomMat.name = FS::readString(i_pfh);
-    v_geomMat.texture = FS::readString(i_pfh);
+    v_geomMat.name = XMFS::readString(i_pfh);
+    v_geomMat.texture = XMFS::readString(i_pfh);
     TColor v_edgeMatColor = TColor(255,255,255,255);
-    v_edgeMatColor.setRed(FS::readInt_LE(i_pfh));
-    v_edgeMatColor.setGreen(FS::readInt_LE(i_pfh));
-    v_edgeMatColor.setBlue(FS::readInt_LE(i_pfh));
-    v_edgeMatColor.setAlpha(FS::readInt_LE(i_pfh));
+    v_edgeMatColor.setRed(XMFS::readInt_LE(i_pfh));
+    v_edgeMatColor.setGreen(XMFS::readInt_LE(i_pfh));
+    v_edgeMatColor.setBlue(XMFS::readInt_LE(i_pfh));
+    v_edgeMatColor.setAlpha(XMFS::readInt_LE(i_pfh));
     v_geomMat.color = v_edgeMatColor;
-    v_geomMat.scale = FS::readFloat_LE(i_pfh);
-    v_geomMat.depth = FS::readFloat_LE(i_pfh);
+    v_geomMat.scale = XMFS::readFloat_LE(i_pfh);
+    v_geomMat.depth = XMFS::readFloat_LE(i_pfh);
     pBlock->m_edgeMaterial.push_back(v_geomMat);
   }
       
   
-  pBlock->setCollisionMethod((CollisionMethod)FS::readInt_LE(i_pfh));
-  pBlock->setCollisionRadius(FS::readFloat_LE(i_pfh));
+  pBlock->setCollisionMethod((CollisionMethod)XMFS::readInt_LE(i_pfh));
+  pBlock->setCollisionRadius(XMFS::readFloat_LE(i_pfh));
   
-  int nNumVertices = FS::readShort_LE(i_pfh);
+  int nNumVertices = XMFS::readShort_LE(i_pfh);
   pBlock->Vertices().reserve(nNumVertices);
   for(int j=0;j<nNumVertices;j++) {
     Vector2f v_Position;
-    v_Position.x = FS::readFloat_LE(i_pfh);
-    v_Position.y = FS::readFloat_LE(i_pfh);
-    std::string v_EdgeEffect = FS::readString(i_pfh);
+    v_Position.x = XMFS::readFloat_LE(i_pfh);
+    v_Position.y = XMFS::readFloat_LE(i_pfh);
+    std::string v_EdgeEffect = XMFS::readString(i_pfh);
     pBlock->Vertices().push_back(new BlockVertex(v_Position, v_EdgeEffect));
   }
 
   return pBlock;
 }
 
-void Block::addEdgeGeom(int geomIndex)
-{
-  m_edgeGeoms.push_back(geomIndex);
-}
-
-std::vector<int>& Block::getEdgeGeoms()
+std::vector<Geom*>& Block::getEdgeGeoms()
 {
   return m_edgeGeoms;
 }
