@@ -67,16 +67,44 @@ Entity::Entity(const std::string& i_id) {
   m_BCircle.reset();
   m_isBBoxDirty = true;
   m_speciality  = ET_NONE;
+  m_isScripted  = false;
+  m_isCheckpoint= false;
 }
 
 Entity::~Entity() {
 }
 
-void Entity::loadToPlay() {
+void Entity::loadToPlay(const std::string& i_ScriptSource) {
   m_dynamicPosition = m_initialPosition;
   /* make every entity alive */
   setAlive(true);
   m_isBBoxDirty = true;
+  if(m_id == "") {
+    return;
+  }
+  
+  /* check if the entity is used in the script (for sprite rendering independently from gfx mode)  
+  NOTE: Entity IDs, that are created dynamically in the script, can still be a problem! 
+  If theyre named "pathxyz", they can be mistaken for scripted, if e.g. block translations are used on blocks named pathxyz
+  Maybe its an idea for the future, to test for the "..i" string in source or look if its a "hardcoded" pathxyz in script */
+  
+  size_t v_pos;
+  std::string v_pureID = m_id; // lets use that one to put the string into it, which is stripped from numbers, that could be used in the scripts  size_t v_pos;
+  do {
+    v_pos = v_pureID.find_last_of("1234567890",v_pureID.length()-1);
+    if(v_pos != std::string::npos) {
+      v_pureID.erase(v_pos);
+    }
+  }
+  while( v_pos != std::string::npos);
+  
+  size_t found = i_ScriptSource.find(v_pureID);
+  if(found != std::string::npos && v_pureID != "path") {
+    this->setScripted( true ); 
+    if(XMSession::instance()->debug()) {
+      LogInfo("The mighty script parser acknowledges %s as a scripted entity.",m_id.c_str());
+    }
+  }
 }
 
 void Entity::unloadToPlay() {
@@ -94,6 +122,10 @@ bool Entity::DoesKill() const {
   return m_doesKill;
 }
 
+bool Entity::IsCheckpoint() const {
+  return false;
+}
+
 Vector2f Entity::InitialPosition() const {
   return m_initialPosition;
 }
@@ -142,6 +174,7 @@ void Entity::setSpeciality(EntitySpeciality i_speciality) {
   m_doesMakeWin = (i_speciality == ET_MAKEWIN);
   m_doesKill    = (i_speciality == ET_KILL);
   m_isToTake    = (i_speciality == ET_ISTOTAKE);
+  m_isCheckpoint= (i_speciality == ET_CHECKPOINT);
 
   m_speciality = i_speciality;
 }
@@ -211,6 +244,8 @@ EntitySpeciality Entity::SpecialityFromStr(std::string& i_typeStr) {
     return ET_PARTICLES_SOURCE;
   if(i_typeStr == "Joint")
     return ET_JOINT;
+  if(i_typeStr == "Checkpoint")
+    return ET_CHECKPOINT;
 
   return ET_NONE;
 }
@@ -235,6 +270,9 @@ std::string Entity::SpecialityToStr(EntitySpeciality i_speciality) {
     case ET_JOINT :
       return "Joint";
       break;
+    case ET_CHECKPOINT :
+      return "Checkpoint";
+      break;
   default:
       return "Sprite";
   }
@@ -285,6 +323,9 @@ Entity* Entity::createEntity(const std::string& id, const std::string& typeId,
   case ET_JOINT:
     v_entity = new Joint(id);
     break;
+  case ET_CHECKPOINT:
+    v_entity = new Checkpoint(id);
+    break;
   default:
     v_entity = new Entity(id);
     break;
@@ -315,6 +356,7 @@ Entity* Entity::createEntity(const std::string& id, const std::string& typeId,
     break;
   case ET_JOINT:
     break;
+  case ET_CHECKPOINT:
   default:
     v_entity->setSpriteName(typeId);
     v_entity->setSprite(v_entity->loadSprite());
@@ -334,7 +376,6 @@ Entity* Entity::createEntity(const std::string& id, const std::string& typeId,
   v_entity->setDrawReversed(reversed);
   v_entity->setZ(z);
 
-
   return v_entity;
 }
 
@@ -391,37 +432,37 @@ Entity* Entity::readFromXml(TiXmlElement *pElem) {
 }
 
 void Entity::saveBinary(FileHandle *i_pfh) {
-  FS::writeString(i_pfh,   Id());
-  FS::writeString(i_pfh,   Entity::SpecialityToStr(Speciality()));
-  FS::writeFloat_LE(i_pfh, Size());
-  FS::writeFloat_LE(i_pfh, Width());       
-  FS::writeFloat_LE(i_pfh, Height()); 
-  FS::writeFloat_LE(i_pfh, InitialPosition().x);
-  FS::writeFloat_LE(i_pfh, InitialPosition().y);
-  FS::writeFloat_LE(i_pfh, DrawAngle());
-  FS::writeBool(i_pfh,  DrawReversed());
+  XMFS::writeString(i_pfh,   Id());
+  XMFS::writeString(i_pfh,   Entity::SpecialityToStr(Speciality()));
+  XMFS::writeFloat_LE(i_pfh, Size());
+  XMFS::writeFloat_LE(i_pfh, Width());       
+  XMFS::writeFloat_LE(i_pfh, Height()); 
+  XMFS::writeFloat_LE(i_pfh, InitialPosition().x);
+  XMFS::writeFloat_LE(i_pfh, InitialPosition().y);
+  XMFS::writeFloat_LE(i_pfh, DrawAngle());
+  XMFS::writeBool(i_pfh,  DrawReversed());
   
   switch(Speciality()) {
   case ET_NONE:
   case ET_PARTICLES_SOURCE:
-    FS::writeByte(i_pfh, 0x02);
+    XMFS::writeByte(i_pfh, 0x02);
     break;
   default:
-    FS::writeByte(i_pfh, 0x01);
+    XMFS::writeByte(i_pfh, 0x01);
   }
 
   std::ostringstream v_z;
   v_z << Z();
-  FS::writeString(i_pfh, "z");
-  FS::writeString(i_pfh, v_z.str());
+  XMFS::writeString(i_pfh, "z");
+  XMFS::writeString(i_pfh, v_z.str());
 
   if(Speciality() == ET_NONE) {
-    FS::writeString(i_pfh, "name");
-    FS::writeString(i_pfh, SpriteName());
+    XMFS::writeString(i_pfh, "name");
+    XMFS::writeString(i_pfh, SpriteName());
   }
   else if(Speciality() == ET_PARTICLES_SOURCE) {
-    FS::writeString(i_pfh, "type");
-    FS::writeString(i_pfh, SpriteName());
+    XMFS::writeString(i_pfh, "type");
+    XMFS::writeString(i_pfh, SpriteName());
   }
 }
 
@@ -440,22 +481,22 @@ Entity* Entity::readFromBinary(FileHandle *i_pfh) {
   std::string v_typeName;
 
   /* read values */
-  v_id         = FS::readString(i_pfh);
-  v_typeId     = FS::readString(i_pfh);
+  v_id         = XMFS::readString(i_pfh);
+  v_typeId     = XMFS::readString(i_pfh);
   v_speciality = Entity::SpecialityFromStr(v_typeId);
-  v_size       = FS::readFloat_LE(i_pfh);
-  v_width      = FS::readFloat_LE(i_pfh);
-  v_height     = FS::readFloat_LE(i_pfh);
-  v_position.x = FS::readFloat_LE(i_pfh);
-  v_position.y = FS::readFloat_LE(i_pfh);
-  v_angle      = FS::readFloat_LE(i_pfh);
-  v_reversed   = FS::readBool(i_pfh);
+  v_size       = XMFS::readFloat_LE(i_pfh);
+  v_width      = XMFS::readFloat_LE(i_pfh);
+  v_height     = XMFS::readFloat_LE(i_pfh);
+  v_position.x = XMFS::readFloat_LE(i_pfh);
+  v_position.y = XMFS::readFloat_LE(i_pfh);
+  v_angle      = XMFS::readFloat_LE(i_pfh);
+  v_reversed   = XMFS::readBool(i_pfh);
   std::string v_paramName;
   std::string v_paramValue;
-  int nNumParams = FS::readByte(i_pfh);
+  int nNumParams = XMFS::readByte(i_pfh);
   for(int j=0;j<nNumParams;j++) {
-    v_paramName  = FS::readString(i_pfh);
-    v_paramValue = FS::readString(i_pfh);
+    v_paramName  = XMFS::readString(i_pfh);
+    v_paramValue = XMFS::readString(i_pfh);
 
     if(v_paramName == "z") {
       v_z = atof(v_paramValue.c_str());
@@ -465,39 +506,65 @@ Entity* Entity::readFromBinary(FileHandle *i_pfh) {
       v_typeName   = v_paramValue;
     }
   }
-
+  
   return  createEntity(v_id, v_typeId, v_speciality, v_position, v_angle,
 		       v_reversed, v_size, v_width, v_height, v_z,
 		       v_spriteName, v_typeName);
 }
 
 
+void Entity::setScripted(bool i_value) {
+  m_isScripted = i_value;
+}
+
+/*==========================================
+        Checkpoints
+==========================================*/
+
+bool Checkpoint::IsCheckpoint() const {
+  return true;
+}
+
+void Checkpoint::deactivate() {
+  m_isVirgin = true;
+}
+
+bool Checkpoint::isActivated() const {
+  return m_isVirgin == false;
+}
+
+void Checkpoint::activate(const std::vector<Entity*>& i_destroyedEntities, DriveDir i_direction) {
+  m_isVirgin = false;
+  m_destroyedEntities = i_destroyedEntities;
+  m_direction = i_direction;
+}
+
 /*===========================================
-									Joints
+	Joints
 ===========================================*/
 
 void Joint::saveBinary(FileHandle *i_pfh)
 {
   Entity::saveBinary(i_pfh);
   
-  FS::writeByte(i_pfh, 0x03);
+  XMFS::writeByte(i_pfh, 0x03);
 
-  FS::writeString(i_pfh, "type");
-  FS::writeString(i_pfh, jointTypeToStr(getJointType()));
-  FS::writeString(i_pfh, "start");
-  FS::writeString(i_pfh, getStartBlockId());
-  FS::writeString(i_pfh, "end");
-  FS::writeString(i_pfh, getEndBlockId());
+  XMFS::writeString(i_pfh, "type");
+  XMFS::writeString(i_pfh, jointTypeToStr(getJointType()));
+  XMFS::writeString(i_pfh, "start");
+  XMFS::writeString(i_pfh, getStartBlockId());
+  XMFS::writeString(i_pfh, "end");
+  XMFS::writeString(i_pfh, getEndBlockId());
 }
 
 void Joint::readFromBinary(FileHandle *i_pfh)
 {
   std::string v_paramName;
   std::string v_paramValue;
-  int nNumParams = FS::readByte(i_pfh);
+  int nNumParams = XMFS::readByte(i_pfh);
   for(int j=0;j<nNumParams;j++) {
-    v_paramName  = FS::readString(i_pfh);
-    v_paramValue = FS::readString(i_pfh);
+    v_paramName  = XMFS::readString(i_pfh);
+    v_paramValue = XMFS::readString(i_pfh);
 
     if(v_paramName == "type") {
       setJointType(jointTypeFromStr(v_paramValue));
@@ -552,7 +619,7 @@ std::string Joint::jointTypeToStr(jointType i_type)
 
 void Joint::loadToPlay(Level* i_level, ChipmunkWorld* i_chipmunkWorld)
 {
-  Entity::loadToPlay();
+  Entity::loadToPlay("");
 
   setStartBlock(i_level->getBlockById(getStartBlockId()));
   setEndBlock(i_level->getBlockById(getEndBlockId()));
@@ -658,7 +725,7 @@ ParticlesSource::~ParticlesSource() {
 }
 
 void ParticlesSource::loadToPlay() {
-  Entity::loadToPlay();
+  Entity::loadToPlay("");
   m_lastParticleTime = 0;
 } 
 
