@@ -60,12 +60,17 @@ Level::Level() {
   m_rSpriteForWecker 	      = "Wrecker";
   m_rSpriteForFlower 	      = "Flower";
   m_rSpriteForStar            = "Star";
+  m_rSpriteForCheckpointDown  = "Checkpoint_0";
+  m_rSpriteForCheckpointUp    = "Checkpoint_1";
   m_rSoundForPickUpStrawberry = "PickUpStrawberry";
+  m_rSoundForCheckpoint       = "Checkpoint";
 
   m_strawberrySprite = NULL;
   m_wreckerSprite    = NULL;
   m_flowerSprite     = NULL;
   m_starSprite       = NULL;
+  m_checkpointSpriteDown = NULL;
+  m_checkpointSpriteUp = NULL;
 }
 
 Level::~Level() {
@@ -245,6 +250,10 @@ void Level::setCollisionSystem(CollisionSystem* p_CollisionSystem) {
   m_pCollisionSystem = p_CollisionSystem;
 }
 
+const std::vector<std::string>& Level::scriptLibraryFileNames() {
+	return m_scriptLibraryFileNames;
+}
+
 std::string Level::scriptFileName() const {
   return m_scriptFileName;
 }
@@ -253,6 +262,10 @@ std::string Level::scriptSource() const {
   return m_scriptSource;
 }
 
+void Level::setScriptLibraryFileNames(std::vector<std::string>& i_scriptLibraryFileNames){
+	m_scriptLibraryFileNames = i_scriptLibraryFileNames;
+}
+
 void Level::setScriptFileName(const std::string& i_scriptFileName) {
   m_scriptFileName = i_scriptFileName;
 }
@@ -265,6 +278,14 @@ std::vector<Zone *> &Level::Zones() {
   return m_zones;
 }
 
+std::vector<Zone *> &Level::DeathZones() {
+  return m_zonesDeath;
+}
+
+std::vector<Zone *> &Level::TeleportZones() {
+  return m_zonesTeleport;
+}
+
 const SkyApparence* Level::Sky() const {
   return m_sky;
 }
@@ -345,7 +366,7 @@ void Level::loadXML() {
     m_xmlSource = new XMLDocument();
   }
 
-  m_xmlSource->readFromFile(m_fileName, NULL, true);
+  m_xmlSource->readFromFile(FDT_DATA, m_fileName, NULL, true);
 
   TiXmlDocument *pDoc = m_xmlSource->getLowLevelAccess();
   if(pDoc == NULL) throw Exception("failed to load level XML " + m_fileName);
@@ -376,7 +397,7 @@ void Level::loadXML() {
   m_description = "";
   m_author = "";
   m_music = "";
-  
+  m_scriptLibraryFileNames.clear();
   m_scriptFileName = "";
   m_scriptSource = "";
   m_borderTexture = "";
@@ -558,6 +579,8 @@ void Level::loadXML() {
 	/* for efficacity and before other are not required, only change main ones */
 	if        (v_old_name == "PickUpStrawberry") {
 	  m_rSoundForPickUpStrawberry = v_new_name;
+	} else if (v_old_name == "Checkpoint") {
+	  m_rSoundForCheckpoint = v_new_name;
 	}
       }
     }
@@ -566,10 +589,18 @@ void Level::loadXML() {
     TiXmlElement *pScriptElem = XML::findElement(*m_xmlSource, pLevelElem,std::string("script"));
     if(pScriptElem != NULL) {
       m_isScripted = true;
-
+        
       /* External script file specified? */
       m_scriptFileName = XML::getOption(pScriptElem,"source");      
       
+      /* Specified libraries ? */
+      TiXmlElement *pLibraryScriptElem = pScriptElem->FirstChildElement("require_library");
+      if (pLibraryScriptElem != NULL) {
+    	  for (pLibraryScriptElem = pScriptElem->FirstChildElement("require_library"); pLibraryScriptElem != 0; pLibraryScriptElem = pLibraryScriptElem->NextSiblingElement("require_library")) {
+    		  m_scriptLibraryFileNames.push_back(pLibraryScriptElem->Attribute("name"));
+    	  }
+      }
+
       /* Encapsulated script? */
       for(TiXmlNode *pScript=pScriptElem->FirstChild();pScript!=NULL;
           pScript=pScript->NextSibling()) {
@@ -592,16 +623,16 @@ void Level::loadXML() {
     for(TiXmlElement *pElem = pLevelElem->FirstChildElement("zone"); pElem!=NULL;
         pElem=pElem->NextSiblingElement("zone")) {
       m_zones.push_back(Zone::readFromXml(pElem));
+      m_zones.back()->updateZoneSpeciality(m_scriptSource);
+      if(m_zones.back()->isDeathZone()) {
+        m_zonesDeath.push_back(Zone::readFromXml(pElem));
+      }
+      else if(m_zones.back()->isTeleportZone()) {
+        m_zonesTeleport.push_back(Zone::readFromXml(pElem)); 
+      }
     }
     
     /* determine whether the levels is physics or not */
-    //for(TiXmlElement *pElem = pLevelElem->FirstChildElement("block"); pElem!=NULL;
-    //    pElem=pElem->NextSiblingElement("block")) {
-    //  if(Block::isPhysics_readFromXml(m_xmlSource, pElem)) {
-    //	m_isPhysics = true;
-    //  }
-    //}
-
     /* Get blocks */
     for(TiXmlElement *pElem = pLevelElem->FirstChildElement("block"); pElem!=NULL;
         pElem=pElem->NextSiblingElement("block")) {
@@ -620,8 +651,8 @@ void Level::loadXML() {
 	Joint* v_joint = (Joint*)v_entity;
 	v_joint->readFromXml(pElem);
 	m_joints.push_back(v_joint);
-      } else
-	m_entities.push_back(v_entity);
+      }
+      else m_entities.push_back(v_entity);
     }    
 
     try {
@@ -656,16 +687,16 @@ void Level::loadXML() {
 bool Level::loadReducedFromFile() {
   std::string cacheFileName;
 
-  m_checkSum = FS::md5sum(FileName());
+  m_checkSum = XMFS::md5sum(FDT_DATA, FileName());
 
   // First try to load it from the cache
   bool cached = false;
   /* Determine name in cache */
-  std::string LevelFileBaseName = FS::getFileBaseName(FileName());
+  std::string LevelFileBaseName = XMFS::getFileBaseName(FileName());
   cacheFileName = getNameInCache();
     
   try {
-    cached = importBinaryHeaderFromFile(cacheFileName, m_checkSum);
+    cached = importBinaryHeaderFromFile(FDT_CACHE, cacheFileName, m_checkSum);
   } catch (Exception &e) {
     LogWarning("Exception while loading binary level, will load "
 	       "XML instead for '%s' (%s)", FileName().c_str(),
@@ -675,7 +706,7 @@ bool Level::loadReducedFromFile() {
   // If we couldn't get it from the cache, then load from (slow) XML
   if (!cached) {
     loadXML();
-    exportBinary(cacheFileName, m_checkSum); /* Cache it now */
+    exportBinary(FDT_CACHE, cacheFileName, m_checkSum); /* Cache it now */
   }
   
   unloadLevelBody(); /* remove body datas */
@@ -684,7 +715,7 @@ bool Level::loadReducedFromFile() {
 }
 
 std::string Level::getNameInCache() const {
-  return "LCache/" + Checksum() + FS::getFileBaseName(FileName()) + ".blv";
+  return "LCache/" + Checksum() + XMFS::getFileBaseName(FileName()) + ".blv";
 }
 
 void Level::removeFromCache(xmDatabase *i_db, const std::string& i_id_level) {
@@ -697,7 +728,7 @@ void Level::removeFromCache(xmDatabase *i_db, const std::string& i_id_level) {
     try {
       std::string v_checkSum = i_db->getResult(v_result, 2, 0, 0);
       std::string v_filePath = i_db->getResult(v_result, 2, 0, 1);
-      FS::deleteFile("LCache/" + v_checkSum + FS::getFileBaseName(v_filePath) + ".blv");
+      XMFS::deleteFile(FDT_CACHE, "LCache/" + v_checkSum + XMFS::getFileBaseName(v_filePath) + ".blv");
     } catch(Exception &e) {
       /* ok, it was perhaps not in cache */
     }
@@ -708,85 +739,94 @@ void Level::removeFromCache(xmDatabase *i_db, const std::string& i_id_level) {
 /*===========================================================================
   Export binary level file
   ===========================================================================*/
-void Level::exportBinary(const std::string &FileName, const std::string& pSum) {
+void Level::exportBinary(FileDataType i_fdt, const std::string &FileName, const std::string& pSum) {
   /* Don't do this if we failed to load level from XML */
   if(isXMotoTooOld()) return;
   
   /* Export binary... */
-  FileHandle *pfh = FS::openOFile(FileName);
+  FileHandle *pfh = XMFS::openOFile(i_fdt, FileName);
   if(pfh == NULL) {
     LogWarning("Failed to export binary: %s",FileName.c_str());
   }
   else {
     exportBinaryHeader(pfh);
 
-    FS::writeString(pfh,   m_sky->Texture());
-    FS::writeFloat_LE(pfh, m_sky->Zoom());
-    FS::writeFloat_LE(pfh, m_sky->Offset());
-    FS::writeInt_LE(pfh,   m_sky->TextureColor().Red());
-    FS::writeInt_LE(pfh,   m_sky->TextureColor().Green());
-    FS::writeInt_LE(pfh,   m_sky->TextureColor().Blue());
-    FS::writeInt_LE(pfh,   m_sky->TextureColor().Alpha());
-    FS::writeBool(pfh,     m_sky->Drifted());
-    FS::writeFloat_LE(pfh, m_sky->DriftZoom());
-    FS::writeInt_LE(pfh,   m_sky->DriftTextureColor().Red());
-    FS::writeInt_LE(pfh,   m_sky->DriftTextureColor().Green());
-    FS::writeInt_LE(pfh,   m_sky->DriftTextureColor().Blue());
-    FS::writeInt_LE(pfh,   m_sky->DriftTextureColor().Alpha());
+    XMFS::writeString(pfh,   m_sky->Texture());
+    XMFS::writeFloat_LE(pfh, m_sky->Zoom());
+    XMFS::writeFloat_LE(pfh, m_sky->Offset());
+    XMFS::writeInt_LE(pfh,   m_sky->TextureColor().Red());
+    XMFS::writeInt_LE(pfh,   m_sky->TextureColor().Green());
+    XMFS::writeInt_LE(pfh,   m_sky->TextureColor().Blue());
+    XMFS::writeInt_LE(pfh,   m_sky->TextureColor().Alpha());
+    XMFS::writeBool(pfh,     m_sky->Drifted());
+    XMFS::writeFloat_LE(pfh, m_sky->DriftZoom());
+    XMFS::writeInt_LE(pfh,   m_sky->DriftTextureColor().Red());
+    XMFS::writeInt_LE(pfh,   m_sky->DriftTextureColor().Green());
+    XMFS::writeInt_LE(pfh,   m_sky->DriftTextureColor().Blue());
+    XMFS::writeInt_LE(pfh,   m_sky->DriftTextureColor().Alpha());
     
-    FS::writeString(pfh,   m_sky->BlendTexture());
+    XMFS::writeString(pfh,   m_sky->BlendTexture());
     
-    FS::writeString(pfh,m_borderTexture);
-    FS::writeString(pfh,m_scriptFileName);
+    XMFS::writeString(pfh,m_borderTexture);
 
-    FS::writeFloat_LE(pfh,m_leftLimit);
-    FS::writeFloat_LE(pfh,m_rightLimit);
-    FS::writeFloat_LE(pfh,m_topLimit);
-    FS::writeFloat_LE(pfh,m_bottomLimit);
+    XMFS::writeInt_LE(pfh, m_scriptLibraryFileNames.size());
+    for(unsigned int i=0; i<m_scriptLibraryFileNames.size(); i++) {
+      XMFS::writeString(pfh, m_scriptLibraryFileNames[i]);
+    }
 
-    FS::writeString(pfh,m_rSpriteForStrawberry);
-    FS::writeString(pfh,m_rSpriteForFlower);
-    FS::writeString(pfh,m_rSpriteForWecker);
-    FS::writeString(pfh,m_rSpriteForStar);
-    FS::writeString(pfh,m_rSoundForPickUpStrawberry);
+    XMFS::writeString(pfh,m_scriptFileName);
 
-    FS::writeInt_LE(pfh, m_numberLayer);
+    XMFS::writeFloat_LE(pfh,m_leftLimit);
+    XMFS::writeFloat_LE(pfh,m_rightLimit);
+    XMFS::writeFloat_LE(pfh,m_topLimit);
+    XMFS::writeFloat_LE(pfh,m_bottomLimit);
+
+    XMFS::writeString(pfh,m_rSpriteForStrawberry);
+    XMFS::writeString(pfh,m_rSpriteForFlower);
+    XMFS::writeString(pfh,m_rSpriteForWecker);
+    XMFS::writeString(pfh,m_rSpriteForStar);
+    XMFS::writeString(pfh,m_rSpriteForCheckpointDown);
+    XMFS::writeString(pfh,m_rSpriteForCheckpointUp);
+    XMFS::writeString(pfh,m_rSoundForPickUpStrawberry);
+    XMFS::writeString(pfh,m_rSoundForCheckpoint);
+
+    XMFS::writeInt_LE(pfh, m_numberLayer);
     for(int i=0; i<m_numberLayer; i++){
-      FS::writeFloat_LE(pfh, m_layerOffsets[i].x);
-      FS::writeFloat_LE(pfh, m_layerOffsets[i].y);
-      FS::writeBool(pfh,     m_isLayerFront[i]);
+      XMFS::writeFloat_LE(pfh, m_layerOffsets[i].x);
+      XMFS::writeFloat_LE(pfh, m_layerOffsets[i].y);
+      XMFS::writeBool(pfh,     m_isLayerFront[i]);
     }    
 
     /* Write script (if any) */
-    FS::writeInt_LE(pfh,m_scriptSource.length());
-    FS::writeBuf(pfh,(char *)m_scriptSource.c_str(),m_scriptSource.length());
+    XMFS::writeInt_LE(pfh,m_scriptSource.length());
+    XMFS::writeBuf(pfh,(char *)m_scriptSource.c_str(),m_scriptSource.length());
       
     /* Write blocks */
-    FS::writeInt_LE(pfh,m_blocks.size());
+    XMFS::writeInt_LE(pfh,m_blocks.size());
     for(unsigned int i=0;i<m_blocks.size();i++) {
       m_blocks[i]->saveBinary(pfh);
     }
       
     /* Write entities */
-    FS::writeInt_LE(pfh,m_entities.size());
+    XMFS::writeInt_LE(pfh,m_entities.size());
     for(unsigned int i=0;i<m_entities.size();i++) {
       m_entities[i]->saveBinary(pfh);
     }
 
     // write joints
-    FS::writeInt_LE(pfh, m_joints.size());
+    XMFS::writeInt_LE(pfh, m_joints.size());
     for(unsigned int i=0; i<m_joints.size(); i++) {
       m_joints[i]->saveBinary(pfh);
     }
       
     /* Write zones */
-    FS::writeInt_LE(pfh,m_zones.size());
+    XMFS::writeInt_LE(pfh,m_zones.size());
     for(unsigned int i=0;i<m_zones.size();i++) {
       m_zones[i]->saveBinary(pfh);
     }
                 
     /* clean up */
-    FS::closeFile(pfh);
+    XMFS::closeFile(pfh);
   }
 }
   
@@ -795,9 +835,9 @@ bool Level::isFullyLoaded() const {
 }
 
 void Level::loadFullyFromFile() {
-  if(importBinary(getNameInCache(), Checksum()) == false) {
+  if(importBinary(FDT_CACHE, getNameInCache(), Checksum()) == false) {
     loadXML();
-    exportBinary(getNameInCache(), m_checkSum);
+    exportBinary(FDT_CACHE, getNameInCache(), m_checkSum);
   }
   loadRemplacementSprites();
 }
@@ -809,23 +849,23 @@ void Level::importBinaryHeader(FileHandle *pfh) {
   m_playerStart  = Vector2f(0.0, 0.0);
   m_xmotoTooOld  = false;
 
-  int nFormat = FS::readInt_LE(pfh);
+  int nFormat = XMFS::readInt_LE(pfh);
   
   if(nFormat != CACHE_LEVEL_FORMAT_VERSION) {
     throw Exception("Old file format");
   }
   
-  m_checkSum    = FS::readString(pfh);
-  m_id      	= FS::readString(pfh);
-  m_pack    	= FS::readString(pfh);
-  m_packNum 	= FS::readString(pfh);
-  m_name    	= FS::readString(pfh);
-  m_description = FS::readString(pfh);
-  m_author 	= FS::readString(pfh);
-  m_date   	= FS::readString(pfh);
-  m_music       = FS::readString(pfh);
-  m_isScripted  = FS::readBool(pfh);
-  m_isPhysics   = FS::readBool(pfh);
+  m_checkSum    = XMFS::readString(pfh);
+  m_id      	= XMFS::readString(pfh);
+  m_pack    	= XMFS::readString(pfh);
+  m_packNum 	= XMFS::readString(pfh);
+  m_name    	= XMFS::readString(pfh);
+  m_description = XMFS::readString(pfh);
+  m_author 	= XMFS::readString(pfh);
+  m_date   	= XMFS::readString(pfh);
+  m_music       = XMFS::readString(pfh);
+  m_isScripted  = XMFS::readBool(pfh);
+  m_isPhysics   = XMFS::readBool(pfh);
 }
 
 void Level::importHeader(const std::string& i_id,
@@ -862,9 +902,9 @@ void Level::importHeader(const std::string& i_id,
   /*===========================================================================
   Import binary level file
   ===========================================================================*/
-bool Level::importBinaryHeaderFromFile(const std::string &FileName, const std::string& pSum) {
+bool Level::importBinaryHeaderFromFile(FileDataType i_fdt, const std::string &FileName, const std::string& pSum) {
   /* Import binary */
-  FileHandle *pfh = FS::openIFile(FileName, true);
+  FileHandle *pfh = XMFS::openIFile(i_fdt, FileName, true);
   if(pfh == NULL) {
     return false;
   }
@@ -873,16 +913,16 @@ bool Level::importBinaryHeaderFromFile(const std::string &FileName, const std::s
     importBinaryHeader(pfh);
     if(m_checkSum != pSum) {
       LogWarning("CRC check failed, can't import: %s",FileName.c_str());
-      FS::closeFile(pfh);
+      XMFS::closeFile(pfh);
       return false;
     }
   } catch(Exception &e) {
-    FS::closeFile(pfh);
+    XMFS::closeFile(pfh);
     return false;
   }
              
   /* clean up */
-  FS::closeFile(pfh);
+  XMFS::closeFile(pfh);
 
   return true;
 }
@@ -893,21 +933,21 @@ void Level::exportBinaryHeader(FileHandle *pfh) {
   /* 4 -> includes level pack num */
   /* 5 -> clean code */
   /* Write CRC32 of XML */
-  FS::writeInt_LE(pfh, CACHE_LEVEL_FORMAT_VERSION);
-  FS::writeString(pfh	    , m_checkSum);
-  FS::writeString(pfh	    , m_id);
-  FS::writeString(pfh	    , m_pack);
-  FS::writeString(pfh	    , m_packNum);
-  FS::writeString(pfh	    , m_name);
-  FS::writeString(pfh	    , m_description);
-  FS::writeString(pfh	    , m_author);
-  FS::writeString(pfh	    , m_date);
-  FS::writeString(pfh	    , m_music);
-  FS::writeBool(  pfh	    , m_isScripted);
-  FS::writeBool(  pfh	    , m_isPhysics);
+  XMFS::writeInt_LE(pfh, CACHE_LEVEL_FORMAT_VERSION);
+  XMFS::writeString(pfh	    , m_checkSum);
+  XMFS::writeString(pfh	    , m_id);
+  XMFS::writeString(pfh	    , m_pack);
+  XMFS::writeString(pfh	    , m_packNum);
+  XMFS::writeString(pfh	    , m_name);
+  XMFS::writeString(pfh	    , m_description);
+  XMFS::writeString(pfh	    , m_author);
+  XMFS::writeString(pfh	    , m_date);
+  XMFS::writeString(pfh	    , m_music);
+  XMFS::writeBool(  pfh	    , m_isScripted);
+  XMFS::writeBool(  pfh	    , m_isPhysics);
 }
 
-bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
+bool Level::importBinary(FileDataType i_fdt, const std::string &FileName, const std::string& pSum) {
   unloadLevelBody();
   bool bRet = true;
   
@@ -916,72 +956,80 @@ bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
   m_xmotoTooOld = false;
   
   /* Import binary */
-  FileHandle *pfh = FS::openIFile(FileName, true);
+  FileHandle *pfh = XMFS::openIFile(i_fdt, FileName, true);
   if(pfh == NULL) {
     return false;
   }
   else {
     /* Read tag - it tells something about the format */
-    int nFormat = FS::readInt_LE(pfh);
+    int nFormat = XMFS::readInt_LE(pfh);
     
     if(nFormat == CACHE_LEVEL_FORMAT_VERSION) { /* reject other formats */
       /* Read "format 1" / "format 2" binary level */
       /* Right */
-      std::string md5sum = FS::readString(pfh);
+      std::string md5sum = XMFS::readString(pfh);
       if(md5sum != pSum) {
         LogWarning("CRC check failed, can't import: %s",FileName.c_str());
         bRet = false;
       }
       else {
         /* Read header */
-        m_id = FS::readString(pfh);
-        m_pack = FS::readString(pfh);
-        m_packNum = FS::readString(pfh);
-        m_name = FS::readString(pfh);
-        m_description = FS::readString(pfh);
-        m_author = FS::readString(pfh);
-        m_date = FS::readString(pfh);
-        m_music = FS::readString(pfh);
-	m_isScripted = FS::readBool(pfh);
-	m_isPhysics  = FS::readBool(pfh);
+        m_id = XMFS::readString(pfh);
+        m_pack = XMFS::readString(pfh);
+        m_packNum = XMFS::readString(pfh);
+        m_name = XMFS::readString(pfh);
+        m_description = XMFS::readString(pfh);
+        m_author = XMFS::readString(pfh);
+        m_date = XMFS::readString(pfh);
+        m_music = XMFS::readString(pfh);
+	m_isScripted = XMFS::readBool(pfh);
+	m_isPhysics  = XMFS::readBool(pfh);
 
 	/* sky */
-        m_sky->setTexture(FS::readString(pfh));
-	m_sky->setZoom(FS::readFloat_LE(pfh));
-	m_sky->setOffset(FS::readFloat_LE(pfh));
+        m_sky->setTexture(XMFS::readString(pfh));
+	m_sky->setZoom(XMFS::readFloat_LE(pfh));
+	m_sky->setOffset(XMFS::readFloat_LE(pfh));
         
 	int v_r, v_g, v_b, v_a;
-	v_r = FS::readInt_LE(pfh);
-	v_g = FS::readInt_LE(pfh);
-	v_b = FS::readInt_LE(pfh);
-	v_a = FS::readInt_LE(pfh);
+	v_r = XMFS::readInt_LE(pfh);
+	v_g = XMFS::readInt_LE(pfh);
+	v_b = XMFS::readInt_LE(pfh);
+	v_a = XMFS::readInt_LE(pfh);
 	m_sky->setTextureColor(TColor(v_r, v_g, v_b, v_a));
 
-	m_sky->setDrifted(FS::readBool(pfh));
-	m_sky->setDriftZoom(FS::readFloat_LE(pfh));
+	m_sky->setDrifted(XMFS::readBool(pfh));
+	m_sky->setDriftZoom(XMFS::readFloat_LE(pfh));
 
-	v_r = FS::readInt_LE(pfh);
-	v_g = FS::readInt_LE(pfh);
-	v_b = FS::readInt_LE(pfh);
-	v_a = FS::readInt_LE(pfh);
+	v_r = XMFS::readInt_LE(pfh);
+	v_g = XMFS::readInt_LE(pfh);
+	v_b = XMFS::readInt_LE(pfh);
+	v_a = XMFS::readInt_LE(pfh);
 	m_sky->setDriftTextureColor(TColor(v_r, v_g, v_b, v_a));
 	/* *** */
-        m_sky->setBlendTexture(FS::readString(pfh));
-        m_borderTexture = FS::readString(pfh);
+        m_sky->setBlendTexture(XMFS::readString(pfh));
+        m_borderTexture = XMFS::readString(pfh);
 
-        m_scriptFileName = FS::readString(pfh);
+	unsigned int v_nbScriptLibraryFileNames = XMFS::readInt_LE(pfh);
+	for(unsigned int i=0; i<v_nbScriptLibraryFileNames; i++) {
+	  m_scriptLibraryFileNames.push_back(XMFS::readString(pfh));
+	}
 
-        m_leftLimit = FS::readFloat_LE(pfh);
-        m_rightLimit = FS::readFloat_LE(pfh);
-        m_topLimit = FS::readFloat_LE(pfh);
-        m_bottomLimit = FS::readFloat_LE(pfh);
+        m_scriptFileName = XMFS::readString(pfh);
+
+        m_leftLimit = XMFS::readFloat_LE(pfh);
+        m_rightLimit = XMFS::readFloat_LE(pfh);
+        m_topLimit = XMFS::readFloat_LE(pfh);
+        m_bottomLimit = XMFS::readFloat_LE(pfh);
 
 	/* theme replacements */
-	m_rSpriteForStrawberry 	    = FS::readString(pfh);
-	m_rSpriteForFlower     	    = FS::readString(pfh);
-	m_rSpriteForWecker     	    = FS::readString(pfh);
-	m_rSpriteForStar       	    = FS::readString(pfh);
-	m_rSoundForPickUpStrawberry = FS::readString(pfh);
+	m_rSpriteForStrawberry 	    = XMFS::readString(pfh);
+	m_rSpriteForFlower     	    = XMFS::readString(pfh);
+	m_rSpriteForWecker     	    = XMFS::readString(pfh);
+	m_rSpriteForStar       	    = XMFS::readString(pfh);
+	m_rSpriteForCheckpointDown  = XMFS::readString(pfh);
+	m_rSpriteForCheckpointUp    = XMFS::readString(pfh);
+	m_rSoundForPickUpStrawberry = XMFS::readString(pfh);
+	m_rSoundForCheckpoint       = XMFS::readString(pfh);
 
 	if(m_rSpriteForStrawberry == "")
 	  m_rSpriteForStrawberry = "Strawberry";
@@ -991,22 +1039,26 @@ bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
 	  m_rSpriteForWecker = "Wrecker";
 	if(m_rSpriteForStar == "")
 	  m_rSpriteForStar = "Star";
+	if(m_rSpriteForCheckpointDown == "")
+	  m_rSpriteForCheckpointDown = "Checkpoint_0";
+	if(m_rSpriteForCheckpointUp == "")
+	  m_rSpriteForCheckpointUp = "Checkpoint_1";
 
 	/* layers */
-	m_numberLayer = FS::readInt_LE(pfh);
+	m_numberLayer = XMFS::readInt_LE(pfh);
 	for(int i=0; i<m_numberLayer; i++){
 	  Vector2f offset;
-	  offset.x = FS::readFloat_LE(pfh);
-	  offset.y = FS::readFloat_LE(pfh);
+	  offset.x = XMFS::readFloat_LE(pfh);
+	  offset.y = XMFS::readFloat_LE(pfh);
 	  m_layerOffsets.push_back(offset);
-	  m_isLayerFront.push_back(FS::readBool(pfh));
+	  m_isLayerFront.push_back(XMFS::readBool(pfh));
 	}
 
         /* Read embedded script */
-        int nScriptSourceLen = FS::readInt_LE(pfh);
+        int nScriptSourceLen = XMFS::readInt_LE(pfh);
         if(nScriptSourceLen > 0) {
           char *pcTemp = new char[nScriptSourceLen+1];
-          FS::readBuf(pfh,(char *)pcTemp,nScriptSourceLen);
+          XMFS::readBuf(pfh,(char *)pcTemp,nScriptSourceLen);
           pcTemp[nScriptSourceLen]='\0';
             
           m_scriptSource = pcTemp;
@@ -1017,15 +1069,15 @@ bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
           m_scriptSource = "";
 
         /* Read blocks */
-        int nNumBlocks = FS::readInt_LE(pfh);
+        int nNumBlocks = XMFS::readInt_LE(pfh);
         m_blocks.reserve(nNumBlocks);
         for(int i=0;i<nNumBlocks;i++) {
           m_blocks.push_back(Block::readFromBinary(pfh));
         }
 
         /* Read entities */
-        int nNumEntities = FS::readInt_LE(pfh);
-        m_entities.reserve(nNumEntities);
+        int nNumEntities = XMFS::readInt_LE(pfh);
+        m_entities.reserve(nNumEntities); // we dont need that much reserved space, since some strawberries were eaten
         for(int i=0;i<nNumEntities;i++) {
 	  Entity* v_entity = Entity::readFromBinary(pfh);
 	  m_entities.push_back(v_entity);
@@ -1037,8 +1089,8 @@ bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
 	  LogWarning("no player start entity for level %s", Id().c_str());
 	  m_playerStart = Vector2f(0.0, 0.0);
 	}
-
-        int nNumJoints = FS::readInt_LE(pfh);
+	
+        int nNumJoints = XMFS::readInt_LE(pfh);
         m_joints.reserve(nNumJoints);
         for(int i=0; i<nNumJoints; i++) {
 	  Entity* v_entity = Entity::readFromBinary(pfh);
@@ -1048,10 +1100,17 @@ bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
         }
 
         /* Read zones */
-        int nNumZones = FS::readInt_LE(pfh);
+        int nNumZones = XMFS::readInt_LE(pfh);
         m_zones.reserve(nNumZones);
         for(int i=0;i<nNumZones;i++) {
           m_zones.push_back(Zone::readFromBinary(pfh));
+          m_zones.back()->updateZoneSpeciality(m_scriptSource);
+          if(m_zones.back()->isDeathZone()) {
+            m_zonesDeath.push_back(m_zones.back());
+          }
+          else if(m_zones.back()->isTeleportZone()) {
+            m_zonesTeleport.push_back(m_zones.back());
+          }
         }                                                                       
       }
     }
@@ -1061,7 +1120,7 @@ bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
     }
              
     /* clean up */
-    FS::closeFile(pfh);
+    XMFS::closeFile(pfh);
   }
     
   m_isBodyLoaded = bRet;
@@ -1162,7 +1221,7 @@ int Level::loadToPlay(ChipmunkWorld* i_chipmunkWorld, PhysicsSettings* i_physics
   
   /* Spawn initial entities */
   for(unsigned int i=0; i<m_entities.size(); i++) {
-    m_entities[i]->loadToPlay();
+    m_entities[i]->loadToPlay(m_scriptSource);
     Vector2f v = m_entities[i]->DynamicPosition();
 
     m_pCollisionSystem->addEntity(m_entities[i]);
@@ -1171,7 +1230,7 @@ int Level::loadToPlay(ChipmunkWorld* i_chipmunkWorld, PhysicsSettings* i_physics
       m_nbEntitiesToTake++;
     }
   }
-
+  
   // create joints
   for(unsigned int i=0; i<m_joints.size(); i++) {
     m_joints[i]->loadToPlay(this, i_chipmunkWorld);
@@ -1207,6 +1266,7 @@ void Level::unloadToPlay() {
   for(unsigned int i=0; i<m_joints.size(); i++) {
     m_joints[i]->unloadToPlay();
   }
+  
 }
 
 void Level::addLimits() {
@@ -1273,6 +1333,10 @@ std::vector<Entity *>& Level::EntitiesExterns() {
   return m_entitiesExterns;
 }
 
+std::vector<Entity *>& Level::EntitiesDestroyed() {
+  return m_entitiesDestroyed;
+}
+
 void Level::unloadLevelBody() {
   unloadToPlay();
 
@@ -1306,7 +1370,7 @@ void Level::unloadLevelBody() {
 }
 
 void Level::rebuildCache() {
-  exportBinary(getNameInCache(), m_checkSum);
+  exportBinary(FDT_CACHE, getNameInCache(), m_checkSum);
 }
 
 std::string Level::SpriteForStrawberry() const {
@@ -1325,10 +1389,22 @@ std::string Level::SpriteForStar() const {
   return m_rSpriteForStar;
 }
 
+std::string Level::SpriteForCheckpointDown() const {
+  return m_rSpriteForCheckpointDown;
+}
+
+std::string Level::SpriteForCheckpointUp() const {
+  return m_rSpriteForCheckpointUp;
+}
+
 std::string Level::SoundForPickUpStrawberry() const {
   return m_rSoundForPickUpStrawberry;
 }
 
+std::string Level::SoundForCheckpoint() const {
+  return m_rSoundForCheckpoint;
+}
+
 Sprite* Level::strawberrySprite()
 {
   return m_strawberrySprite;
@@ -1349,12 +1425,24 @@ Sprite* Level::starSprite()
   return m_starSprite;
 }
 
+Sprite* Level::checkpointSpriteDown()
+{
+  return m_checkpointSpriteDown;
+}
+
+Sprite* Level::checkpointSpriteUp()
+{
+  return m_checkpointSpriteUp;
+}
+
 void Level::loadRemplacementSprites()
 {
   m_strawberrySprite = Theme::instance()->getSprite(SPRITE_TYPE_ANIMATION, SpriteForStrawberry());
   m_wreckerSprite    = Theme::instance()->getSprite(SPRITE_TYPE_ANIMATION, SpriteForWecker());
   m_flowerSprite     = Theme::instance()->getSprite(SPRITE_TYPE_ANIMATION, SpriteForFlower());
   m_starSprite       = Theme::instance()->getSprite(SPRITE_TYPE_ANIMATION, SpriteForStar());
+  m_checkpointSpriteDown  = Theme::instance()->getSprite(SPRITE_TYPE_ANIMATION, SpriteForCheckpointDown());
+  m_checkpointSpriteUp    = Theme::instance()->getSprite(SPRITE_TYPE_ANIMATION, SpriteForCheckpointUp());
 }
 
 float Level::averagePhysicBlocksSize() const {
